import matplotlib
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}",r"\renewcommand{\seriesdefault}{\bfdefault}",
r"\usepackage{fixltx2e}"]
params = {'text.usetex' : True,
          'font.size' : 16,
          'font.family' : 'lmodern',
          'text.latex.unicode': True,
          }
matplotlib.rcParams.update(params) 
matplotlib.use("cairo")
import matplotlib.pyplot as plt
import os
import glob
import numpy as np
from Molecule import Molecule
from atomsel import atomsel
import vmdnumpy as vnp
bigdict = dict()
for sys in ['dimers']:
	dirlist = glob.glob("../tests/%s/*55*/" % sys)
	valuedict = dict()
	for d in dirlist:
		print d
		for subd in ['optimized']:#, 'cgenff']:
			if os.path.exists(d+subd):
				dcdlist = glob.glob(d+subd+"/*.dcd")
				for dcd in dcdlist:
					split = dcd[:-4].split("_")
					key=split[2]
					num = int(split[1])
					mol = Molecule()
					mol.load(d+subd+"/sys-"+str(num)+".psf")
					mol.load(dcd)
					#refsel = atomsel("not water")
					#refsel.frame=0
					sel = atomsel("name C2 C5")
					idxs = np.array(sel.get("index"))
					angles = np.empty(mol.numFrames()-1, dtype=np.float)
					for f in range(1,mol.numFrames()):
						R = vnp.timestep(mol, f)[idxs]
						v1 = R[1]-R[0]
						v1 /= np.linalg.norm(v1)
						v2 = R[3]-R[2]
						v2 /= np.linalg.norm(v2)
						angles[f-1] = np.degrees(np.arccos(np.dot(v1,v2)))
					print angles
					#exit()
					if key in valuedict:
						valuedict[key].append(np.mean(angles))
					else:
						valuedict[key] = [np.mean(angles)]
					mol.delete()
	bigdict[sys]=valuedict
print bigdict
#exit()
fig, axes = plt.subplots(1,2,sharey=True)
fig.subplots_adjust(wspace=0, bottom=0.3, top=0.95)
keys = sorted(bigdict['dimers'].keys())
for i, key in enumerate(['dimers']):
	ax = axes[i]
	y = np.empty(len(keys), dtype=np.float)
	yerr = np.empty(len(keys), dtype=np.float)
	for j, k in enumerate(keys):
		print key, k, np.mean(bigdict[key][k]), np.std(bigdict[key][k])
		y[j] = np.mean(bigdict[key][k])
		yerr[j] = np.std(bigdict[key][k])
	ax.bar(np.arange(len(keys)), y, yerr=yerr)
	ax.set_xticks(np.arange(len(keys)))
	ax.set_xticklabels(keys, rotation=45, ha='right', fontsize=10)
axes[0].set_ylabel("Mean RMSD from QM (\AA)")
fig.savefig("anglerecordtest.png")

colors = ['red', 'goldenrod', 'green', 'skyblue']
x = np.linspace(150,180,500)
fig, axes = plt.subplots(1,4, sharey=True, sharex=True, figsize=(12,6))
fig.subplots_adjust(wspace=0,hspace=0, left=0.08, top=0.92, right=0.97)
f = np.array([0, 0.02, 0.1, 0.5])
midfixes = ['noforce', '', 'strongforce', 'reallystrongforce']
prefixes = ['tightbounds', 'tightbounds', 'tightbounds', 'loosebounds']
postfix = ['', 'fixdelta', 'fixdeltaupper2', '']
labels = ['Dihedral Set A', 'Dihedral Set B', 'Dihedral Set C', 'Free Dihedral']
state = ['Dimers']
for q, key in enumerate(['dimers']):
	for i in range(4):
		ax = axes[i]
		k = 's-cgo2'
		data = bigdict[key][k]
		H, edges = np.histogram(data, bins=x)
		H = H.astype(float) / np.sum(H)
		ax.plot(0.5*(edges[1:]+edges[:-1]), np.cumsum(H), label="CGenFF", color='k')
		if q == 0:
			ax.text(0.5, 1.01, labels[i], transform=ax.transAxes, ha='center')
		#ax.text(0.5, 1.01, labels[i], transform=ax.transAxes, ha='center')
		for j in range(4):
			k = prefixes[i] + midfixes[j] + postfix[i]
			data = bigdict[key][k]
			H, edges = np.histogram(data, bins=x)
			H = H.astype(float) / np.sum(H)
			vlabel = "v=%.1f" % f[j]
			if j == 1:
				vlabel = "v=%.2f" % f[j]
			ax.plot(0.5*(edges[1:]+edges[:-1]), np.cumsum(H), label=vlabel, color=colors[j])
			offset = 0
			if i == 3:
				offset = 0.25
			ax.text(0.02, 0.95-offset-j*0.045, "$%.1f \pm %.1f$" % (np.mean(data), np.std(data)), transform=ax.transAxes, color=colors[j])
		ax.set_xlim(160,180)
		ax.set_ylim(0,1)
		ax.grid(True)
		if i >0:
			ax.get_xticklabels()[0].set_visible(False)
		#ax.set_xticks([0, 0.25, 0.5, 0.75, 1])
	#ax.text(1.02,0.5, state[q], transform=ax.transAxes, rotation='vertical', va='center')
#axes[0].get_yticklabels()[0].set_visible(False)
axes[3].legend(fontsize=10)
fig.text(0.5, 0.03, "Solvent 5-5 Angle ($^\circ$)", transform=fig.transFigure, ha='center')
fig.text(0.02, 0.5, "Cumulative Probability", transform=fig.transFigure, rotation='vertical', va='center')
fig.savefig("angledistributionsolvvcompare.png", dpi=300)

fig, axes = plt.subplots(1,4, sharey=True, sharex=True, figsize=(12,6))
fig.subplots_adjust(wspace=0,hspace=0, left=0.08, top=0.92, right=0.97, bottom=0.12)
colors = ['indianred', 'orange', 'limegreen', 'royalblue']
midfixes = ['noforce', '', 'strongforce', 'reallystrongforce']
prefixes = ['tightbounds', 'tightbounds', 'tightbounds', 'loosebounds']
postfix = ['', 'fixdelta', 'fixdeltaupper2', '']
labels = ['Dihedral Set A', 'Dihedral Set B', 'Dihedral Set C', 'Free Dihedral']

for q, key in enumerate(['dimers']):
	for i in range(4):
		ax = axes[i]
		k = 's-cgo2'
		data = bigdict[key][k]
		H, edges = np.histogram(data, bins=x)
		H = H.astype(float) / np.sum(H)
		ax.plot(0.5*(edges[1:]+edges[:-1]), np.cumsum(H), label="CGenFF", color='k')
		if q == 0:
			vlabel = "v=%.1f" % f[i]
			if i == 1:
				vlabel = "v=%.2f" % f[i]
			ax.text(0.5, 1.01, vlabel, transform=ax.transAxes, ha='center')
		for j in range(4):
			k = prefixes[j] + midfixes[i] + postfix[j]
			data = bigdict[key][k]
			H, edges = np.histogram(data, bins=x)
			H = H.astype(float) / np.sum(H)
			ax.plot(0.5*(edges[1:]+edges[:-1]), np.cumsum(H), label=labels[j], color=colors[j])
			offset = 0
			if i == 3:
				offset = 0.25
			ax.text(0.02, -offset + 0.95-j*0.045, "$%.1f \pm %.1f$" % (np.mean(data), np.std(data)), transform=ax.transAxes, color=colors[j])
		ax.set_xlim(160,180)
		ax.set_ylim(0,1)
		ax.grid(True)
		#ax.set_xticks([0, 0.25, 0.5, 0.75, 1])
		if i >0:
			ax.get_xticklabels()[0].set_visible(False)
	#ax.text(1.02,0.5, state[q], transform=ax.transAxes, rotation='vertical', va='center')
axes[3].legend(fontsize=10)

#axes[0].get_yaxis().set_ticklabels([])
fig.text(0.5, 0.03, "Solvent 5-5 Angle ($^\circ$)", transform=fig.transFigure, ha='center')
fig.text(0.02, 0.5, "Cumulative Probability", transform=fig.transFigure, rotation='vertical', va='center')
fig.savefig("angledistributionsolvdihedcompare.png", dpi=300)
exit()
