import matplotlib
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}",r"\renewcommand{\seriesdefault}{\bfdefault}",
r"\usepackage{fixltx2e}"]
params = {'text.usetex' : True,
          'font.size' : 16,
          'font.family' : 'lmodern',
          'text.latex.unicode': True,
          }
matplotlib.rcParams.update(params) 
matplotlib.use("cairo")
import matplotlib.pyplot as plt
import os
import glob
import numpy as np
from Molecule import Molecule
from atomsel import atomsel
bigdict = dict()
for sys in ['monomers', 'dimers']:
	dirlist = glob.glob("../tests/%s/*/" % sys)
	valuedict = dict()
	for d in dirlist:
		print d
		for subd in ['optimized']:#, 'cgenff']:
			if os.path.exists(d+subd):
				dcdlist = glob.glob(d+subd+"/*.dcd")
				for dcd in dcdlist:
					split = dcd[:-4].split("_")
					key=split[2]
					num = int(split[1])
					mol = Molecule()
					mol.load(d+subd+"/sys-"+str(num)+".psf")
					mol.load(dcd)
					refsel = atomsel("not water")
					refsel.frame=0
					sel = atomsel("not water")
					rmsds = np.empty(mol.numFrames()-1, dtype=np.float)
					for f in range(1,mol.numFrames()):
						sel.frame=f
						rmsds[f-1] = sel.rmsdQCP(refsel)
					if key in valuedict:
						valuedict[key].append(np.mean(rmsds))
					else:
						valuedict[key] = [np.mean(rmsds)]
					mol.delete()
	bigdict[sys]=valuedict
print bigdict
#exit()
fig, axes = plt.subplots(1,2,sharey=True)
fig.subplots_adjust(wspace=0, bottom=0.3, top=0.95)
keys = sorted(bigdict['monomers'].keys())
for i, key in enumerate(['monomers', 'dimers']):
	ax = axes[i]
	y = np.empty(len(keys), dtype=np.float)
	yerr = np.empty(len(keys), dtype=np.float)
	for j, k in enumerate(keys):
		print key, k, np.mean(bigdict[key][k]), np.std(bigdict[key][k])
		y[j] = np.mean(bigdict[key][k])
		yerr[j] = np.std(bigdict[key][k])
	ax.bar(np.arange(len(keys)), y, yerr=yerr)
	ax.set_xticks(np.arange(len(keys)))
	ax.set_xticklabels(keys, rotation=45, ha='right', fontsize=10)
axes[0].set_ylabel("Mean RMSD from QM (\AA)")
fig.savefig("rmsdrecordsolvtest.png")
fout = open("rmsdtable.tex", "w")
fout.write('''\\begin{tabular}{ccccc}
\\toprule
 & \multicolumn{4}{c}{Monomers}\\\\
 \cmidrule(lr){2-5} 
Dihedral Set & v=0 & v=0.02 & v=0.1 & v=0.5 \\\\
\midrule\n
''')
colors = ['red', 'goldenrod', 'green', 'skyblue']
x = np.linspace(0,1.4,200)
fig, axes = plt.subplots(2,4, sharey=True, sharex=True, figsize=(12,6))
fig.subplots_adjust(wspace=0,hspace=0, left=0.08, top=0.92, right=0.98)
f = np.array([0, 0.02, 0.1, 0.5])
midfixes = ['noforce', '', 'strongforce', 'reallystrongforce']
prefixes = ['tightbounds', 'tightbounds', 'tightbounds', 'loosebounds']
postfix = ['', 'fixdelta', 'fixdeltaupper2', '']
labels = ['Dihedral Set A', 'Dihedral Set B', 'Dihedral Set C', 'Free Dihedral']
state = ['Monmers', 'Dimers']
for q, key in enumerate(['monomers', 'dimers']):
	for i in range(4):
		fout.write(labels[i])
		ax = axes[q,i]
		k = 's-cgo2'
		data = bigdict[key][k]
		H, edges = np.histogram(data, bins=x)
		H = H.astype(float) / np.sum(H)
		ax.plot(0.5*(edges[1:]+edges[:-1]), np.cumsum(H), label="CGenFF", color='k')
		if q == 0:
			ax.text(0.5, 1.01, labels[i], transform=ax.transAxes, ha='center')
		delayedwrite="\cmidrule(lr){1-2}\nCGenFF & $%.2f\pm%.2f$ & & & \\\\\n" % (np.mean(data), np.std(data))
		#ax.text(0.5, 1.01, labels[i], transform=ax.transAxes, ha='center')
		for j in range(4):
			k = prefixes[i] + midfixes[j] + postfix[i]
			data = bigdict[key][k]
			H, edges = np.histogram(data, bins=x)
			H = H.astype(float) / np.sum(H)
			vlabel = "v=%.1f" % f[j]
			if j == 1:
				vlabel = "v=%.2f" % f[j]
			ax.plot(0.5*(edges[1:]+edges[:-1]), np.cumsum(H), label=vlabel, color=colors[j])
			fout.write(" & $%.2f\pm%.2f$" % (np.mean(data), np.std(data)))
			#ax.text(0.02, 0.95-j*0.045, "$%.1f \pm %.1f$" % (np.mean(data), np.std(data)), transform=ax.transAxes, color=colors[j])
		ax.set_xlim(0, 1.0)
		ax.set_ylim(0,1)
		ax.grid(True)
		ax.set_xticks([0, 0.25, 0.5, 0.75, 1])
		if q == 1 and i >0:
			ax.get_xticklabels()[0].set_visible(False)
		fout.write("\\\\\n")
		if i == 3:
			fout.write(delayedwrite)
	if q == 0:
		fout.write(''' \midrule
 & \multicolumn{4}{c}{Dimers} \\\\
\cmidrule(lr){2-5}\n''')
	ax.text(1.02,0.5, state[q], transform=ax.transAxes, rotation='vertical', va='center')
fout.write("\\bottomrule\n\end{tabular}")
fout.close()
axes[0,0].get_yticklabels()[0].set_visible(False)
axes[0,3].legend(fontsize=10)
fig.text(0.5, 0.03, "Solvated RMSD (\AA)", transform=fig.transFigure, ha='center')
fig.text(0.02, 0.5, "Cumulative Probability", transform=fig.transFigure, rotation='vertical', va='center')
fig.savefig("rmsddistributionsolvvcompare.png", dpi=300)

fig, axes = plt.subplots(2,4, sharey=True, sharex=True, figsize=(12,6))
fig.subplots_adjust(wspace=0,hspace=0, left=0.08, top=0.92, right=0.98, bottom=0.12)
colors = ['indianred', 'orange', 'limegreen', 'royalblue']
midfixes = ['noforce', '', 'strongforce', 'reallystrongforce']
prefixes = ['tightbounds', 'tightbounds', 'tightbounds', 'loosebounds']
postfix = ['', 'fixdelta', 'fixdeltaupper2', '']
labels = ['Dihedral Set A', 'Dihedral Set B', 'Dihedral Set C', 'Free Dihedral']
for q, key in enumerate(['monomers', 'dimers']):
	for i in range(4):
		ax = axes[q,i]
		k = 's-cgo2'
		data = bigdict[key][k]
		H, edges = np.histogram(data, bins=x)
		H = H.astype(float) / np.sum(H)
		ax.plot(0.5*(edges[1:]+edges[:-1]), np.cumsum(H), label="CGenFF", color='k')
		if q == 0:
			vlabel = "v=%.1f" % f[i]
			if i == 1:
				vlabel = "v=%.2f" % f[i]
			ax.text(0.5, 1.01, vlabel, transform=ax.transAxes, ha='center')
		for j in range(4):
			k = prefixes[j] + midfixes[i] + postfix[j]
			data = bigdict[key][k]
			H, edges = np.histogram(data, bins=x)
			H = H.astype(float) / np.sum(H)
			ax.plot(0.5*(edges[1:]+edges[:-1]), np.cumsum(H), label=labels[j], color=colors[j])
			#ax.text(0.02, 0.95-j*0.045, "$%.1f \pm %.1f$" % (np.mean(data), np.std(data)), transform=ax.transAxes, color=colors[j])
		ax.set_xlim(0, 1.0)
		ax.set_ylim(0,1)
		ax.grid(True)
		ax.set_xticks([0, 0.25, 0.5, 0.75, 1])
		if q == 1 and i >0:
			ax.get_xticklabels()[0].set_visible(False)
	ax.text(1.02,0.5, state[q], transform=ax.transAxes, rotation='vertical', va='center')
axes[0,0].get_yticklabels()[0].set_visible(False)
axes[0,3].legend(fontsize=10)
#axes[0].get_yaxis().set_ticklabels([])
fig.text(0.5, 0.03, "Solvated RMSD (\AA)", transform=fig.transFigure, ha='center')
fig.text(0.02, 0.5, "Cumulative Probability", transform=fig.transFigure, rotation='vertical', va='center')
fig.savefig("rmsddistributionsolvdihedcompare.png", dpi=300)
exit()
