from Molecule import *
from atomsel import *
from VMD import evaltcl
import networkx as nx
import numpy as np
import vmdnumpy as vnp
import networkx.algorithms.isomorphism as iso
import operator
import measure
import os
import math
def sortpriority(a, b):
	#Return negative numbers for higher priority.
	for i in range(min(len(a),len(b))):
		if a[i] > b[i]:
			return -1
		elif b[i] > a[i]:
			return 1
	if len(a) > len(b):
		return -1
	elif len(b) > len(a):
		return 1
	else:
		return 0

def setchirality(molid):
	allsel = atomsel("all", molid=molid)
	G = nx.Graph()
	G.add_nodes_from(allsel.get("index"))
	for i, blist in enumerate(allsel.bonds):
		for j in blist:
			if j > i:
				G.add_edge(i,j)
	evaltcl("package require topotools; topo guessatom element mass -molid %d; " % molid)
	atomicnumbers = dict()
	for i, anum in enumerate(allsel.get("atomicnumber")):
		atomicnumbers[i] = anum
	nx.set_node_attributes(G, "atomicnumber", atomicnumbers)
	suspiciouscarbons = atomsel("name \"C.*\" and numbonds == 4", molid=molid)
	carbonidxs = suspiciouscarbons.get("index")
	#print carbonidxs
	structurelist = []
	for i in range(len(suspiciouscarbons)):
		H = G.copy()
		H.remove_node(carbonidxs[i])
		adjacentatoms = allsel.bonds[carbonidxs[i]]
		subgraphs = list(nx.connected_component_subgraphs(H))
		subG = []
		#This convoluted search is needed because of rings. For every adjacent atom, it will be in exactly one subgraph, but I don't know which one.
		for index in adjacentatoms:
			for j, g in enumerate(subgraphs):
				if index in g.nodes():
					H = g.copy()
					#Add the central carbon back in. Makes the isomorphic test work.
					H.add_node(carbonidxs[i], atomicnumber=0)
					H.add_edge(carbonidxs[i], index)
					subG.append(H)
		#print subG
		knownachiral = False
		for j, graph in enumerate(subG):
			for k, graph2 in enumerate(subG):
				if j < k:
					nm = iso.numerical_node_match('atomicnumber', 1)
					knownachiral |= nx.is_isomorphic(graph, graph2, node_match=nm)
		#print carbonidxs[i], suspiciouscarbons.bonds[i], knownachiral
		if knownachiral:
			structurelist.append('')
			continue
		#print "This carbon is chiral. Lets figure out R vs S"
		priorities = dict()
		for j, graph in enumerate(subG):
			#localindex = np.nonzero(np.in1d(adjacentatoms, graph.nodes()))[0][0]
			localindex = j #Because of how these are added in order now above, we don't use the in1d search to get the local index.
			pathlengths = np.array(list(nx.shortest_path_length(graph, source=carbonidxs[i]).values()),dtype=np.int)
			atomnumbers = np.array(list(nx.get_node_attributes(graph,'atomicnumber').values()), dtype=np.int)
			priorities[localindex] = np.bincount(pathlengths,weights=atomnumbers)[1:]
		#print priorities
		#print len(subG)
		priorityorder = sorted(priorities, key=priorities.get, cmp=sortpriority)
		#print priorityorder, adjacentatoms
		#print measure.dihedral(adjacentatoms[priorityorder[0]], adjacentatoms[priorityorder[1]], adjacentatoms[priorityorder[2]], adjacentatoms[priorityorder[3]])[0]
		if measure.dihedral(adjacentatoms[priorityorder[0]], adjacentatoms[priorityorder[1]], adjacentatoms[priorityorder[2]], adjacentatoms[priorityorder[3]])[0] > 0:
			structurelist.append('R')
		else:
			structurelist.append('S')
	suspiciouscarbons.set('chain', structurelist)
	return structurelist

for linkage in ['BO4', '55', '4O5', 'AO4', 'B5', 'B1', 'BB', 'BDO', 'GOG']:
	#These linkages enforce some combinatorial constraints. For instance, BDO is only possible using C-lignin
	reslist1 = ['PHP'] # Shrinks the space considerably, since usually the parts that are different will be far away anyway.
	reslist2 = ['PHP', 'GUAI', 'SYR', 'CAT']
	if linkage == 'GOG':
		reslist1 = ['PCA']
	if linkage == 'BDO':
		reslist2 = ['CAT']
	if linkage in ['4O5', 'B5', '55']:
		reslist2 = ['PHP', 'GUAI', 'CAT']
		if linkage == '55':
			reslist1 = ['PHP', 'GUAI', 'CAT']
		elif linkage == '4O5':
			reslist1 = ['PHP', 'GUAI', 'CAT']
	if linkage == 'BB':
		reslist2 = ['PHP']
	#This is H, G, S, and C-lignin.
	for resname in reslist1:
		for resname2 in reslist2:
			residue = (resname[0] + linkage + resname2[0]).lower()
			mol = Molecule()
			mol.load(residue + '.psf')
			mol.load(residue + '.pdb')
			structurelist = setchirality(int(mol))
			chiralcarbons = atomsel("chain R S and (withinbonds 3 of resid 1) and (withinbonds 3 of resid 2)")
			rslist = chiralcarbons.get("chain")
			idxlist = chiralcarbons.get("index")
			#Now we start changing the chirality.
			mol.save(residue + "".join(rslist).lower() + ".pdb")
			charmmscript = '''ioformat extended
read rtf card name ../parameters/lignin.top
read para card name ../parameters/lignin.prm

read psf card name %s.psf
read coor pdb name %s.pdb
mini sd nstep 5000
write coor pdb name %s.pdb
stop
''' % (residue, residue + "".join(rslist).lower(), residue + "".join(rslist).lower())
			fout = open("%s-chiralminimize.inp" % residue, "w")
			fout.write(charmmscript)
			fout.close()
			os.system("charmm < %s-chiralminimize.inp > %s-chiralminimize-0.log" % (residue, residue))
			R = vnp.timestep(mol, 0)
			numlist = range(2**len(idxlist))
			for i in range(1, 2**len(idxlist)):
				#http://codegolf.stackexchange.com/questions/7343/traverse-all-numbers-with-only-one-bit-flip-per-step
				swapidx = idxlist[int(math.log(abs((i^i//2) - ((i-1)^(i-1)//2)),2))]
				print swapidx
				pickhydrogen = atomsel("name \"H.*\" and withinbonds 1 of index %d" % swapidx)
				hidx = pickhydrogen.get("index")
				R[hidx] -= 2 * (R[hidx] - R[swapidx])
				structurelist = setchirality(int(mol))
				rslist = chiralcarbons.get("chain")
				mol.save(residue + "".join(rslist).lower() + ".pdb")
				charmmscript = '''ioformat extended
read rtf card name ../parameters/lignin.top
read para card name ../parameters/lignin.prm

read psf card name %s.psf
read coor pdb name %s.pdb
mini sd nstep 5000
write coor pdb name %s.pdb
stop
''' % (residue, residue + "".join(rslist).lower(), residue + "".join(rslist).lower())
				fout = open("%s-chiralminimize.inp" % residue, "w")
				fout.write(charmmscript)
				fout.close()
				os.system("charmm < %s-chiralminimize.inp > %s-chiralminimize-%d.log" % (residue, residue, i))
exit()
