from Molecule import Molecule
from atomsel import atomsel
import topology
import glob
import numpy as np
#This script needs to look through all the psfs of the monomers, and see how much this covers, and what I still need to do to cover all the bond, angle, dihedral terms.

knownparameters = dict()
knownparameters['bonds'] = dict()
knownparameters['angles'] = dict()
knownparameters['dihedrals'] = dict()
knownparameters['nonbonded'] = dict()

def readparameters(filename):
	fin = open(filename)
	lines = fin.readlines()
	fin.close()
	readstate = 0
	for l in lines:
		s = l.strip()
		if s.find("!") != -1:
			s = s[:s.find("!")]
		if len(s):
			splt = s.split()
			if splt[0] == "BONDS":
				readstate = 1
				skiplines = 1
			elif splt[0] == "ANGLES":
				readstate = 2
				skiplines = 1
			elif splt[0] == "DIHEDRALS":
				readstate = 3
				skiplines = 1
			elif splt[0] == "IMPROPERS":
				readstate = 0
			elif splt[0] == "NONBONDED":
				readstate = 4
				skiplines = 2
			elif splt[0] == "END":
				readstate = 0
			elif splt[0] == "HBOND":
				readstate = 0
			if readstate != 0:
				if skiplines:
					skiplines -= 1
				else:
					#print splt
					if readstate == 1:
						bondkey = tuple(sorted(splt[:2]))
						bondprm = np.array(splt[2:], dtype=np.float)
						#print bondkey, bondprm
						knownparameters['bonds'][bondkey] = bondprm
					elif readstate == 2:
						q = splt[:3]
						if splt[0] > splt[2]: 
							q.reverse()
						anglekey = tuple(q)
						anglprm = np.array(splt[3:], dtype=np.float)
						knownparameters['angles'][anglekey] = anglprm
					elif readstate == 3:
						q = splt[:4]
						if splt[0] > splt[3]: 
							q.reverse()
						elif splt[0] == splt[3] and splt[1] > splt[2]:
							q.reverse()
						anglekey = tuple(q)
						#print anglekey
						anglprm = tuple(np.array(splt[4:], dtype=np.float))
						if anglekey in knownparameters['dihedrals']:
							knownparameters['dihedrals'][anglekey].add(anglprm)
							#print len(knownparameters['dihedrals'][anglekey])
						else:
							knownparameters['dihedrals'][anglekey] = set([anglprm])
					else:
						knownparameters['nonbonded'][splt[0]] = np.array(splt[1:], dtype=np.float)

readparameters("/projects/LigninParameterization/optimization/chargeoptimization/test/ligins-cgenff.prm")

bondcounts = dict()
anglecounts = dict()
dihedralcounts = dict()
missingbondset = set()
missingangleset = set()
missingdihedralset = set()
filelist = sorted(glob.glob("../chargeoptimization/test/psfs/*psf"))
monolist = []
for f in filelist:
	#print f
	mol = Molecule()
	mol.load(f)
	sel = atomsel("all")
	if len(set(sel.get("residue"))) == 1:
		print f
		monolist.append(f)
		typelist = sel.get("type")
		for b in topology.bonds():
			t1 = typelist[b[0]]
			t2 = typelist[b[1]]
			k = tuple(sorted([t1,t2]))
			if k in bondcounts.keys():
				bondcounts[k] += 1
			else:
				bondcounts[k] = 1
		for a in topology.angles():
			t1 = typelist[a[0]]
			t2 = typelist[a[1]]
			t3 = typelist[a[2]]
			if t1 > t3:
				k = tuple([t3,t2,t1])
			else:
				k = tuple([t1, t2, t3])
			if k in anglecounts.keys():
				anglecounts[k] += 1
			else:
				anglecounts[k] = 1
		for d in topology.dihedrals():
			t1 = typelist[d[0]]
			t2 = typelist[d[1]]
			t3 = typelist[d[2]]
			t4 = typelist[d[3]]
			if t1 > t4 or (t1 == t4 and t2 > t3):
				k = tuple([t4,t3,t2,t1])
			else:
				k = tuple([t1,t2,t3,t4])
			if k in dihedralcounts.keys():
				dihedralcounts[k] += 1
			else:
				dihedralcounts[k] = 1
	else:
		typelist = sel.get("type")
		for b in topology.bonds():
			t1 = typelist[b[0]]
			t2 = typelist[b[1]]
			k = tuple(sorted([t1,t2]))
			missingbondset.add(k)
		for a in topology.angles():
			t1 = typelist[a[0]]
			t2 = typelist[a[1]]
			t3 = typelist[a[2]]
			if t1 > t3:
				k = tuple([t3,t2,t1])
			else:
				k = tuple([t1, t2, t3])
			missingangleset.add(k)
		for d in topology.dihedrals():
			t1 = typelist[d[0]]
			t2 = typelist[d[1]]
			t3 = typelist[d[2]]
			t4 = typelist[d[3]]
			if t1 > t4 or (t1 == t4 and t2 > t3):
				k = tuple([t4,t3,t2,t1])
			else:
				k = tuple([t1,t2,t3,t4])
			missingdihedralset.add(k)
for k in bondcounts.keys():
	if k in missingbondset:
		missingbondset.remove(k)
for k in anglecounts.keys():
	if k in missingangleset:
		missingangleset.remove(k)
for k in dihedralcounts.keys():
	if k in missingdihedralset:
		missingdihedralset.remove(k)
print monolist
print missingbondset
print missingangleset
print missingdihedralset
exit()