#Alright, I need to make a box of molecules, and a single molecule, both with CGenFF and with the new force field. Simulate both, and see how the energies compare.

#I have experimental comparison data for phenol, 2-methoxy phenol (guaiacol), and 2,6-dimethoxy phenol (syringol), 1,2 dihydroxybenzene (catechol)
#10.1021/je025641j
#10.1016/0040-6031(95)90670-3
#10.1016/j.tca.2008.02.016
from Molecule import Molecule
import os
import subprocess
from VMD import evaltcl
def mymkdir(s):
	if not os.path.exists(s):
		os.makedirs(s)
def mysymlink(source, dest):
	#print source, dest
	if not os.path.exists(dest):
		os.symlink(source, dest)
mymkdir("gas")
mymkdir("liquid")

script = '''

#############################################################
## JOB DESCRIPTION                                         ##
#############################################################

# equilibrate things.

#############################################################
## ADJUSTABLE PARAMETERS                                   ##
#############################################################

set system %s
set shortname %s
structure          $system.psf
coordinates        $shortname.pdb

set runnumber 0

set outputname		$system-run_$runnumber
set inputname		$system-run_[expr {$runnumber -1}]
set temperature    298
set runSteps       1000000

# restart file :input in either inputname or cellBasis
#  0 is new run, 1 is restart run, 2 is restart run from old files.
set restartfoo 1
if { $runnumber == 0 } {
	set restartfoo -1
} else {
	set restartfoo 1
}

proc get_first_ts { xscfile } {

	 set fd [open $xscfile r]
	 gets $fd
	 gets $fd
	 gets $fd line
	 set ts [lindex $line 0]
	 close $fd
	 return $ts

}

if { $restartfoo == -1} {
	temperature         $temperature
	set firsttime 0
	firsttimestep 0
	
} elseif { $restartfoo == 0 } {

	bincoordinates     ./$inputname.coor
	binvelocities      ./$inputname.vel
	extendedSystem     ./$inputname.xsc
	 
	set firsttime [get_first_ts ./$inputname.xsc]
	firsttimestep 0
 
} elseif { $restartfoo == 1 } {
	
	bincoordinates     ./$inputname.restart.coor
	binvelocities      ./$inputname.restart.vel
	extendedSystem     ./$inputname.restart.xsc
	 
	set firsttime [get_first_ts ./$inputname.restart.xsc]
	firsttimestep $firsttime

} elseif { $restartfoo == 2  } {
	
	bincoordinates     ./$inputname.restart.coor.old
	binvelocities      ./$inputname.restart.vel.old
	extendedSystem     ./$inputname.restart.xsc.old
	 
	set firsttime [ get_first_ts ./$inputname.restart.xsc.old ]
	firsttimestep $firsttime

}


#############################################################
## SIMULATION PARAMETERS                                   ##
#############################################################

%s

# NOTE: Do not set the initial velocity temperature if you 
# have also specified a .vel restart file!
# temperature         $temperature
 

# Periodic Boundary conditions
# NOTE: Do not set the periodic cell basis if you have also 
# specified an .xsc restart file!



# Force-Field Parameters
exclude             scaled1-4
1-4scaling          1.0
cutoff              12.
switching           on
switchdist          10.
pairlistdist        14
margin               5


# Integrator Parameters
rigidBonds all
timestep            2.0
nonbondedFreq       1
fullElectFrequency  2  
stepspercycle       20


%s



# Constant Temperature Control
langevin            on    ;# do langevin dynamics
langevinDamping     1     ;# damping coefficient (gamma) of 5/ps
langevinTemp        $temperature
langevinHydrogen    no    ;# don't couple langevin bath to hydrogens



# Output
outputName          $outputname
restartname         $outputname.restart
dcdfile             $outputname.dcd
#veldcdfile          $outputname.veldcd
xstFile             $outputname.xst

restartfreq         10000
dcdfreq             10000
xstFreq             10000
outputEnergies      1000


#############################################################
## EXTRA PARAMETERS                                        ##
#############################################################

proc get_volume { xscfile } {

	 set fd [open $xscfile r]
	 gets $fd
	 gets $fd
	 gets $fd line
	 set ts [lindex $line 1]
	 close $fd
	 return $ts

}

#############################################################
## EXECUTION SCRIPT                                        ##
#############################################################

# Minimization
if { $restartfoo <  0 } {
	minimize 1000
	reinitvels          $temperature
	run $runSteps
} else {
	run $runSteps  ;
}

'''

pressurecontrol = '''
#Constant Pressure Control
LangevinPiston on
LangevinPistonTarget 1.01325
LangevinPistonPeriod 200
LangevinPistonDecay 100
LangevinPistonTemp $temperature


if { $restartfoo < 0 } { 
	cellBasisVector1 60 0 0
	cellBasisVector2 0 60 0
	cellBasisVector3 0 0 60
	cellOrigin 0 0 0
}

wrapWater           on
wrapAll             on

PME on
PMEGridSpacing 1.2
PMEInterpOrder 6

'''

myparameters = '''
# Input
paraTypeCharmm      on
parameters          ../lignin.prm
'''

cgenffparameters = '''
# Input
paraTypeCharmm      on
parameters          ../par_all36_cgenff.prm
parameters          ../par_$shortname.prm
'''

for sysname in ['syrlch', 'phplch', 'guailch', 'catlch']:
	cgenffpsf = "../../typing/%s-cgenff.psf" % sysname
	newpsf = "../../optimization/chargegroups/psfs/%s.psf" % sysname
	cgenffpdb = "../../typing/%s.pdb" % sysname
	fin = open("../typing/%s.str" % sysname)
	fout = open("par_%s.prm" % sysname, "w")
	lines = fin.readlines()
	fin.close()
	write = False
	for l in lines:
		if l.strip() == "* Parameters generated by analogy by":
			write = True
		if write:
			fout.write(l)
		if l.strip() == "END":
			write = False
	fout.close()
	os.chdir("gas")
	mysymlink(cgenffpsf, 'cgenff' + sysname + ".psf")
	mysymlink(cgenffpdb, sysname + ".pdb")
	mysymlink(newpsf, sysname + ".psf")
	fout = open("eq-%s.namd" % sysname, "w")
	fout.write(script % (sysname, sysname, myparameters, ""))
	fout.close()
	fout = open("eq-cgenff%s.namd" % sysname, "w")
	fout.write(script % ("cgenff" + sysname, sysname, cgenffparameters, ""))
	fout.close()
	os.chdir("../liquid")
	#Phenol has a molar mass of 94 g/mol, density of 1.07 g/cm^3 -> ~1 mol/cm^3. 100 molecules means you want the cube root of 100 * 10^24 / 10^23 ~ 10.
	subprocess.call("gmx insert-molecules -ci %s -o %s -box 6 6 6 -nmol 500" % (cgenffpdb, sysname+".pdb"), shell=True)
	#print "package require topotools; package require pbctools; set mol [mol load psf %s pdb %s]; pbc set [list 1 1 1 90 90 90] ; set newmol [::TopoTools::replicatemol $mol 10 10 1]; animate write psf cgenff%s.psf" % (cgenffpsf, cgenffpdb, sysname)
	evaltcl("package require topotools; package require pbctools; set mol [mol load psf %s pdb %s]; pbc set [list 1 1 1 90 90 90] ; set newmol [::TopoTools::replicatemol $mol 10 10 5]; animate write psf cgenff%s.psf" % (cgenffpsf, cgenffpdb, sysname))
	evaltcl("package require topotools; package require pbctools; set mol [mol load psf %s pdb %s]; pbc set [list 1 1 1 90 90 90] ; set newmol [::TopoTools::replicatemol $mol 10 10 5]; animate write psf %s.psf" % (newpsf, cgenffpdb, sysname))
	fout = open("eq-%s.namd" % sysname, "w")
	fout.write(script % (sysname, sysname, myparameters, pressurecontrol))
	fout.close()
	fout = open("eq-cgenff%s.namd" % sysname, "w")
	fout.write(script % ("cgenff" + sysname, sysname, cgenffparameters, pressurecontrol))
	fout.close()
	os.chdir("..")
exit()