from Molecule import *
from atomsel import *
from VMD import evaltcl
import networkx as nx
import numpy as np
import vmdnumpy as vnp
import networkx.algorithms.isomorphism as iso
import operator
import measure
import os
def sortpriority(a, b):
	#Return negative numbers for higher priority.
	for i in range(min(len(a),len(b))):
		if a[i] > b[i]:
			return -1
		elif b[i] > a[i]:
			return 1
	if len(a) > len(b):
		return -1
	elif len(b) > len(a):
		return 1
	else:
		return 0

def setchirality(molid):
	allsel = atomsel("all", molid=molid)
	G = nx.Graph()
	G.add_nodes_from(allsel.get("index"))
	for i, blist in enumerate(allsel.bonds):
		for j in blist:
			if j > i:
				G.add_edge(i,j)
	evaltcl("package require topotools; topo guessatom element mass -molid %d; " % molid)
	atomicnumbers = dict()
	for i, anum in enumerate(allsel.get("atomicnumber")):
		atomicnumbers[i] = anum
	nx.set_node_attributes(G, "atomicnumber", atomicnumbers)
	suspiciouscarbons = atomsel("name \"C.*\" and numbonds == 4", molid=molid)
	carbonidxs = suspiciouscarbons.get("index")
	#print carbonidxs
	structurelist = []
	for i in range(len(suspiciouscarbons)):
		H = G.copy()
		H.remove_node(carbonidxs[i])
		adjacentatoms = allsel.bonds[carbonidxs[i]]
		subG = list(nx.connected_component_subgraphs(H))
		for g in subG:
			#Add the central carbon back in. Makes the isomorphic test work.
			localindex = np.nonzero(np.in1d(adjacentatoms, g.nodes()))[0][0]
			g.add_node(carbonidxs[i], atomicnumber=0)
			g.add_edge(carbonidxs[i], adjacentatoms[localindex])
		#print subG
		knownachiral = False
		for j, graph in enumerate(subG):
			for k, graph2 in enumerate(subG):
				if j < k:
					nm = iso.numerical_node_match('atomicnumber', 1)
					knownachiral |= nx.is_isomorphic(graph, graph2, node_match=nm)
		#print carbonidxs[i], suspiciouscarbons.bonds[i], knownachiral
		if knownachiral:
			structurelist.append('')
			continue
		#print "This carbon is chiral. Lets figure out R vs S"
		priorities = dict()
		for j, graph in enumerate(subG):
			localindex = np.nonzero(np.in1d(adjacentatoms, graph.nodes()))[0][0]
			pathlengths = np.array(list(nx.shortest_path_length(graph, source=carbonidxs[i]).values()),dtype=np.int)
			atomnumbers = np.array(list(nx.get_node_attributes(graph,'atomicnumber').values()), dtype=np.int)
			priorities[localindex] = np.bincount(pathlengths,weights=atomnumbers)[1:]
		#print priorities
		priorityorder = sorted(priorities, key=priorities.get, cmp=sortpriority)
		if measure.dihedral(adjacentatoms[priorityorder[0]], adjacentatoms[priorityorder[1]], adjacentatoms[priorityorder[2]], adjacentatoms[priorityorder[3]])[0] > 0:
			structurelist.append('R')
		else:
			structurelist.append('S')
	suspiciouscarbons.set('chain', structurelist)
	return structurelist

for residue in ['cat', 'php', 'syr', 'guai']:
	mol = Molecule()
	mol.load(residue + '.psf')
	mol.load(residue + '.pdb')
	structurelist = setchirality(int(mol))
	#Now we start chaning the chirality.
	mol.save(residue + "".join(structurelist).lower() + ".pdb")
	charmmscript = '''ioformat extended
read rtf card name ../parameters/lignin.top
read para card name ../parameters/lignin.prm

read psf card name %s.psf
read coor pdb name %s.pdb
mini sd nstep 5000
write coor pdb name %s.pdb
stop
''' % (residue, residue + "".join(structurelist).lower(), residue + "".join(structurelist).lower())
	fout = open("%s-chiralminimize.inp" % residue, "w")
	fout.write(charmmscript)
	fout.close()
	os.system("charmm < %s-chiralminimize.inp > %s-chiralminimize.log" % (residue, residue))
	chiralcarbons = atomsel("chain R S")
	idxlist = chiralcarbons.get("index")
	R = vnp.timestep(mol, 0)
	while len(idxlist):
		for swapidx in idxlist:
			pickhydrogen = atomsel("name \"H.*\" and withinbonds 1 of index %d" % swapidx)
			hidx = pickhydrogen.get("index")
			R[hidx] -= 2 * (R[hidx] - R[swapidx])
			structurelist = setchirality(int(mol))
			mol.save(residue + "".join(structurelist).lower() + ".pdb")
			charmmscript = '''ioformat extended
read rtf card name ../parameters/lignin.top
read para card name ../parameters/lignin.prm

read psf card name %s.psf
read coor pdb name %s.pdb
mini sd nstep 5000
write coor pdb name %s.pdb
stop
''' % (residue, residue + "".join(structurelist).lower(), residue + "".join(structurelist).lower())
			fout = open("%s-chiralminimize.inp" % residue, "w")
			fout.write(charmmscript)
			fout.close()
			os.system("charmm < %s-chiralminimize.inp > %s-chiralminimize.log" % (residue, residue))
		idxlist = idxlist[:-1]
exit()
