from Molecule import Molecule
import molecule
from atomsel import atomsel
import topology
import glob
import numpy as np
import vmdnumpy as vnp
import os
import measure
from VMD import evaltcl

optfileheader = '''-#- CacheSize=163840
-M- 16GB
-#- MaxDisk=128GB
-P- 16

# opt=modredundant MP2/6-31g(d) Geom=PrintInputOrient

%s Scan at MP2/6-31G*

%d 1
'''
def mymkdir(s):
	if not os.path.exists(s):
		os.makedirs(s)
#This script needs to look through all the psfs of the monomers, and see how much this covers, and what I still need to do to cover all the bond, angle, dihedral terms.

knownparameters = dict()
knownparameters['bonds'] = dict()
knownparameters['angles'] = dict()
knownparameters['dihedrals'] = dict()
knownparameters['nonbonded'] = dict()

def readparameters(filename):
	fin = open(filename)
	lines = fin.readlines()
	fin.close()
	readstate = 0
	for l in lines:
		s = l.strip()
		if s.find("!") != -1:
			s = s[:s.find("!")]
		if len(s):
			splt = s.split()
			if splt[0] == "BONDS":
				readstate = 1
				skiplines = 1
			elif splt[0] == "ANGLES":
				readstate = 2
				skiplines = 1
			elif splt[0] == "DIHEDRALS":
				readstate = 3
				skiplines = 1
			elif splt[0] == "IMPROPERS" or splt[0] == "IMPROPER":
				readstate = 0
			elif splt[0] == "NONBONDED":
				readstate = 4
				skiplines = 2
			elif splt[0] == "END":
				readstate = 0
			elif splt[0] == "HBOND":
				readstate = 0
			if readstate != 0:
				if skiplines:
					skiplines -= 1
				else:
					#print splt
					if readstate == 1:
						bondkey = tuple(sorted(splt[:2]))
						bondprm = np.array(splt[2:], dtype=np.float)
						#print bondkey, bondprm
						knownparameters['bonds'][bondkey] = bondprm
					elif readstate == 2:
						q = splt[:3]
						if splt[0] > splt[2]: 
							q.reverse()
						anglekey = tuple(q)
						anglprm = np.array(splt[3:], dtype=np.float)
						knownparameters['angles'][anglekey] = anglprm
					elif readstate == 3:
						q = splt[:4]
						print q
						if splt[0] > splt[3]: 
							q.reverse()
						elif splt[0] == splt[3] and splt[1] > splt[2]:
							q.reverse()
						anglekey = tuple(q)
						#print anglekey
						anglprm = tuple(np.array(splt[4:], dtype=np.float))
						if anglekey in knownparameters['dihedrals']:
							knownparameters['dihedrals'][anglekey].add(anglprm)
							#print len(knownparameters['dihedrals'][anglekey])
						else:
							knownparameters['dihedrals'][anglekey] = set([anglprm])
					else:
						knownparameters['nonbonded'][splt[0]] = np.array(splt[1:], dtype=np.float)
#readparameters("/projects/LigninParameterization/typing/lignins-cgenff.prm")
readparameters("/projects/LigninParameterization/typing/lignins-cgo2.prm")
readparameters("/home/josh/toppar/par_all36_carb.prm")

bondcounts = dict()
anglecounts = dict()
dihedralcounts = dict()
missingbondset = set()
missingangleset = set()
missingdihedralset = set()
filelist = sorted(glob.glob("../chargegroups/psfs/*psf"))
print filelist
#move the new ones to the end. Otherwise we have problems.
newfilelist = []
oldfilelist = []
newstrings = ['bet', 'dbal', 'dbac', 'dbap', 'goa', 'lcac', 'lcal', 'lcap', 'lch', 'lihe', 'pgys', 'aoa', 'aog']
for f in filelist:
	found = False
	for s in newstrings:
		if f.find(s) != -1:
			found = True
	if found:
		newfilelist.append(f)
	else:
		oldfilelist.append(f)
filelist = oldfilelist + newfilelist
print filelist
#exit()
monolist = []
for f in filelist:
	print f
	pdbnames = glob.glob("../../*/%s*-optimized.pdb" % f.split('/')[-1][:-4])
	if len(pdbnames) == 0:
		continue
	if f.find("ferut") != -1 or f.find("diferu") != -1 or f.find("feruarab") != -1:
		continue 
	#exit()
	mol = Molecule()
	mol.load(f)
	for pdb in pdbnames:
		mol.load(pdb)
	if mol.numFrames() == 0:
		print "Help!"
		exit()
	#exit()
	evaltcl("package require topotools; topo guessatom element mass;")
	sel = atomsel("all")
	if len(set(sel.get("residue"))) == 1:
		print f
		monolist.append(f)
		typelist = sel.get("type")
		for b in topology.bonds():
			t1 = typelist[b[0]]
			t2 = typelist[b[1]]
			k = tuple(sorted([t1,t2]))
			if k in bondcounts.keys():
				bondcounts[k] += 1
			else:
				bondcounts[k] = 1
		for a in topology.angles():
			t1 = typelist[a[0]]
			t2 = typelist[a[1]]
			t3 = typelist[a[2]]
			if t1 > t3:
				k = tuple([t3,t2,t1])
			else:
				k = tuple([t1, t2, t3])
			if k in anglecounts.keys():
				anglecounts[k] += 1
			else:
				anglecounts[k] = 1
		for d in topology.dihedrals():
			t1 = typelist[d[0]]
			t2 = typelist[d[1]]
			t3 = typelist[d[2]]
			t4 = typelist[d[3]]
			if t1 > t4 or (t1 == t4 and t2 > t3):
				k = tuple([t4,t3,t2,t1])
			else:
				k = tuple([t1,t2,t3,t4])
			if k in dihedralcounts.keys():
				dihedralcounts[k] += 1
			else:
				dihedralcounts[k] = 1
	else:
		typelist = sel.get("type")
		for b in topology.bonds():
			t1 = typelist[b[0]]
			t2 = typelist[b[1]]
			k = tuple(sorted([t1,t2]))
			missingbondset.add(k)
		for a in topology.angles():
			t1 = typelist[a[0]]
			t2 = typelist[a[1]]
			t3 = typelist[a[2]]
			if t1 > t3:
				k = tuple([t3,t2,t1])
			else:
				k = tuple([t1, t2, t3])
			missingangleset.add(k)
		for d in topology.dihedrals():
			t1 = typelist[d[0]]
			t2 = typelist[d[1]]
			t3 = typelist[d[2]]
			t4 = typelist[d[3]]
			if t1 > t4 or (t1 == t4 and t2 > t3):
				k = tuple([t4,t3,t2,t1])
			else:
				k = tuple([t1,t2,t3,t4])
			missingdihedralset.add(k)
for k in bondcounts.keys():
	if k in missingbondset:
		missingbondset.remove(k)
for k in anglecounts.keys():
	if k in missingangleset:
		missingangleset.remove(k)
for k in dihedralcounts.keys():
	if k in missingdihedralset:
		missingdihedralset.remove(k)
print monolist
print len(missingbondset)
print len(missingangleset)
print len(missingdihedralset)
#exit()
aromatictypelist = ['CG2R61', 'CG2DC1', 'CG2O3', 'CG2R67', 'CG2RC0', 'CGO']
for m, mid in enumerate(molecule.listall()):
	sel = atomsel("all", molid=mid)
	
	if len(set(sel.get("residue"))) == 2:
		for f in [0, molecule.numframes(mid)-1]:
			R = vnp.timestep(mid, f)
			typelist = sel.get("type")
			names = sel.get("name")
			counter = 0
			for b in topology.bonds(molid=mid):
				t1 = typelist[b[0]]
				t2 = typelist[b[1]]
				k = tuple(sorted([t1,t2]))
				if k in missingbondset:
					for j, scantext in enumerate(["pos", "neg"]):
						titletext = "B %d %d %s Bond" % (b[0]+1,b[1]+1, scantext)
						buildname = str(mid) + "bond" + str(counter) + scantext + str(f)
						if not os.path.isdir("missingterms/" + buildname):
							print "%s doesn't exist!" % buildname
							print molecule.get_filenames(mid), k
							#exit()
							mymkdir("missingterms/" + buildname)
							fout = open("missingterms/" + buildname + "/" + buildname + ".gau", "w")
							fout.write(optfileheader % (titletext, int(np.round(np.sum(sel.get("charge"))))))
							for i in range(len(sel)):
								fout.write("%s%d %f %f %f\n" % (names[i][0], i+1, R[i][0], R[i][1], R[i][2]))
							fout.write("\nB %d %d S 2 %f\n\n" % (b[0]+1,b[1]+1,0.05 - j * 0.1))
							fout.close()
							sel.write("psf", "missingterms/%s/system.psf" % buildname)
						else:
							print "%s exists!" % buildname
							print molecule.get_filenames(mid)
							sel.write("psf", "missingterms/%s/system.psf" % buildname)
							fout = open("missingterms/%s/sysname.txt" % buildname, "w")
							fout.write(str(molecule.get_filenames(mid)))
							fout.write("\n")
							fout.close()
					if f == molecule.numframes(mid)-1:
						missingbondset.remove(k)
				counter += 1
			counter = 0
			for a in topology.angles(molid=mid):
				t1 = typelist[a[0]]
				t2 = typelist[a[1]]
				t3 = typelist[a[2]]
				if t1 > t3:
					k = tuple([t3,t2,t1])
				else:
					k = tuple([t1, t2, t3])
				if k in missingangleset:
					for j, scantext in enumerate(["pos", "neg"]):
						titletext = "A %d %d %d %s Angle" % (a[0]+1,a[1]+1, a[2]+1, scantext)
						buildname = str(mid) + "angle" + str(counter) + scantext + str(f)
						if not os.path.isdir("missingterms/" + buildname):
							print "%s doesn't exist!" % buildname, k
							print molecule.get_filenames(mid)
							#exit()
							mymkdir("missingterms/" + buildname)
							fout = open("missingterms/" + buildname + "/" + buildname + ".gau", "w")
							fout.write(optfileheader % (titletext, int(np.round(np.sum(sel.get("charge"))))))
							for i in range(len(sel)):
								fout.write("%s%d %f %f %f\n" % (names[i][0], i+1, R[i][0], R[i][1], R[i][2]))
							fout.write("\nA %d %d %d S 2 %f\n\n" % (a[0]+1,a[1]+1,a[2]+1,5 - 10*j))
							fout.close()
							sel.write("psf", "missingterms/%s/system.psf" % buildname)
						else:
							print "%s exists!" % buildname
							print molecule.get_filenames(mid)
							sel.write("psf", "missingterms/%s/system.psf" % buildname)
							fout = open("missingterms/%s/sysname.txt" % buildname, "w")
							fout.write(str(molecule.get_filenames(mid)))
							fout.write("\n")
							fout.close()
					if f == molecule.numframes(mid)-1:
						missingangleset.remove(k)
				counter += 1
			counter = 0
			samedihed = set()
			for d in topology.dihedrals(molid=mid):
				t1 = typelist[d[0]]
				t2 = typelist[d[1]]
				t3 = typelist[d[2]]
				t4 = typelist[d[3]]
				if t1 > t4 or (t1 == t4 and t2 > t3):
					k = tuple([t4,t3,t2,t1])
				else:
					k = tuple([t1,t2,t3,t4])
				if k in missingdihedralset:
					if not tuple(sorted([d[1],d[2]])) in samedihed:
						for j, scantext in enumerate(["pos", "neg"]):
							titletext = "D %d %d %d %d %s Dihedral" % (d[0]+1,d[1]+1,d[2]+1,d[3]+1, scantext)
							buildname = str(mid) + "dihedral" + str(counter) + scantext + str(f)
							if not os.path.isdir("missingterms/" + buildname):
								print "%s doesn't exist!" % buildname, k
								print molecule.get_filenames(mid)
								#exit()
								mymkdir("missingterms/" + buildname)
								fout = open("missingterms/" + buildname + "/" + buildname + ".gau", "w")
								fout.write(optfileheader % (titletext, int(np.round(np.sum(sel.get("charge"))))))
								for i in range(len(sel)):
									fout.write("%s%d %f %f %f\n" % (names[i][0], i+1, R[i][0], R[i][1], R[i][2]))
								if t2 in aromatictypelist and t3 in aromatictypelist and not ((t2 == t3) and t2 in ['CG2R67']):
									fout.write("\nD %d %d %d %d S 6 %f\n\n" % (d[0]+1,d[1]+1,d[2]+1,d[3]+1,5 - j * 10))
								else:
									fout.write("\nD %d %d %d %d S 12 %f\n\n" % (d[0]+1,d[1]+1,d[2]+1,d[3]+1,15 - j * 30))
								fout.close()
								sel.write("psf", "missingterms/%s/system.psf" % buildname)
							else:
								print "%s exists!" % buildname
								print molecule.get_filenames(mid)
								sel.write("psf", "missingterms/%s/system.psf" % buildname)
								fout = open("missingterms/%s/sysname.txt" % buildname, "w")
								fout.write(str(molecule.get_filenames(mid)))
								fout.write("\n")
								fout.close()
						samedihed.add(tuple(sorted([d[1],d[2]])))
					if f == molecule.numframes(mid)-1:
						missingdihedralset.remove(k)
				counter += 1
		print len(missingbondset)
		print len(missingangleset)
		print len(missingdihedralset)
		#exit()
print missingbondset
print missingangleset
print missingdihedralset
exit()