import numpy as np
import cPickle as pickle
import time
import scipy.optimize
start = time.clock()
knownparameters, qmE, mmE, typesmap, geometries = pickle.load( open( "everything.pkl", "rb" ) )
end = time.clock()
print "Spent %.2f seconds loading precalculated entries" % (end - start)

bondp = np.zeros(2*len(knownparameters['bonds']), dtype=np.float)
for i, k in enumerate(sorted(knownparameters['bonds'].keys())):
	bondp[2*i+0] = knownparameters['bonds'][k][0]
	bondp[2*i+1] = knownparameters['bonds'][k][1]
anglep = np.zeros(4 * len(knownparameters['angles']), dtype=np.float)
for i, k in enumerate(sorted(knownparameters['angles'].keys())):
	anglep[4*i+0] = knownparameters['angles'][k][0]
	anglep[4*i+1] = knownparameters['angles'][k][1]
	if len(knownparameters['angles'][k]) > 2:
		anglep[4*i+2] = knownparameters['angles'][k][2]
		anglep[4*i+3] = knownparameters['angles'][k][3]
dihedp = np.zeros(5*len(knownparameters['dihedrals']), dtype=np.float)
for i, k in enumerate(sorted(knownparameters['dihedrals'].keys())):
	setels = list(knownparameters['dihedrals'][k])
	for j in range(len(setels)):
		l = int(setels[j][1])
		if l == 6:
			l = 5
		sign = 1
		if setels[j][2] > 0:
			sign = -1
		dihedp[5*i+l-1] = sign * setels[j][0]

def objectivefunction(x, qmE, mmE, geometries, typesmap):
	objstart = time.clock()
	f = 0
	gradf = np.zeros(len(x), dtype=np.float)
	#print x.shape
	boffset = len(qmE.keys())
	aoffset = boffset + 2*len(knownparameters['bonds'])
	doffset = aoffset + 4 * len(knownparameters['angles'])
	for mol in qmE.keys():
		weights = 1e-4 * np.exp(-qmE[mol]/4)
		targetE = qmE[mol] - mmE[mol]
		bondedE = np.zeros(len(qmE[mol]), dtype=np.float)
		lgradf = np.zeros((len(x), len(qmE[mol])), dtype=np.float)
		#print lgradf.shape
		for t, vals in zip(typesmap[mol]['b'],geometries[mol]['b']):
			#print vals, x[boffset + 2*t + 1]
			bondedE += x[boffset + 2*t] * np.square(vals - x[boffset + 2*t + 1])
			lgradf[boffset + 2*t+0] += np.square(vals - x[boffset + 2*t + 1])
			lgradf[boffset + 2*t+1] -= 2 * x[boffset + 2*t+0] * (vals - x[boffset + 2*t + 1])
		#print bondedE
		#print targetE
		# if mol == 1:
		# 	exit()
		#print lgradf.shape
		for t, valsA, valsUB in zip(typesmap[mol]['a'], geometries[mol]['a'], geometries[mol]['ub']):
			bondedE += x[aoffset + 4*t] * np.square(np.radians(valsA - x[aoffset + 4*t + 1]))
			lgradf[aoffset + 4*t] += np.square(np.radians(valsA - x[aoffset + 4*t + 1]))
			lgradf[aoffset + 4*t+1] -= 2 * x[aoffset + 4*t] * np.radians(valsA - x[aoffset + 4*t + 1]) * np.pi / 180 #The extra term comes from squaring "radians"
			bondedE += x[aoffset + 4*t+2] * np.square(valsUB - x[aoffset + 4*t + 3])
			lgradf[aoffset + 4*t+2] += np.square(valsUB - x[aoffset + 4*t + 3])
			lgradf[aoffset + 4*t+3] -= 2 * x[aoffset + 4*t+2] * (valsUB - x[aoffset + 4*t + 3])
		#print lgradf.shape
		for t, vals in zip(typesmap[mol]['d'], geometries[mol]['d']):
			rads = np.radians(vals)
			for i, n in enumerate([1,2,3,4,6]):
				tmp = (1+np.cos(n*rads))
				bondedE += x[doffset + 5*t + i] * tmp
				lgradf[doffset + 5*t + i] += tmp
		#print lgradf.shape
		lgradf[mol] = np.ones(len(qmE[mol]))
		residuals = targetE - bondedE - x[mol]
		f += np.sum(weights * np.square(residuals))
		gradf -= np.sum(2 * (weights * residuals) * lgradf, axis=1)
	#f += 1e-2 * np.sum(np.square(x[doffset:]))
	#gradf[doffset:] += 1e-2 * 2 * x[doffset]
	objend = time.clock()
	print "Took %.2f seconds for objective function evaluation" % (objend - objstart)
	print f, gradf
	np.save("gradfexample.npy", gradf)
	return (f, gradf)
def func(x, qmE, mmE, geometries, typesmap):
	return objectivefunction(x,qmE,mmE,geometries,typesmap)[0]
def funcgrad(x, qmE, mmE, geometries, typesmap):
	return objectivefunction(x,qmE,mmE,geometries,typesmap)[1]
parameters = np.hstack([np.zeros(len(qmE.keys())), bondp, anglep, dihedp])
start = time.clock()
objectivefunction(parameters, qmE, mmE, geometries, typesmap)
end = time.clock()
exit()
# print "Spent %.2f seconds on a function evaluation" % (end - start)
# #print scipy.optimize.check_grad(func, funcgrad, parameters, qmE, mmE, geometries, typesmap)
# def checklocalgrad(i, x, qmE, mmE, geometries, typesmap):
# 	offset = np.zeros(len(x), dtype=np.float)
# 	offset[i] = 1e-4
# 	a, ga = objectivefunction(parameters-offset, qmE, mmE, geometries, typesmap)
# 	b, gb = objectivefunction(parameters+offset, qmE, mmE, geometries, typesmap)
# 	f, g = objectivefunction(parameters, qmE, mmE, geometries, typesmap)
# 	approxg = (b - a) / 2e-4
# 	print i, g[i], approxg, g[i] - approxg
# checklocalgrad(0, parameters, qmE, mmE, geometries, typesmap)
# checklocalgrad(len(qmE.keys()), parameters, qmE, mmE, geometries, typesmap)
# checklocalgrad(len(qmE.keys())+  2*len(knownparameters['bonds']), parameters, qmE, mmE, geometries, typesmap)
# checklocalgrad(len(qmE.keys())+  2*len(knownparameters['bonds'])+4 * len(knownparameters['angles']), parameters, qmE, mmE, geometries, typesmap)
# ivals = np.random.randint(0,len(parameters),size=20)
# for val in ivals:
# 	checklocalgrad(val, parameters, qmE, mmE, geometries, typesmap)
boundslist = []
for i in range(len(qmE.keys())):
	boundslist.append([None, None])
for i in range(len(knownparameters['bonds'])):
	boundslist.append([0, 1000])
	boundslist.append([0, 10])
for i in range(len(knownparameters['angles'])):
	boundslist.append([0, 500])
	boundslist.append([0, 180])
	boundslist.append([0, 100])
	boundslist.append([0, 10])
for i in range(5*len(knownparameters['dihedrals'])):
	boundslist.append([-10,10])
boffset = len(qmE.keys())
aoffset = boffset + 2*len(knownparameters['bonds'])
doffset = aoffset + 4 * len(knownparameters['angles'])
tightboundslist = []
for i in range(len(qmE.keys())):
	tightboundslist.append([None, None])
for i in range(len(knownparameters['bonds'])):
	if parameters[boffset+2*i] != 0:
		tightboundslist.append([0, 1000])
		tightboundslist.append([0, 10])
	else:
		tightboundslist.append([0,0])
		tightboundslist.append([0,0])
for i in range(len(knownparameters['angles'])):
	if parameters[aoffset+4*i] != 0:
		tightboundslist.append([0, 500])
		tightboundslist.append([0, 180])
	else:
		tightboundslist.append([0,0])
		tightboundslist.append([0,0])
	if parameters[aoffset+4*i+2] != 0:
		tightboundslist.append([0, 100])
		tightboundslist.append([0, 10])
	else:
		tightboundslist.append([0,0])
		tightboundslist.append([0,0])
for i in range(5*len(knownparameters['dihedrals'])):
	if parameters[doffset+i] != 0:
		tightboundslist.append([-10,10])
	else:
		tightboundslist.append([0,0])
counter = 0
def cb(x):
	global itstart, counter
	print counter, time.clock()-itstart
	itstart = time.clock()
	counter += 1
itstart = time.clock()
start = time.clock()
result = scipy.optimize.minimize(objectivefunction, parameters, args=(qmE, mmE, geometries, typesmap), jac=True, method='SLSQP',bounds=boundslist,options={'maxiter': 10000}, callback=cb)
end = time.clock()
print "Optimization took %.2f seconds" % (end-start)
print result
np.save("unrestrictedsolution.npy", result.x)
start = time.clock()
result2 = scipy.optimize.minimize(objectivefunction, parameters, args=(qmE, mmE, geometries, typesmap), jac=True, method='SLSQP',bounds=tightboundslist,options={'maxiter': 10000}, callback=cb)
end = time.clock()
print "Optimization took %.2f seconds" % (end-start)
print result2
np.save("restrictedsolution.npy", result2.x)