import matplotlib
import numpy as np
from scipy.interpolate import interp1d
from matplotlib.ticker import MaxNLocator
matplotlib.use('Cairo')
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}",r"\renewcommand{\seriesdefault}{\bfdefault}",
r"\usepackage{amsfonts}",r"\usepackage{textcomp}",r"\usepackage{gensymb}", r'\boldmath'] #Using \boldmath makes the axis label numbers bold as well! Looks good.
params = {'text.usetex' : True,
          'font.size' : 14,
          'font.family' : 'lmodern',
          'text.latex.unicode': True,
          'figure.figsize' : (5.6, 4.2), #8, 6 originally. Make it ~30% smaller so the text is ~30% bigger. (Multiply by .7)
          'figure.dpi' : 120
          }
matplotlib.rcParams.update(params)
import matplotlib.pyplot as plt
import glob
import os
import matplotlib.cm as cm
from scipy.stats import norm, cauchy
cmap = cm.get_cmap('viridis')
weightfactors = np.load("weightingfactors.npy")
numframes = np.load("framecount.npy")
numframes2 = np.load("framecount-cgenff.npy")
print len(numframes), len(numframes2)
#exit()
filelist = sorted(glob.glob("scanresults/*.js"))
filelist2 = sorted(glob.glob("scanresultscgenff/*.js"))
energies = np.load("energies.npy")
energies2 = np.load("energies-cgenff.npy")
qmenergies = energies[0]
mmenergies = energies[1]
residuals = []#[np.load("pythonlibrary/initialresiduals.npy")]
labels = []#['CGenFF']
flist = sorted(glob.glob("pythonlibrary/*residuals.npy"))
for i, stuff in enumerate(flist):
	residuals.append(np.load(stuff))
	labels.append(os.path.basename(stuff)[:-13])
cgenffresiduals = np.load("pythonlibrary/initialresiduals.npy")
idxs = np.nonzero(qmenergies<5)[0]
#idxs2 = np.nonzero(energies2[0]<5)[0]
print idxs
#print np.sqrt(np.mean(np.square(cgenffresiduals[idxs2]))), np.mean(np.abs(cgenffresiduals))
for i in range(len(residuals)):
	print flist[i], np.sqrt(np.mean(np.square(residuals[i][idxs]))), np.mean(np.abs(residuals[i]))
#exit()
fig, ax =plt.subplots(1,1)
#labels=['CGenFF', 'Restricted Fit', 'Restricted Fit + 10', 'Unrestricted Fit', 'Tightened Fit']
x = np.linspace(-5,5,200)
for i, data in enumerate(residuals):
	#mu, std = cauchy.fit(data[np.nonzero(np.abs(data)<5)[0]])
	mu, std = cauchy.fit(data)
	print mu, std, cauchy.cdf(0.5,mu,std)-cauchy.cdf(-0.5,mu,std), flist[i]
	y = cauchy.pdf(x, mu, std)
	ax.plot(x,y, label=labels[i], color=cmap(float(i)/(len(residuals)-1)))
	ax.hist(data, 50, range=[-5,5], color=cmap(float(i)/(len(residuals)-1)), alpha=0.2, normed=True)
# ax.hist(cgenffresiduals,50, range=[-10,10],color='k', alpha=0.2)
# ax.hist(residuals[0],50, range=[-10,10],color='b', alpha=0.2)
# ax.hist(residuals[1],50, range=[-10,10],color='r', alpha=0.2)
ax.set_xlabel("Frame Residual (kcal/mol)")
ax.set_ylabel("Probability")
ax.legend(fontsize=6)
fig.tight_layout()
fig.savefig("residualplot.png")

f = np.array([0, 0.02, 0.1, 0.5])
colors = ['red', 'goldenrod', 'green', 'skyblue', 'navyblue']
midfixes = ['noforce', '', 'strongforce', 'reallystrongforce']
prefixes = ['tightbounds', 'tightbounds', 'tightbounds', 'loosebounds']
postfix = ['', 'fixdelta', 'fixdeltaupper2', '']
labels = ['Set A', 'Set B', 'Set C',  'Free']
fout2 = open("rmsetable.tex", "w")
fout2.write('''\\begin{tabular}{ccccc}
\\toprule
Dihedral Set & v=0 & v=0.02 & v=0.1 & v=0.5 \\\\
\midrule
''')
fout = open("residualfittable.tex", "w")
fout.write('''\\begin{tabular}{ccccc}
\\toprule
Dihedral Set & v=0 & v=0.02 & v=0.1 & v=0.5 \\\\
\midrule
''')
for i in range(4):
	s = "%s" % labels[i]
	s2 = "%s" % labels[i]
	for j in range(4):
		data = np.load("pythonlibrary/%s%s%sresiduals.npy" % (prefixes[i], midfixes[j], postfix[i]))
		mu, std = cauchy.fit(data)
		s += " & %.2g" % std
		s2 += " & %.2g" % np.sqrt(np.mean(np.square(data)))
	s += " \\\\\n"
	s2 += " \\\\\n"
	fout.write(s)
	fout2.write(s2)
fout2.write('''\cmidrule(lr){1-2}
CGenFF & %.2g & & & \\\\
\\bottomrule
\end{tabular}
''' % np.sqrt(np.mean(np.square(cgenffresiduals))))
fout.write('''\cmidrule(lr){1-2}
CGenFF & %.2g & & & \\\\
\\bottomrule
\end{tabular}
''' % cauchy.fit(cgenffresiduals)[1])
fout.close()
fout2.close()
fig, ax = plt.subplots(1,1)
ax.hist(cgenffresiduals, 100, range=[-5,5], color='k', alpha=0.2, normed=True, label='Histogram')
mu, std = cauchy.fit(cgenffresiduals)
ax.plot(x,cauchy.pdf(x, mu, std), label='Cauchy', color='r')
mu, std = norm.fit(cgenffresiduals)
ax.plot(x,norm.pdf(x, mu, std), '--', label='Normal', color='dodgerblue')
ax.legend(fontsize=10)
ax.set_xlabel("Frame Residual (kcal/mol)")
ax.set_ylabel("Probability")
ax.set_xlim(-5,5)
ax.set_yticks([])
fig.tight_layout()
fig.savefig("distributiontest.png", dpi=300)
fig.savefig("distributiontest.pdf", dpi=300)
#exit()
fig, axes = plt.subplots(1,4, sharey=True, figsize=(12,4))
fig.subplots_adjust(wspace=0,hspace=0, left=0.08, top=0.92, right=0.98, bottom=0.15)

for i in range(4):
	ax = axes[i]
	ax.fill_between([-0.5,0.5],[0,0],[1,1], color='k', alpha=0.2)
	mu, std = cauchy.fit(cgenffresiduals)
	y = cauchy.cdf(x,mu,std)
	ax.plot(x, y, label="CGenFF", color='k')
	ax.text(0.5, 1.01, labels[i], transform=ax.transAxes, ha='center')
	for j in range(4):
		data = np.load("pythonlibrary/%s%s%sresiduals.npy" % (prefixes[i], midfixes[j], postfix[i]))
		mu, std = cauchy.fit(data)
		y = cauchy.cdf(x, mu, std)
		vlabel = "v=%.1f" % f[j]
		if j == 1:
			vlabel = "v=%.2f" % f[j]
		ax.plot(x, y, label=vlabel, color=colors[j])
	ax.set_ylim(0, 1)
	ax.set_xlim(-5,5)
	ax.grid(True)
	ax.set_xticks(np.array([-4,-2,0,2,4]))
axes[3].legend(fontsize=10)
#axes[0].get_yaxis().set_ticklabels([])
fig.text(0.5, 0.03, "Frame Residual (kcal/mol)", transform=fig.transFigure, ha='center')
fig.text(0.02, 0.5, "Cumulative Probability", transform=fig.transFigure, rotation='vertical', va='center')
fig.savefig("forcecomparisionresiduals.png", dpi=300)
fig, axes = plt.subplots(1,4, sharey=True, figsize=(12,4))
fig.subplots_adjust(wspace=0,hspace=0, left=0.08, top=0.92, right=0.98, bottom=0.15)
colors = ['indianred', 'orange', 'limegreen', 'royalblue']
midfixes = ['noforce', '', 'strongforce', 'reallystrongforce']
prefixes = ['tightbounds', 'tightbounds', 'tightbounds', 'loosebounds']
postfix = ['', 'fixdelta', 'fixdeltaupper2', '']
labels = ['Set A', 'Set B', 'Set C', 'Free']
for i in range(4):
	ax = axes[i]
	ax.fill_between([-0.5,0.5],[0,0],[1,1], color='k', alpha=0.2)
	mu, std = cauchy.fit(cgenffresiduals)
	y = cauchy.cdf(x,mu,std)
	ax.plot(x, y, label="CGenFF", color='k')
	vlabel = "v=%.1f" % f[i]
	if i == 1:
		vlabel = "v=%.2f" % f[i]
	ax.text(0.5, 1.01, vlabel, transform=ax.transAxes, ha='center')
	for j in range(4):
		data = np.load("pythonlibrary/%s%s%sresiduals.npy" % (prefixes[j], midfixes[i], postfix[j]))
		mu, std = cauchy.fit(data)
		y = cauchy.cdf(x, mu, std)
		ax.plot(x, y, label=labels[j], color=colors[j])
	ax.set_ylim(0, 1)
	ax.set_xlim(-5,5)
	ax.set_xticks(np.array([-4,-2,0,2,4]))
	ax.grid(True)
axes[3].legend(fontsize=10)
#axes[0].get_yaxis().set_ticklabels([])
fig.text(0.5, 0.03, "Frame Residual (kcal/mol)", transform=fig.transFigure, ha='center')
fig.text(0.02, 0.5, "Cumulative Probability", transform=fig.transFigure, rotation='vertical', va='center')
fig.savefig("dihedralcomparisionresiduals.png", dpi=300)
#exit()
offset = 0
residuals = [cgenffresiduals]
for j in range(4):
	residuals.append(np.load("pythonlibrary/%s%s%sresiduals.npy" % (prefixes[j], '', postfix[j])))
for r in residuals:
	print r.shape
print qmenergies.shape
datasets = [qmenergies,qmenergies-residuals[0], qmenergies-residuals[1], qmenergies-residuals[2], qmenergies-residuals[3], qmenergies-residuals[4]]
colors = ['k', 'gray', 'indianred', 'orange', 'limegreen', 'royalblue']
labels = ['QM', 'CGenFF', 'Set A', 'Set B', 'Set C', 'Free']
lfsum=np.zeros(5,dtype=np.float)
cgenffcounter = 0
coffset = 0
lfstatistics = []
fout = open("largeresiduals.txt", "w")
for i, fcount in enumerate(numframes):
	print filelist[i], lfsum#, filelist2[cgenffcounter]
	#exit()
	fig, ax = plt.subplots(1,1)
	x = 1 + np.arange(fcount)
	newx = 1 + np.linspace(0,fcount-1, 200)
	w = np.minimum(np.exp(-0.25*datasets[0][offset:offset+fcount]),np.ones(fcount))
	w *= weightfactors[i]
	for j in range(6):
		#if j < 4:
		ax.plot(x, datasets[j][offset:offset+fcount], 'o', color=colors[j], label=labels[j])
		# elif filelist[i].split("/")[-1] != filelist2[cgenffcounter].split("/")[-1]:
		# 	continue
		# else:
		#ax.plot(x, datasets[j][coffset:coffset+fcount], 'o', color=colors[j])
		if fcount >= 4:
			# if j >=4:
			# 	f = interp1d(x, datasets[j][coffset:coffset+fcount], kind='cubic')
			#else:
			f = interp1d(x, datasets[j][offset:offset+fcount], kind='cubic')
			
			ax.plot(newx, f(newx), '-', color=colors[j])
		if j >0:
			lf = np.sum(w * residuals[j-1][offset:offset+fcount] * residuals[j-1][offset:offset+fcount])
			lfsum[j-1] += lf
			if j == 4:
				lfstatistics.append(lf)
				if lf > 100:
					fout.write("%s\n" % filelist[i][12:-3])
			if j == 1:
				l = labels[j]
			elif j == 5:
				l = "Free"
			else:
				l = labels[j][-5:]
			#ax.text(0.02, 0.97-0.05*j, "%-6s f = %.1f" % (l,lf), transform=ax.transAxes, color=colors[j])
		# elif j == 4:
		# 	lf = np.sum(w * cgenffresiduals[coffset:coffset+fcount] * cgenffresiduals[coffset:coffset+fcount])
		# 	lfsum[j-2] += lf
		# 	ax.text(0.02, 1.02-0.05*j, "localf = %.1f" % lf, transform=ax.transAxes, color=colors[j])
	ax.set_xlabel("Pose")
	ax.xaxis.set_major_locator(MaxNLocator(integer=True))
	ax.set_ylabel("Energy (kcal/mol)")
	ax.legend(fontsize=10)
	fig.tight_layout()
	fig.savefig("posterplots/%s.png" % filelist[i][12:-3], dpi=300, transparent=True)
	plt.close('all')
	offset += fcount
	if filelist[i].split("/")[-1] == filelist2[cgenffcounter].split("/")[-1]:
		coffset += fcount
		cgenffcounter += 1
fig, ax = plt.subplots(1,1)
logdata = np.log10(lfstatistics)
plt.hist(logdata)
fig.savefig("logresidualplot.png", transparent=True)
print lfsum
fout.close()