from Molecule import Molecule
from VMD import evaltcl
import numpy as np
from atomsel import *
import glob
import os
import vmdnumpy as vnp
os.chdir("scanresults")

filelist = glob.glob("*.js")


def isfloat(f):
	try:
		float(f)
		return True
	except ValueError:
		return False
def readLJparameters(ljdict, filename):
	fin = open(filename)
	lines = fin.readlines()[:-2]
	fin.close()
	dictfill = False
	for i, l in enumerate(lines):
		if i == len(lines) - 2:
			break
		string = l.strip()
		if len(string):
			idx = string.find("!")
			if idx < 0:
				s = string
			else:
				s = string[:idx]
			if len(s):
				splt = s.split()
				if splt[0] == "NONBONDED":
					dictfill = True
				elif splt[0] == "HBOND":
					dictfill = False
				elif splt[0] == "NBFIX":
					dictfill = False
				elif dictfill and isfloat(splt[2]) and isfloat(splt[3]):
					if len(splt) > 4:
						ljdict[splt[0]] = (float(splt[2]),float(splt[3]),float(splt[5]), float(splt[6]))
					else:
						ljdict[splt[0]] = (float(splt[2]),float(splt[3]))
				#print s, len(s)
def nonbondedterm(R, types, charges, p14, p123):
	total = 0
	for i in range(len(R)):
		pi = R[i]
		iLJprm = LJparams[types[i]]
		for j in range(i+1, len(R)):
			pj = R[j]
			#print types[j]
			jLJprm = LJparams[types[j]]
			if (i,j) in p123:
				continue
			elif (i,j) in p14:
				if len(iLJprm) > 2:
					ieps = iLJprm[2]
					imin = iLJprm[3]
				else:
					ieps = iLJprm[0]
					imin = iLJprm[1]
				if len(jLJprm) > 2:
					jeps = jLJprm[2]
					jmin = jLJprm[3]
				else:
					jeps = jLJprm[0]
					jmin = jLJprm[1]
				eps = np.sqrt(ieps * jeps)
				rmin = imin + jmin
			else:
				eps = np.sqrt(iLJprm[0] * jLJprm[0])
				rmin = iLJprm[1] + jLJprm[1]
			r = np.linalg.norm(pi-pj)
			total += eps * ((rmin/r)**12-2*(rmin/r)**6)
			total += 332.0716 * charges[i] * charges[j] / r
	return total
def nonbondedforces(R, types, charges, p14, p123):
	forces = np.zeros(R.shape, dtype=np.float)
	for i in range(len(R)):
		pi = R[i]
		iLJprm = LJparams[types[i]]
		for j in range(i+1, len(R)):
			pj = R[j]
			#print types[j]
			jLJprm = LJparams[types[j]]
			if (i,j) in p123:
				continue
			elif (i,j) in p14:
				if len(iLJprm) > 2:
					ieps = iLJprm[2]
					imin = iLJprm[3]
				else:
					ieps = iLJprm[0]
					imin = iLJprm[1]
				if len(jLJprm) > 2:
					jeps = jLJprm[2]
					jmin = jLJprm[3]
				else:
					jeps = jLJprm[0]
					jmin = jLJprm[1]
				eps = np.sqrt(ieps * jeps)
				rmin = imin + jmin
			else:
				eps = np.sqrt(iLJprm[0] * jLJprm[0])
				rmin = iLJprm[1] + jLJprm[1]
			r = np.linalg.norm(pi-pj)
			rvec = pi-pj #vector is the one pointing from j TO i.
			r6 = (rmin/r)**6
			vdwmag = eps * 12 * (r6*r6-r6) / r**2
			forces[i] += rvec * (vdwmag + 332.0716 * charges[i] * charges[j] / r**3) #If the charges are of opposite sign, then the force should point towards j. Similarly, vdwmag < 0 when the force should be attractive.
			forces[j] += -rvec * (vdwmag + 332.0716 * charges[i] * charges[j] / r**3)
	return forces
def getpairs(sel):
	bondlist = sel.bonds
	#print bondlist
	excl12 = set()
	excl13 = set()
	excl14 = set()
	idxlist = sel.get("index")
	for i in idxlist:
		bonds = sorted(bondlist[idxlist.index(i)])
		for j in bonds:
			if i < j:
				bondj = sorted(bondlist[idxlist.index(j)])
				excl12.add((i, j))
				for k in bonds:
					if k < j:
						excl13.add((k,j))
					if k != j:
						for l in bondj:
							if l != i and l != k:
								if k < l:
									excl14.add((k,l))
								else:
									excl14.add((l,k))
			else:
				for k in bonds:
					if k < j:
						excl13.add((k,j))
	excl123 = excl12.union(excl13)
	excl14 = excl14.difference(excl123)
	return (excl14, excl123)
LJparams = dict()

readLJparameters(LJparams, "/home/josh/toppar/par_all36_cgenff.prm")
readLJparameters(LJparams, "/home/josh/toppar/par_all36_carb.prm")
readLJparameters(LJparams, "/projects/LigninParameterization/typing/lignins-cgo2.prm")
readLJparameters(LJparams, "/home/josh/toppar/toppar_water_ions.str")

for f in filelist:
	mol = Molecule()
	mol.load(f)
	qmenergies = np.load(f[:-2] + "npy")
	mmenergies = np.zeros(len(qmenergies), dtype = np.float)
	#mmforces = np.zeros((mol.numAtoms(), 3), dtype=np.float)
	allsel = atomsel("all")
	ex14, ex123 = getpairs(allsel)
	charges = np.array(allsel.get("charge"), dtype=np.float)
	types = allsel.get("type")
	for i in range(mol.numFrames()):
		R = vnp.timestep(mol, i)
		mmenergies[i] = nonbondedterm(R, types, charges, ex14, ex123)
	mol.delete()
	mmenergies -= np.amin(mmenergies)
	np.save(f[:-3] + "-mm.npy", mmenergies)
	#np.save(f[:-3] + "-mmforces.npy", mmforces)
os.chdir("..")
filelist = glob.glob("minimumstructures/*.js")
for f in filelist:
	mol = Molecule()
	mol.load(f)
	#qmenergies = np.load(f[:-2] + "npy")
	mmenergies = np.zeros(mol.numFrames(), dtype = np.float)
	mmforces = np.zeros((mol.numFrames(), mol.numAtoms(), 3), dtype=np.float)
	allsel = atomsel("all")
	ex14, ex123 = getpairs(allsel)
	charges = np.array(allsel.get("charge"), dtype=np.float)
	types = allsel.get("type")
	for i in range(mol.numFrames()):
		R = vnp.timestep(mol, i)
		mmenergies[i] = nonbondedterm(R, types, charges, ex14, ex123)
		mmforces[i] = nonbondedforces(R, types, charges, ex14, ex123)
	mol.delete()
	mmenergies -= np.amin(mmenergies)
	np.save(f[:-3] + "-mm.npy", mmenergies)
	np.save(f[:-3] + "-mmforces.npy", mmforces)
exit()