import glob
import numpy as np
from Molecule import Molecule
import molecule
from VMD import evaltcl
from atomsel import *
import vmdnumpy as vnp
import topology
import measure
import cPickle as pickle
from scipy.spatial.distance import pdist
from scipy.spatial.distance import squareform
knownparameters = dict()
knownparameters['bonds'] = dict()
knownparameters['angles'] = dict()
knownparameters['dihedrals'] = dict()
knownparameters['nonbonded'] = dict()

#Read in the bond, angle, and dihedral terms that CGenFF gives us.
def readparameters(filename):
	fin = open(filename)
	lines = fin.readlines()
	fin.close()
	readstate = 0
	for l in lines:
		s = l.strip()
		if s.find("!") != -1:
			s = s[:s.find("!")]
		if len(s):
			splt = s.split()
			if splt[0] == "BONDS":
				readstate = 1
				skiplines = 1
			elif splt[0] == "ANGLES":
				readstate = 2
				skiplines = 1
			elif splt[0] == "DIHEDRALS":
				readstate = 3
				skiplines = 1
			elif splt[0] == "IMPROPERS":
				readstate = 0
			elif splt[0] == "IMPROPER":
				readstate = 0
			elif splt[0] == "NONBONDED":
				readstate = 4
				skiplines = 2
			elif splt[0] == "END":
				readstate = 0
			elif splt[0] == "HBOND":
				readstate = 0
			if readstate != 0:
				if skiplines:
					skiplines -= 1
				else:
					#print splt
					if readstate == 1:
						bondkey = "-".join(sorted(splt[:2]))
						bondprm = np.array(splt[2:], dtype=np.float)
						#print bondkey, bondprm
						knownparameters['bonds'][bondkey] = bondprm
					elif readstate == 2:
						q = splt[:3]
						if splt[0] > splt[2]: 
							q.reverse()
						anglekey = "-".join(q)
						anglprm = np.array(splt[3:], dtype=np.float)
						knownparameters['angles'][anglekey] = anglprm
					elif readstate == 3:
						q = splt[:4]
						oldq = q[:]
						print q
						#Topotools uses the inner atoms to sort dihedral order, so we should too.
						if splt[1] > splt[2]: 
							q.reverse()
							first = True
						elif (splt[1] == splt[2]) and (splt[0] > splt[3]):
							q.reverse()
							first = False
						if q != oldq:
							print oldq, q, first, splt[0], splt[3]
						anglekey = "-".join(q)
						#print anglekey
						anglprm = tuple(np.array(splt[4:], dtype=np.float))
						if anglekey in knownparameters['dihedrals']:
							knownparameters['dihedrals'][anglekey].add(anglprm)
							#print len(knownparameters['dihedrals'][anglekey])
						else:
							knownparameters['dihedrals'][anglekey] = set([anglprm])
					else:
						knownparameters['nonbonded'][splt[0]] = np.array(splt[1:], dtype=np.float)


readparameters("/home/josh/toppar/par_all36_cgenff.prm")
readparameters("/home/josh/toppar/par_all36_carb.prm")
readparameters("../../typing/lignins-cgo2.prm")
print knownparameters['dihedrals']['CGO2-CGO2-OG301-CGO2']
#exit()
from atomsel import atomsel
filelist = sorted(glob.glob("minimumstructures/*.js"))

btypes = set()
atypes = set()
dtypes = set()
molint = 0
for i, f in enumerate(filelist):
	mol = Molecule()
	mol.load(f)
	#sel = atomsel("type CGO")
	#sel.set("type", "CG2R61")
	molint = int(mol)
	print f
	evaltcl("package require topotools; topo retypebonds; topo retypeangles; topo retypedihedrals; topo retypeimpropers")
	for b in topology.bonds(type=1):
		btypes.add(b[2])
		knownparameters['bonds'][b[2]]
	for a in topology.angles(type=1):
		#print atypes
		atypes.add(a[3])
		knownparameters['angles'][a[3]]
	for d in topology.dihedrals(type=1):
		dtypes.add(d[4])
		knownparameters['dihedrals'][d[4]]
	mol.delete()
#exit()
btypes = sorted(list(btypes))
atypes = sorted(list(atypes))
dtypes = sorted(list(dtypes))

print dtypes

qmE = dict()
mmE = dict()
typesmap = dict()
geometries = dict()
accumulatedframes = 0
ubdistances = dict()
weights = []
angletypes = set()
filelist = sorted(glob.glob("scanresults/*.js"))
for i, f in enumerate(filelist):
	mol = Molecule()
	mol.load(f)
	# sel = atomsel("type CGO")
	# sel.set("type", "CG2R61")
	print f
	if mol.numFrames() == 0:
		print "Zero frames!"
		exit()
	accumulatedframes += mol.numFrames()
	molidx = int(mol)-molint-1
	typesmap[molidx] = dict()
	evaltcl("package require topotools; topo retypebonds; topo retypeangles; topo retypedihedrals; topo retypeimpropers")
	
	bvallist = []
	avallist = []
	ubvallist = []
	dvallist = []
	btypelist = []
	frameexclusions = np.zeros(mol.numFrames(), dtype=np.bool)
	#Are there any atoms that are within 1.65 angstroms of something they are not bonded to? If so, we should exclude the frame, since Gaussian gave us a strange trajectory.
	sel = atomsel("all")
	bondlist = sel.bonds

	masses = np.array(sel.get("mass"))
	#exit()
	for frame in range(mol.numFrames()):
		R = vnp.timestep(mol, frame)
		distances = squareform(pdist(R))
		#print distances
		nnz = np.nonzero(distances < 1.65)
		for i, j in zip(nnz[0], nnz[1]):
			if not (i == j or j in bondlist[i] or masses[i] + masses[j] < 3):
				print frame
				frameexclusions[frame] = True
	mmenergies = np.load(f[:-3] + "-mm.npy")
	frameexclusions |= (mmenergies - np.amin(mmenergies)) > 20
	if np.any(frameexclusions):
		print f
		#exit()
	for b in topology.bonds(type=1):
		btypelist.append(btypes.index(b[2]))
		bondlengths = np.array(measure.bond(b[0],b[1],first=0, last=mol.numFrames()-1), dtype=np.float)
		frameexclusions |= bondlengths > 1.65
		if np.any(bondlengths > 1.65):
			print f
		bvallist.append(bondlengths)
	if np.sum(frameexclusions) == mol.numFrames():
		print "All frames excluded"
		exit()
	atypelist = []
	for a in topology.angles(type=1):
		#print atypes
		atypelist.append(atypes.index(a[3]))
		avallist.append(np.array(measure.angle(a[0],a[1],a[2],first=0, last=mol.numFrames()-1), dtype=np.float))
		ubdist = np.array(measure.bond(a[0],a[2],first=0, last=mol.numFrames()-1), dtype=np.float)
		ubvallist.append(ubdist)
		ubdistances[atypes.index(a[3])] = ubdist
	dtypelist = []
	maxr = 0
	for d in topology.dihedrals(type=1):
		dtypelist.append(dtypes.index(d[4]))
		diheds = np.array(measure.dihedral(d[0],d[1],d[2],d[3],first=0, last=mol.numFrames()-1), dtype=np.float)
		r = (np.amax(diheds) - np.amin(diheds))
		r -= 360 * np.round(r/360)
		if r > maxr:
			maxr = r
		dvallist.append(diheds)
	print maxr, mol.numFrames()
	if mol.numFrames() <=5 and maxr <= 25:
		weights.append(20)
	else:
		weights.append(1)
	typesmap[molidx]['b'] = btypelist
	typesmap[molidx]['a'] = atypelist
	typesmap[molidx]['d'] = dtypelist
	#print typesmap
	if np.any(frameexclusions):
		print "Some frames should be excluded, since the geometries so perturbed they don't represent sane structures."
		print f
		idxs = np.nonzero(frameexclusions == 0)[0]
		print idxs
		mmE[ molidx ] = np.load(f[:-3] + "-mm.npy")[idxs]
		qmE[ molidx ] = np.load(f[:-2] + "npy")[idxs]
		geometries[molidx] = dict()
		geometries[molidx]['b'] = np.vstack(bvallist)[:,idxs]
		geometries[molidx]['a'] = np.vstack(avallist)[:,idxs]
		geometries[molidx]['ub'] = np.vstack(ubvallist)[:,idxs]
		geometries[molidx]['d'] = np.vstack(dvallist)[:,idxs]
		#exit()
	else:
		mmE[ molidx ] = np.load(f[:-3] + "-mm.npy")
		qmE[ molidx ] = np.load(f[:-2] + "npy")
		geometries[molidx] = dict()
		geometries[molidx]['b'] = np.vstack(bvallist)
		geometries[molidx]['a'] = np.vstack(avallist)
		geometries[molidx]['ub'] = np.vstack(ubvallist)
		geometries[molidx]['d'] = np.vstack(dvallist)
	aromaticc = atomsel("name \"C.*\" and numbonds == 3")
	aromaticclist = aromaticc.get("index")
	for idx, t in zip(aromaticclist, aromaticc.get("type")):
		localsel = atomsel("(withinbonds 1 of index %d) and not index %d" % (idx, idx))
		ltypes = localsel.get("type")
		typecombos = []
		for j,k in zip([0,0,1],[1,2,2]):
			typetest = "-".join([ltypes[j],t,ltypes[k]])
			if typetest not in atypes:
				typetest = "-".join([ltypes[k],t,ltypes[j]])
			index = atypes.index(typetest)
			if index == -1:
				print "WTF??"
				exit()
			typecombos.append(index)
		angletypes.add(tuple(typecombos))
		#lindex = localsel.get("index")
	#print geometries
	#exit()
print angletypes
#exit()
print molecule.listall()
weights = np.array(weights, dtype=np.float)


filelist = sorted(glob.glob("minimumstructures/*.js"))
numframes = np.empty(len(filelist), dtype=np.int)
numatoms = np.empty(len(filelist), dtype=np.int)
electroforcelist = []
minimumtypemap = dict()
coordinatelist = []
for i, f in enumerate(filelist):
	mol = Molecule()
	mol.load(f)
	# sel = atomsel("type CGO")
	# sel.set("type", "CG2R61")
	molxx = int(mol)-molidx-1
	minimumtypemap[molxx] = dict()
	numatoms[i] = mol.numAtoms()
	numframes[i] = mol.numFrames()
	forces = np.load(f[:-3]+ "-mmforces.npy")
	forces = forces.reshape(-1,3)
	electroforcelist.append(forces)
	for frame in range(mol.numFrames()):
		R = vnp.timestep(mol, frame)
		coordinatelist.append(R)
	evaltcl("package require topotools; topo retypebonds; topo retypeangles; topo retypedihedrals; topo retypeimpropers")

	btypelist = []
	for b in topology.bonds(type=1):
		btypelist.append(np.array([b[0], b[1], btypes.index(b[2])]))
		bondlengths = np.array(measure.bond(b[0],b[1],first=0, last=mol.numFrames()-1), dtype=np.float)
		if np.any(bondlengths > 1.65):
			print f
			print "I have a problem... Long bonds!"
			exit()
	atypelist = []
	for a in topology.angles(type=1):
		#print atypes
		atypelist.append(np.array([a[0], a[1], a[2], atypes.index(a[3])]))
	dtypelist = []
	maxr = 0
	for d in topology.dihedrals(type=1):
		dtypelist.append(np.array([d[0],d[1],d[2],d[3],dtypes.index(d[4])]))
	minimumtypemap[molxx]['b'] = btypelist
	minimumtypemap[molxx]['a'] = atypelist
	minimumtypemap[molxx]['d'] = dtypelist
np.save("nbforces.npy", np.vstack(electroforcelist).flatten(order='C'))
np.save("minimumcoordinates.npy", np.vstack(coordinatelist).flatten(order='C'))
pickle.dump( (knownparameters, qmE, mmE, typesmap, geometries, btypes, atypes, dtypes, angletypes), open( "everything.pkl", "wb" ) )
np.save("weightingfactors.npy", weights)
np.save("minimumnumframes.npy", numframes)
np.save("minimumnumatoms.npy", numatoms)




#exit()
bondp = np.zeros(2*len(btypes), dtype=np.float)
for i, k in enumerate(btypes):
	bondp[2*i+0] = knownparameters['bonds'][k][0]
	bondp[2*i+1] = knownparameters['bonds'][k][1]
anglep = np.zeros(4 * len(atypes), dtype=np.float)
for i, k in enumerate(atypes):
	anglep[4*i+0] = knownparameters['angles'][k][0]
	anglep[4*i+1] = knownparameters['angles'][k][1]
	if len(knownparameters['angles'][k]) > 2:
		anglep[4*i+2] = knownparameters['angles'][k][2]
		anglep[4*i+3] = knownparameters['angles'][k][3]
	else:
		anglep[4*i+3] = np.mean(ubdistances[i])
dihedp = np.zeros(5*len(dtypes), dtype=np.float)
for i, k in enumerate(dtypes):
	setels = list(knownparameters['dihedrals'][k])
	for j in range(len(setels)):
		l = int(setels[j][1])
		if l == 6:
			l = 5
		sign = 1
		if setels[j][2] > 0:
			sign = -1
		dihedp[5*i+l-1] = sign * setels[j][0]

np.save("parameters.npy", np.hstack([np.zeros(len(qmE.keys())), bondp, anglep, dihedp]))
framecount = np.empty(len(qmE.keys()), dtype=np.int)
for i, f in enumerate(sorted(qmE.keys())):
	framecount[i] = len(qmE[f])
fcountsum = np.cumsum(framecount)
print fcountsum
print accumulatedframes

for i, f in enumerate(sorted(qmE.keys())):
	print fcountsum[i], f
np.save("framecount.npy", framecount)
np.save("numbers.npy", np.array([np.sum(framecount), len(qmE.keys()), len(btypes), len(atypes), len(dtypes)]))
qE = np.empty(np.sum(framecount), dtype=np.float)
mE = np.empty(np.sum(framecount), dtype=np.float)
csum = np.hstack([0,np.cumsum(framecount)])
for i, f in enumerate(sorted(qmE.keys())):
	qE[csum[i]:csum[i+1]] = qmE[f]
	mE[csum[i]:csum[i+1]] = mmE[f]
np.save("energies.npy", np.vstack([qE, mE]))
types = np.empty((len(qmE.keys()), 3), dtype=np.int)
btypes = []
atypes = []
dtypes = []
bgeom = []
ageom = []
ubgeom = []
dgeom = []
for i, f in enumerate(sorted(qmE.keys())):
	types[i][0] = len(typesmap[f]['b'])
	types[i][1] = len(typesmap[f]['a'])
	types[i][2] = len(typesmap[f]['d'])
	btypes.append(np.array(typesmap[f]['b'], dtype=np.int))
	atypes.append(np.array(typesmap[f]['a'], dtype=np.int))
	dtypes.append(np.array(typesmap[f]['d'], dtype=np.int))
	#print geometries[f]['b'].shape
	bgeom.append(geometries[f]['b'].flatten())
	ageom.append(geometries[f]['a'].flatten())
	ubgeom.append(geometries[f]['ub'].flatten())
	dgeom.append(geometries[f]['d'].flatten())
np.save("typecounts.npy", types)
np.save("btypes.npy", np.hstack(btypes))
np.save("atypes.npy", np.hstack(atypes))
np.save("dtypes.npy", np.hstack(dtypes))
np.save("bgeom.npy", np.hstack(bgeom))
np.save("ageom.npy", np.hstack(ageom))
np.save("ubgeom.npy", np.hstack(ubgeom))
np.save("dgeom.npy", np.hstack(dgeom))

types = np.empty((len(minimumtypemap.keys()), 3), dtype=np.int)
btypes = []
atypes = []
dtypes = []
for i, f in enumerate(sorted(minimumtypemap.keys())):
	types[i][0] = len(minimumtypemap[f]['b'])
	types[i][1] = len(minimumtypemap[f]['a'])
	types[i][2] = len(minimumtypemap[f]['d'])
	btypes.append(np.hstack(minimumtypemap[f]['b']))
	atypes.append(np.hstack(minimumtypemap[f]['a']))
	dtypes.append(np.hstack(minimumtypemap[f]['d']))
np.save("minimumtypecounts.npy", types)
np.save("minimumbtypes.npy", np.hstack(btypes))
np.save("minimumatypes.npy", np.hstack(atypes))
np.save("minimumdtypes.npy", np.hstack(dtypes))
print qmE.keys()
exit()