#include <iostream>
#include "cnpy.h"
#include <vector>
#include <cstring>
#include <cstdio>
#include <typeinfo>
#include <cmath>
#include <ctime>
#include "vector_types.h" //For float3

#define _USE_MATH_DEFINES
using namespace std;

#include "optimization.h"


cublasHandle_t cublasHd;

int *framecount, *mininumframes, *mininumatoms;
int numframes, nummols, numbondp, numanglep, numdihedp, paramlength, mininummols;
double *devg, *devx;
double forceweight;
int coordsize, framemols;
int *numbondtypes, *numangletypes, *numdihedtypes;
int *numminimumbondtypes, *numminimumangletypes, *numminimumdihedtypes;
int *minimumbtypes, *minimumatypes, *minimumdtypes;
int *btypes, *atypes, *dtypes;
double *d_residuals;
double *bgeom, *ageom, *ubgeom, *dgeom;
double *geometries, *nbforces;
double *targetE;
double *weights;
GPUdata *Gdata;



template <typename T>
struct weighting
{
	__host__ __device__
		T operator()(const T& x, const T& y) const { 
			return exp(-0.25*x);// * exp(-2e-2*y);
	}
};
template <typename T>
struct square {
	__host__ __device__
	T operator()(const T& x) const {
		return x * x;
	}
};
template <typename T>
struct atanderiv
{
	__host__ __device__
	T operator()(const T& x) const {
		//return atan(x) / (x*x + 1);
		return tanh(x) / (cosh(x) * cosh(x));
	}
};
//This is a very handy function that is part of the thrust examples. Lets me turn a list of lengths into a mapping.
template <typename InputIterator1,
			typename InputIterator2,
			typename OutputIterator>
OutputIterator expand(InputIterator1 first1,
						InputIterator1 last1,
						InputIterator2 first2,
						OutputIterator output)
{
	typedef typename thrust::iterator_difference<InputIterator1>::type difference_type;
	
	difference_type input_size  = thrust::distance(first1, last1);
	difference_type output_size = thrust::reduce(first1, last1);

	// scan the counts to obtain output offsets for each input element
	thrust::device_vector<difference_type> output_offsets(input_size, 0);
	thrust::exclusive_scan(first1, last1, output_offsets.begin()); 

	// scatter the nonzero counts into their corresponding output positions
	thrust::device_vector<difference_type> output_indices(output_size, 0);
	thrust::scatter_if
	(thrust::counting_iterator<difference_type>(0),
	 thrust::counting_iterator<difference_type>(input_size),
	 output_offsets.begin(),
	 first1,
	 output_indices.begin());

	// compute max-scan over the output indices, filling in the holes
	thrust::inclusive_scan
	(output_indices.begin(),
	 output_indices.end(),
	 output_indices.begin(),
	 thrust::maximum<difference_type>());

	// gather input values according to index array (output = first2[output_indices])
	OutputIterator output_end = output; thrust::advance(output_end, output_size);
	thrust::gather(output_indices.begin(),
				 output_indices.end(),
				 first2,
				 output);

	// return output + output_size
	thrust::advance(output, output_size);
	return output;
}
template<typename Iterator, typename T, typename BinaryOperation, typename Pointer>
__global__ void reduce_kernel(Iterator first, Iterator last, T init, BinaryOperation binary_op, Pointer result)
{
	*result = thrust::reduce(thrust::cuda::par, first, last, init, binary_op);
}

__global__
void reweightbondandangles(double *w, double *weightfactor, int *molmap, int numframes) {
	int f = blockIdx.x*blockDim.x + threadIdx.x;
	if (f < numframes) {
		int mol = molmap[f];
		double wf = weightfactor[mol];
		w[f] *= wf;
	}
}

//This is the routine to load the data onto the GPU, and prepare it for evaluation.
void prepGPU(double *qm, double *mm) {
	Gdata->weights_dev = thrust::device_vector<double> (numframes);
	Gdata->targetE_dev = thrust::device_vector<double> (numframes);
	//qm_dev and mm_dev are local variables.
	thrust::device_vector<double>qm_dev(qm, qm+numframes);
	thrust::device_vector<double>mm_dev(mm, mm+numframes);
	thrust::transform(qm_dev.begin(), qm_dev.end(), mm_dev.begin(), Gdata->targetE_dev.begin(), thrust::minus<real>());
	thrust::transform(qm_dev.begin(), qm_dev.end(), mm_dev.begin(), Gdata->weights_dev.begin(), weighting<double>());
	Gdata->framebondproduct_dev = thrust::device_vector<int> (nummols);
	Gdata->frameangleproduct_dev = thrust::device_vector<int> (nummols);
	Gdata->framedihedproduct_dev = thrust::device_vector<int> (nummols);

	Gdata->sumnumbondtypes_dev = thrust::device_vector<int> (nummols);
	Gdata->sumnumangletypes_dev = thrust::device_vector<int> (nummols);
	Gdata->sumnumdihedtypes_dev = thrust::device_vector<int> (nummols);
	Gdata->frameoffsets = thrust::device_vector<int> (nummols);
	Gdata->framecount_dev = thrust::device_vector<int> (framecount, framecount + nummols);
	Gdata->numbondtypes_dev = thrust::device_vector<int> (numbondtypes, numbondtypes + nummols);
	Gdata->numangletypes_dev = thrust::device_vector<int> (numangletypes, numangletypes + nummols);
	Gdata->numdihedtypes_dev = thrust::device_vector<int> (numdihedtypes, numdihedtypes + nummols);
	thrust::transform(Gdata->framecount_dev.begin(), Gdata->framecount_dev.end(), Gdata->numbondtypes_dev.begin(), Gdata->framebondproduct_dev.begin(), thrust::multiplies<int>());
	thrust::transform(Gdata->framecount_dev.begin(), Gdata->framecount_dev.end(), Gdata->numangletypes_dev.begin(), Gdata->frameangleproduct_dev.begin(), thrust::multiplies<int>());
	thrust::transform(Gdata->framecount_dev.begin(), Gdata->framecount_dev.end(), Gdata->numdihedtypes_dev.begin(), Gdata->framedihedproduct_dev.begin(), thrust::multiplies<int>());
	thrust::inclusive_scan(Gdata->framebondproduct_dev.begin(), Gdata->framebondproduct_dev.end(), Gdata->framebondproduct_dev.begin());
	thrust::inclusive_scan(Gdata->frameangleproduct_dev.begin(), Gdata->frameangleproduct_dev.end(), Gdata->frameangleproduct_dev.begin());
	thrust::inclusive_scan(Gdata->framedihedproduct_dev.begin(), Gdata->framedihedproduct_dev.end(), Gdata->framedihedproduct_dev.begin());
	//At this point, the frameXproduct has the geomoffsets I need
	thrust::exclusive_scan(Gdata->numbondtypes_dev.begin(), Gdata->numbondtypes_dev.end(), Gdata->sumnumbondtypes_dev.begin());
	thrust::exclusive_scan(Gdata->numangletypes_dev.begin(), Gdata->numangletypes_dev.end(), Gdata->sumnumangletypes_dev.begin());
	thrust::exclusive_scan(Gdata->numdihedtypes_dev.begin(), Gdata->numdihedtypes_dev.end(), Gdata->sumnumdihedtypes_dev.begin());
	//Now sumnumXtypes can be used to get the Xtypeoffsets.
	thrust::exclusive_scan(Gdata->framecount_dev.begin(), Gdata->framecount_dev.end(), Gdata->frameoffsets.begin());

	thrust::counting_iterator<int> firstmol(0);
	Gdata->molmap = thrust::device_vector<int> (numframes);
	expand(Gdata->framecount_dev.begin(), Gdata->framecount_dev.end(),
		 firstmol,
		 Gdata->molmap.begin());
	Gdata->btypes_dev = thrust::device_vector<int> (btypes, btypes + thrust::reduce(Gdata->numbondtypes_dev.begin(), Gdata->numbondtypes_dev.end()));
	Gdata->atypes_dev = thrust::device_vector<int> (atypes, atypes + thrust::reduce(Gdata->numangletypes_dev.begin(), Gdata->numangletypes_dev.end()));
	Gdata->dtypes_dev = thrust::device_vector<int> (dtypes, dtypes + thrust::reduce(Gdata->numdihedtypes_dev.begin(), Gdata->numdihedtypes_dev.end()));
	Gdata->bgeom_dev = thrust::device_vector<double> (bgeom, bgeom + *(Gdata->framebondproduct_dev.end()-1));
	Gdata->ageom_dev = thrust::device_vector<double> (ageom, ageom + *(Gdata->frameangleproduct_dev.end()-1));
	Gdata->ubgeom_dev = thrust::device_vector<double> (ubgeom, ubgeom + *(Gdata->frameangleproduct_dev.end()-1));
	Gdata->dgeom_dev = thrust::device_vector<double> (dgeom, dgeom + *(Gdata->framedihedproduct_dev.end()-1));
	// //memCopy(thrust::raw_pointer_cast(&weights_dev[0]), weights, sizeof(real) * numframes, cudaMemcpyDeviceToHost);
	// thrust::copy(qm_dev.begin(), qm_dev.begin() + 10, std::ostream_iterator<float>(std::cout, " "));
	// printf("\n");
	// thrust::copy(mm_dev.begin(), mm_dev.begin() + 10, std::ostream_iterator<float>(std::cout, " "));
	// printf("\n");
	// thrust::copy(targetE_dev.begin(), targetE_dev.begin() + 10, std::ostream_iterator<float>(std::cout, " "));
	// printf("\n");
	// thrust::copy(weights_dev.begin(), weights_dev.begin() + 10, std::ostream_iterator<float>(std::cout, " "));
	// printf("\n");
	Gdata->molbondproduct_dev = thrust::device_vector<int> (mininummols);
	Gdata->molangleproduct_dev = thrust::device_vector<int> (mininummols);
	Gdata->moldihedproduct_dev = thrust::device_vector<int> (mininummols);
	Gdata->numminimumbondtypes_dev = thrust::device_vector<int> (numminimumbondtypes, numminimumbondtypes + mininummols);
	Gdata->numminimumangletypes_dev = thrust::device_vector<int> (numminimumangletypes, numminimumangletypes + mininummols);
	Gdata->numminimumdihedtypes_dev = thrust::device_vector<int> (numminimumdihedtypes, numminimumdihedtypes + mininummols);
	Gdata->minibtypes_dev = thrust::device_vector<int> (minimumbtypes, minimumbtypes + 3 * thrust::reduce(Gdata->numminimumbondtypes_dev.begin(), Gdata->numminimumbondtypes_dev.end()));
	Gdata->miniatypes_dev = thrust::device_vector<int> (minimumatypes, minimumatypes + 4 * thrust::reduce(Gdata->numminimumangletypes_dev.begin(), Gdata->numminimumangletypes_dev.end()));
	Gdata->minidtypes_dev = thrust::device_vector<int> (minimumdtypes, minimumdtypes + 5 * thrust::reduce(Gdata->numminimumdihedtypes_dev.begin(), Gdata->numminimumdihedtypes_dev.end()));
	Gdata->numatoms_dev = thrust::device_vector<int> (mininumatoms, mininumatoms + mininummols);
	Gdata->minimumframecount_dev = thrust::device_vector<int> (mininumframes, mininumframes + mininummols);
	Gdata->molsize_dev = thrust::device_vector<int> (mininummols);
	thrust::transform(Gdata->numatoms_dev.begin(), Gdata->numatoms_dev.end(), Gdata->minimumframecount_dev.begin(), Gdata->molsize_dev.begin(), thrust::multiplies<int>());
	thrust::transform(Gdata->numatoms_dev.begin(), Gdata->numatoms_dev.end(), Gdata->numminimumbondtypes_dev.begin(), Gdata->molbondproduct_dev.begin(), thrust::multiplies<int>());
	thrust::transform(Gdata->numatoms_dev.begin(), Gdata->numatoms_dev.end(), Gdata->numminimumangletypes_dev.begin(), Gdata->molangleproduct_dev.begin(), thrust::multiplies<int>());
	thrust::transform(Gdata->numatoms_dev.begin(), Gdata->numatoms_dev.end(), Gdata->numminimumdihedtypes_dev.begin(), Gdata->moldihedproduct_dev.begin(), thrust::multiplies<int>());
	thrust::inclusive_scan(Gdata->molbondproduct_dev.begin(), Gdata->molbondproduct_dev.end(), Gdata->molbondproduct_dev.begin());
	thrust::inclusive_scan(Gdata->molangleproduct_dev.begin(), Gdata->molangleproduct_dev.end(), Gdata->molangleproduct_dev.begin());
	thrust::inclusive_scan(Gdata->moldihedproduct_dev.begin(), Gdata->moldihedproduct_dev.end(), Gdata->moldihedproduct_dev.begin());
	Gdata->sumnumminimumbondtypes_dev = thrust::device_vector<int> (mininummols);
	Gdata->sumnumminimumangletypes_dev = thrust::device_vector<int> (mininummols);
	Gdata->sumnumminimumdihedtypes_dev = thrust::device_vector<int> (mininummols);
	thrust::exclusive_scan(Gdata->numminimumbondtypes_dev.begin(), Gdata->numminimumbondtypes_dev.end(), Gdata->sumnumminimumbondtypes_dev.begin());
	thrust::exclusive_scan(Gdata->numminimumangletypes_dev.begin(), Gdata->numminimumangletypes_dev.end(), Gdata->sumnumminimumangletypes_dev.begin());
	thrust::exclusive_scan(Gdata->numminimumdihedtypes_dev.begin(), Gdata->numminimumdihedtypes_dev.end(), Gdata->sumnumminimumdihedtypes_dev.begin());
	Gdata->moloffsets = thrust::device_vector<int> (mininummols);
	thrust::exclusive_scan(Gdata->molsize_dev.begin(), Gdata->molsize_dev.end(), Gdata->moloffsets.begin());
	Gdata->coord_dev = thrust::device_vector<double> (geometries, geometries + 3 * coordsize);
	printf("\nCoorddata:\n");
	thrust::copy(Gdata->coord_dev.begin(), Gdata->coord_dev.begin() + 20, std::ostream_iterator<float>(std::cout, " "));
	printf("\n");
	Gdata->nbforce_dev = thrust::device_vector<double> (nbforces, nbforces + 3 * coordsize);
	Gdata->minimolmap = thrust::device_vector<int> (framemols);
	expand(Gdata->minimumframecount_dev.begin(), Gdata->minimumframecount_dev.end(),
		 firstmol,
		 Gdata->minimolmap.begin());
	thrust::copy(Gdata->minimumframecount_dev.begin(), Gdata->minimumframecount_dev.begin() + 20, std::ostream_iterator<float>(std::cout, " "));
	printf("\n");
	thrust::copy(Gdata->minimolmap.begin(), Gdata->minimolmap.begin() + 20, std::ostream_iterator<float>(std::cout, " "));
	printf("\n");
	thrust::copy(Gdata->numatoms_dev.begin(), Gdata->numatoms_dev.begin() + 20, std::ostream_iterator<float>(std::cout, " "));
	printf("\n");
	thrust::exclusive_scan(Gdata->minimumframecount_dev.begin(), Gdata->minimumframecount_dev.end(), Gdata->minimumframecount_dev.begin());
}
//Reasonable call:
//computeBondForces(coord_dev, minimol, moloffsets, numatoms_dev, sumnumminimumbondtypes_dev, sumnumminimumangletypes_dev, sumnumminimumdihedtypes_dev, 
//numminimumbondtypes_dev, numminimumangletypes_dev, numminimumdihedtypes_dev)
__global__
void computeBondForces(double *x, double *coords, double *forces, double *forcegrad, int *minimolmap, int *moloffsets, int *numatoms, int *sumbondtypes, int *sumangletypes, int *sumdihedtypes, int *numbondtypes,
		int *numangletypes, int *numdihedtypes, int *bondtypes, int *angletypes, int *dihedtypes, int boffset, int aoffset, int doffset, int framemols, int coordsize, int *frameoffsets) {
	int f = blockIdx.x*blockDim.x + threadIdx.x;
	if (f < framemols) {
		int mol = minimolmap[f];
		int frame = f - frameoffsets[mol];
		int coordoffset = moloffsets[mol] + numatoms[mol] * frame;
		//printf("coordoffset %d, mol %d, frame %d, moloffsets %d\n", coordoffset, mol, f, moloffsets[mol]);
		int btypeoffset = sumbondtypes[mol];
		int atypeoffset = sumangletypes[mol];
		int dtypeoffset = sumdihedtypes[mol];
		//printf("Starting bond calc, %d\n", f);
		for ( int idx = 0; idx < numbondtypes[mol]; idx++ ) {
			//Do math to figure out the forces, and gradients to the force too.
			int t = bondtypes[3 * (btypeoffset + idx) + 2];
			double k = x[boffset + 2*t];
			double bo = x[boffset + 2*t+1];
			int i = bondtypes[3 * (btypeoffset + idx) + 0];
			int j = bondtypes[3 * (btypeoffset + idx) + 1];
			//printf("Coordsize %d, coordoffset %d, i: %d, j: %d\n", coordsize, coordoffset, i, j);
			double xij = coords[3*(coordoffset+i)+0] - coords[3*(coordoffset+j)+0];
			double yij = coords[3*(coordoffset+i)+1] - coords[3*(coordoffset+j)+1];
			double zij = coords[3*(coordoffset+i)+2] - coords[3*(coordoffset+j)+2];
			double r = sqrt(xij*xij + yij*yij + zij*zij);
			double tmp = r - bo;
			double fij = -2 / r;
			//printf("k %f, tmp, %f, r: %f bo: %f\n", k, tmp, r, bo);
			forces[3*(coordoffset+i)+0] += xij * fij * k * tmp;
			forces[3*(coordoffset+i)+1] += yij * fij * k * tmp;
			forces[3*(coordoffset+i)+2] += zij * fij * k * tmp;
			forces[3*(coordoffset+j)+0] -= xij * fij * k * tmp;
			forces[3*(coordoffset+j)+1] -= yij * fij * k * tmp;
			forces[3*(coordoffset+j)+2] -= zij * fij * k * tmp;
			//forcegrad needs to be coordsize * paramlength in size.
			forcegrad[(boffset+2*t) * 3 * coordsize + 3*(coordoffset+i)+0] += xij * fij * tmp;
			forcegrad[(boffset+2*t) * 3 * coordsize + 3*(coordoffset+i)+1] += yij * fij * tmp;
			forcegrad[(boffset+2*t) * 3 * coordsize + 3*(coordoffset+i)+2] += zij * fij * tmp;
			forcegrad[(boffset+2*t) * 3 * coordsize + 3*(coordoffset+j)+0] -= xij * fij * tmp;
			forcegrad[(boffset+2*t) * 3 * coordsize + 3*(coordoffset+j)+1] -= yij * fij * tmp;
			forcegrad[(boffset+2*t) * 3 * coordsize + 3*(coordoffset+j)+2] -= zij * fij * tmp;
			//Gradient also has a piece involved with the bonds. Following the chain rule, you just divide out the tmp part, and flip the sign.
			forcegrad[(boffset+2*t+1) * 3 * coordsize + 3*(coordoffset+i)+0] -= xij * fij * k;
			forcegrad[(boffset+2*t+1) * 3 * coordsize + 3*(coordoffset+i)+1] -= yij * fij * k;
			forcegrad[(boffset+2*t+1) * 3 * coordsize + 3*(coordoffset+i)+2] -= zij * fij * k;
			forcegrad[(boffset+2*t+1) * 3 * coordsize + 3*(coordoffset+j)+0] += xij * fij * k;
			forcegrad[(boffset+2*t+1) * 3 * coordsize + 3*(coordoffset+j)+1] += yij * fij * k;
			forcegrad[(boffset+2*t+1) * 3 * coordsize + 3*(coordoffset+j)+2] += zij * fij * k;
		}
		//printf("Starting angle calc, %d\n", f);
		for ( int idx = 0; idx < numangletypes[mol]; idx++ ) {
			//Do math to figure out the forces.
			int t = angletypes[4 * (atypeoffset + idx) + 3];
			double ktheta = x[aoffset + 4*t];
			double ao = x[aoffset + 4*t+1];
			double kub = x[aoffset + 4*t+2];
			double bub = x[aoffset + 4*t+3];
			int i = angletypes[4 * (atypeoffset + idx) + 0];
			int j = angletypes[4 * (atypeoffset + idx) + 1];
			int k = angletypes[4 * (atypeoffset + idx) + 2];
			double xij = coords[3*(coordoffset+i)+0] - coords[3*(coordoffset+j)+0];
			double yij = coords[3*(coordoffset+i)+1] - coords[3*(coordoffset+j)+1];
			double zij = coords[3*(coordoffset+i)+2] - coords[3*(coordoffset+j)+2];

			double xkj = coords[3*(coordoffset+k)+0] - coords[3*(coordoffset+j)+0];
			double ykj = coords[3*(coordoffset+k)+1] - coords[3*(coordoffset+j)+1];
			double zkj = coords[3*(coordoffset+k)+2] - coords[3*(coordoffset+j)+2];

			double xik = coords[3*(coordoffset+i)+0] - coords[3*(coordoffset+k)+0];
			double yik = coords[3*(coordoffset+i)+1] - coords[3*(coordoffset+k)+1];
			double zik = coords[3*(coordoffset+i)+2] - coords[3*(coordoffset+k)+2];
			double rij_inv = rsqrt(xij*xij + yij*yij + zij*zij);
			double rkj_inv = rsqrt(xkj*xkj + ykj*ykj + zkj*zkj);
			double rik_inv = rsqrt(xik*xik + yik*yik + zik*zik);
			double rik = 1.0f/rik_inv;
			double xijr = xij*rij_inv;
			double yijr = yij*rij_inv;
			double zijr = zij*rij_inv;
			double xkjr = xkj*rkj_inv;
			double ykjr = ykj*rkj_inv;
			double zkjr = zkj*rkj_inv;
			double cos_theta = xijr*xkjr + yijr*ykjr + zijr*zkjr;
			cos_theta = min(0.999999f, max(-0.999999f, cos_theta));
			double diff = acos(cos_theta) - radians(ao);
			double inv_sin_theta = rsqrt(1.0f - cos_theta*cos_theta);
			if (inv_sin_theta > 1.0e6) {
				diff = (diff < 0.0f) ? 1.0f : -1.0f;
				inv_sin_theta = -diff;
			} else {
				diff *= -inv_sin_theta;
			}
			//diff *= -2 * ktheta;

			double dtxi = rij_inv*(xkjr - cos_theta*xijr);
			double dtxj = rkj_inv*(xijr - cos_theta*xkjr);
			double dtyi = rij_inv*(ykjr - cos_theta*yijr);
			double dtyj = rkj_inv*(yijr - cos_theta*ykjr);
			double dtzi = rij_inv*(zkjr - cos_theta*zijr);
			double dtzj = rkj_inv*(zijr - cos_theta*zkjr);
			//printf("I'm here! %d, %d\n", f, idx);
			forces[3*(coordoffset+i)+0] += -2 * ktheta * diff * dtxi;
			forces[3*(coordoffset+i)+1] += -2 * ktheta * diff * dtyi;
			forces[3*(coordoffset+i)+2] += -2 * ktheta * diff * dtzi;
			forces[3*(coordoffset+j)+0] -= -2 * ktheta * diff * (dtxi+dtxj);
			forces[3*(coordoffset+j)+1] -= -2 * ktheta * diff * (dtyi+dtyj);
			forces[3*(coordoffset+j)+2] -= -2 * ktheta * diff * (dtzi+dtzj);
			forces[3*(coordoffset+k)+0] += -2 * ktheta * diff * dtxj;
			forces[3*(coordoffset+k)+1] += -2 * ktheta * diff * dtyj;
			forces[3*(coordoffset+k)+2] += -2 * ktheta * diff * dtzj;
			//printf("I'm past the forces. %d, %d\n", f, idx);
			forcegrad[(aoffset+4*t) * 3 * coordsize + 3*(coordoffset+i)+0] += -2 * diff * dtxi;
			forcegrad[(aoffset+4*t) * 3 * coordsize + 3*(coordoffset+i)+1] += -2 * diff * dtyi;
			forcegrad[(aoffset+4*t) * 3 * coordsize + 3*(coordoffset+i)+2] += -2 * diff * dtzi;
			forcegrad[(aoffset+4*t) * 3 * coordsize + 3*(coordoffset+j)+0] -= -2 * diff * (dtxi+dtxj);
			forcegrad[(aoffset+4*t) * 3 * coordsize + 3*(coordoffset+j)+1] -= -2 * diff * (dtyi+dtyj);
			forcegrad[(aoffset+4*t) * 3 * coordsize + 3*(coordoffset+j)+2] -= -2 * diff * (dtzi+dtzj);
			forcegrad[(aoffset+4*t) * 3 * coordsize + 3*(coordoffset+k)+0] += -2 * diff * dtxj;
			forcegrad[(aoffset+4*t) * 3 * coordsize + 3*(coordoffset+k)+1] += -2 * diff * dtyj;
			forcegrad[(aoffset+4*t) * 3 * coordsize + 3*(coordoffset+k)+2] += -2 * diff * dtzj;
			forcegrad[(aoffset+4*t+1) * 3 * coordsize + 3*(coordoffset+i)+0] += -2 * ktheta * inv_sin_theta * dtxi * M_PI / 180;
			forcegrad[(aoffset+4*t+1) * 3 * coordsize + 3*(coordoffset+i)+1] += -2 * ktheta * inv_sin_theta * dtyi * M_PI / 180;
			forcegrad[(aoffset+4*t+1) * 3 * coordsize + 3*(coordoffset+i)+2] += -2 * ktheta * inv_sin_theta * dtzi * M_PI / 180;
			forcegrad[(aoffset+4*t+1) * 3 * coordsize + 3*(coordoffset+j)+0] -= -2 * ktheta * inv_sin_theta * (dtxi+dtxj) * M_PI / 180;
			forcegrad[(aoffset+4*t+1) * 3 * coordsize + 3*(coordoffset+j)+1] -= -2 * ktheta * inv_sin_theta * (dtyi+dtyj) * M_PI / 180;
			forcegrad[(aoffset+4*t+1) * 3 * coordsize + 3*(coordoffset+j)+2] -= -2 * ktheta * inv_sin_theta * (dtzi+dtzj) * M_PI / 180;
			forcegrad[(aoffset+4*t+1) * 3 * coordsize + 3*(coordoffset+k)+0] += -2 * ktheta * inv_sin_theta * dtxj * M_PI / 180;
			forcegrad[(aoffset+4*t+1) * 3 * coordsize + 3*(coordoffset+k)+1] += -2 * ktheta * inv_sin_theta * dtyj * M_PI / 180;
			forcegrad[(aoffset+4*t+1) * 3 * coordsize + 3*(coordoffset+k)+2] += -2 * ktheta * inv_sin_theta * dtzj * M_PI / 180;
			//printf("Now calculating UB. %d, %d\n", f, idx);
			double diff_ub = rik - bub;
			//diff_ub *= -2.0*kub*rik_inv;
			forces[3*(coordoffset+i)+0] += -2 * kub * rik_inv * xik * diff_ub;
			forces[3*(coordoffset+i)+1] += -2 * kub * rik_inv * yik * diff_ub;
			forces[3*(coordoffset+i)+2] += -2 * kub * rik_inv * zik * diff_ub;
			forces[3*(coordoffset+k)+0] -= -2 * kub * rik_inv * xik * diff_ub;
			forces[3*(coordoffset+k)+1] -= -2 * kub * rik_inv * yik * diff_ub;
			forces[3*(coordoffset+k)+2] -= -2 * kub * rik_inv * zik * diff_ub;

			forcegrad[(aoffset+4*t+2) * 3 * coordsize + 3*(coordoffset+i)+0] += -2 * rik_inv * xik * diff_ub;
			forcegrad[(aoffset+4*t+2) * 3 * coordsize + 3*(coordoffset+i)+1] += -2 * rik_inv * yik * diff_ub;
			forcegrad[(aoffset+4*t+2) * 3 * coordsize + 3*(coordoffset+i)+2] += -2 * rik_inv * zik * diff_ub;
			forcegrad[(aoffset+4*t+2) * 3 * coordsize + 3*(coordoffset+k)+0] -= -2 * rik_inv * xik * diff_ub;
			forcegrad[(aoffset+4*t+2) * 3 * coordsize + 3*(coordoffset+k)+1] -= -2 * rik_inv * yik * diff_ub;
			forcegrad[(aoffset+4*t+2) * 3 * coordsize + 3*(coordoffset+k)+2] -= -2 * rik_inv * zik * diff_ub;
			forcegrad[(aoffset+4*t+3) * 3 * coordsize + 3*(coordoffset+i)+0] += 2 * kub * rik_inv * xik;
			forcegrad[(aoffset+4*t+3) * 3 * coordsize + 3*(coordoffset+i)+1] += 2 * kub * rik_inv * yik;
			forcegrad[(aoffset+4*t+3) * 3 * coordsize + 3*(coordoffset+i)+2] += 2 * kub * rik_inv * zik;
			forcegrad[(aoffset+4*t+3) * 3 * coordsize + 3*(coordoffset+k)+0] -= 2 * kub * rik_inv * xik;
			forcegrad[(aoffset+4*t+3) * 3 * coordsize + 3*(coordoffset+k)+1] -= 2 * kub * rik_inv * yik;
			forcegrad[(aoffset+4*t+3) * 3 * coordsize + 3*(coordoffset+k)+2] -= 2 * kub * rik_inv * zik;
			//printf("At the end of the loop.%d, %d\n", f, idx);
		}
		//printf("Starting dihed calc\n");
		for ( int idx = 0; idx < numdihedtypes[mol]; idx++ ) {
			//Do math to figure out the forces.
			int t = dihedtypes[5 * (dtypeoffset + idx) + 4];
			int i = dihedtypes[5 * (dtypeoffset + idx) + 0];
			int j = dihedtypes[5 * (dtypeoffset + idx) + 1];
			int k = dihedtypes[5 * (dtypeoffset + idx) + 2];
			int l = dihedtypes[5 * (dtypeoffset + idx) + 3];

			double xij = coords[3*(coordoffset+i)+0] - coords[3*(coordoffset+j)+0];
			double yij = coords[3*(coordoffset+i)+1] - coords[3*(coordoffset+j)+1];
			double zij = coords[3*(coordoffset+i)+2] - coords[3*(coordoffset+j)+2];

			double xjk = coords[3*(coordoffset+j)+0] - coords[3*(coordoffset+k)+0];
			double yjk = coords[3*(coordoffset+j)+1] - coords[3*(coordoffset+k)+1];
			double zjk = coords[3*(coordoffset+j)+2] - coords[3*(coordoffset+k)+2];

			double xlk = coords[3*(coordoffset+l)+0] - coords[3*(coordoffset+k)+0];
			double ylk = coords[3*(coordoffset+l)+1] - coords[3*(coordoffset+k)+1];
			double zlk = coords[3*(coordoffset+l)+2] - coords[3*(coordoffset+k)+2];

			// A=F^G, B=H^G.
			double ax = yij*zjk - zij*yjk;
			double ay = zij*xjk - xij*zjk;
			double az = xij*yjk - yij*xjk;
			double bx = ylk*zjk - zlk*yjk;
			double by = zlk*xjk - xlk*zjk;
			double bz = xlk*yjk - ylk*xjk;

			double ra2 = ax*ax + ay*ay + az*az;
			double rb2 = bx*bx + by*by + bz*bz;
			double rg = sqrt(xjk*xjk + yjk*yjk + zjk*zjk);
			double rgr = 1.0f / rg;
			double ra2r = 1.0f / ra2;
			double rb2r = 1.0f / rb2;
			double rabr = sqrt(ra2r*rb2r);
			double cos_phi = (ax*bx + ay*by + az*bz)*rabr;
			double sin_phi = rg*rabr*(ax*xlk + ay*ylk + az*zlk);
			double phi = atan2(sin_phi, cos_phi);
			double fg = xij*xjk + yij*yjk + zij*zjk;
			double hg = xlk*xjk + ylk*yjk + zlk*zjk;
			double fga = fg*ra2r*rgr;
			double hgb = hg*rb2r*rgr;
			double gaa = ra2r*rg;
			double gbb = rb2r*rg;
			for (int jj = 0; jj < 5; jj++) {
				int n;
				switch (jj) {
					case 0:
					case 1:
					case 2:
					case 3:
					n=jj+1;
					break;
					case 4:
					n=jj+2;
					break;
				}
				double sin_x = sin(n * phi);
				double kk = x[doffset + 5*t + jj];
				//double df = (n*k*sin_x);
				forces[3*(coordoffset+i)+0] += -gaa * ax * n * kk * sin_x;
				forces[3*(coordoffset+i)+1] += -gaa * ay * n * kk * sin_x;
				forces[3*(coordoffset+i)+2] += -gaa * az * n * kk * sin_x;
				forcegrad[(doffset+5*t+jj) * 3 * coordsize + 3*(coordoffset+i)+0] += -gaa * ax * n * sin_x;
				forcegrad[(doffset+5*t+jj) * 3 * coordsize + 3*(coordoffset+i)+1] += -gaa * ay * n * sin_x;
				forcegrad[(doffset+5*t+jj) * 3 * coordsize + 3*(coordoffset+i)+2] += -gaa * az * n * sin_x;
				forces[3*(coordoffset+j)+0] += (fga * ax - hgb * bx + gaa * ax) * n * kk * sin_x;
				forces[3*(coordoffset+j)+1] += (fga * ay - hgb * by + gaa * ay) * n * kk * sin_x;
				forces[3*(coordoffset+j)+2] += (fga * az - hgb * bz + gaa * az) * n * kk * sin_x;
				forcegrad[(doffset+5*t+jj) * 3 * coordsize + 3*(coordoffset+j)+0] += (fga * ax - hgb * bx + gaa * ax) * n * sin_x;
				forcegrad[(doffset+5*t+jj) * 3 * coordsize + 3*(coordoffset+j)+1] += (fga * ay - hgb * by + gaa * ay) * n * sin_x;
				forcegrad[(doffset+5*t+jj) * 3 * coordsize + 3*(coordoffset+j)+2] += (fga * az - hgb * bz + gaa * az) * n * sin_x;
				forces[3*(coordoffset+k)+0] -= (fga * ax - hgb * bx + gaa * ax + gbb * bx) * n * kk * sin_x;
				forces[3*(coordoffset+k)+1] -= (fga * ay - hgb * by + gaa * ay + gbb * by) * n * kk * sin_x;
				forces[3*(coordoffset+k)+2] -= (fga * az - hgb * bz + gaa * az + gbb * bz) * n * kk * sin_x;
				forcegrad[(doffset+5*t+jj) * 3 * coordsize + 3*(coordoffset+k)+0] -= (fga * ax - hgb * bx + gaa * ax + gbb * bx) * n * sin_x;
				forcegrad[(doffset+5*t+jj) * 3 * coordsize + 3*(coordoffset+k)+1] -= (fga * ay - hgb * by + gaa * ay + gbb * by) * n * sin_x;
				forcegrad[(doffset+5*t+jj) * 3 * coordsize + 3*(coordoffset+k)+2] -= (fga * az - hgb * bz + gaa * az + gbb * bz) * n * sin_x;
				forces[3*(coordoffset+l)+0] += (gbb * bx) * n * kk * sin_x;
				forces[3*(coordoffset+l)+1] += (gbb * by) * n * kk * sin_x;
				forces[3*(coordoffset+l)+2] += (gbb * bz) * n * kk * sin_x;
				forcegrad[(doffset+5*t+jj) * 3 * coordsize + 3*(coordoffset+l)+0] += (gbb * bx) * n * sin_x;
				forcegrad[(doffset+5*t+jj) * 3 * coordsize + 3*(coordoffset+l)+1] += (gbb * by) * n * sin_x;
				forcegrad[(doffset+5*t+jj) * 3 * coordsize + 3*(coordoffset+l)+2] += (gbb * bz) * n * sin_x;
			}
		}
	}
}
__global__
void funcgradeval(double *residuals, double *lgradf, double *localf, double *x, int *framebondproduct, int *frameangleproduct,
	int *framedihedproduct, int *frameoffsets, int *framecount, int *sumnumbondtypes, int *sumnumangletypes, int *sumnumdihedtypes,
	int *numbondtypes, int *numangletypes, int *numdihedtypes, int *btypes, int *atypes, int *dtypes, double *bgeom, double *ageom, double *ubgeom, double *dgeom,
	int boffset, int aoffset, int doffset, int numframes, double *targetE, double *weights, int *molmap) {
	double tmp;
	int f = blockIdx.x*blockDim.x + threadIdx.x;
	if (f < numframes) {
		int mol = molmap[f];
		int bgeomoffset;
		int ageomoffset;
		int dgeomoffset;
		if (mol == 0) {
			bgeomoffset = 0;
			ageomoffset = 0;
			dgeomoffset = 0;			
		}
		else {
			bgeomoffset = framebondproduct[mol-1];
			ageomoffset = frameangleproduct[mol-1];
			dgeomoffset = framedihedproduct[mol-1];
		}
		int frameoffset = frameoffsets[mol];
		int lframecount = framecount[mol];
		int btypeoffset = sumnumbondtypes[mol];
		int atypeoffset = sumnumangletypes[mol];
		int dtypeoffset = sumnumdihedtypes[mol];
		double localE = 0.0;
		for (int i = 0; i < numbondtypes[mol]; i++) {
			int t = btypes[btypeoffset+i];
			double k = x[boffset + 2*t];
			double bo = x[boffset + 2*t+1];
			double tmp = bgeom[bgeomoffset + i * lframecount + f - frameoffset] - bo;
			localE += k * tmp * tmp;
			lgradf[(boffset+2*t)*numframes + f] += tmp * tmp;
			lgradf[(boffset+2*t+1)*numframes + f] -= 2 * k * tmp;
		}
		for (int i = 0; i < numangletypes[mol]; i++) {
			int t = atypes[atypeoffset+i];
			double k = x[aoffset + 4*t];
			double ao = x[aoffset + 4*t+1];
			double kub = x[aoffset + 4*t+2];
			double bub = x[aoffset + 4*t+3];
			double tmp = radians(ageom[ageomoffset + i * lframecount + f - frameoffset] - ao);
			double tmp2 = ubgeom[ageomoffset + i * lframecount + f - frameoffset] - bub;
			localE += k * tmp * tmp;
			lgradf[(aoffset+4*t)*numframes + f] += tmp * tmp;
			lgradf[(aoffset+4*t+1)*numframes + f] -= 2 * k * tmp * M_PI / 180;
			localE += kub * tmp2 * tmp2;
			lgradf[(aoffset+4*t+2)*numframes + f] += tmp2 * tmp2;
			lgradf[(aoffset+4*t+3)*numframes + f] -= 2 * kub * tmp2;
		}
		for (int i = 0; i < numdihedtypes[mol]; i++) {
			int t = dtypes[dtypeoffset+i];
			for (int j = 0; j < 5; j++) {
				int n;
				switch (j) {
					case 0:
					case 1:
					case 2:
					case 3:
					n=j+1;
					break;
					case 4:
					n=j+2;
					break;
				}
				double k = x[doffset + 5*t + j];
				double rads = radians(dgeom[dgeomoffset + i * lframecount + f - frameoffset]);
				double tmp = (1+cos(n*rads));
				localE += k * tmp;
				lgradf[(doffset+5*t+j)*numframes + f] += tmp;
			}
		}
		lgradf[mol*numframes + f] = 1.0;
		residuals[f] = targetE[f] - localE - x[mol];
		localf[f] = weights[f] * residuals[f] * residuals[f];
		//tmp = tanh(residuals[f]);
		//localf[f] = tmp * tmp;
	}
}
__global__
void dihedralbiasgradient(real *x, real *g, int doffset, int n) {
	int i = blockIdx.x*blockDim.x + threadIdx.x;
	if (i < n) {
		g[doffset + i] += 1e-2 * 2 * x[doffset + i];
	}
}
double dihedralbias(real *x, real *g, int doffset) {
	double f = 1e-2 * thrust::transform_reduce(thrust::device_pointer_cast(x+doffset), thrust::device_pointer_cast(x+doffset+5*numdihedp), square<double>(), 0.0, thrust::plus<double>());
	dihedralbiasgradient<<<(5*numdihedp+255)/256,256,0>>>(x, g, doffset, 5 * numdihedp);
	return f;
}
void funcgrad(real* x, double &f, real* g, const cudaStream_t& stream) {
	double alpha = -2.0;
	double beta = 0.0;
	//printf("Entered funcgrad\n");
	cublasStatus_t stat;
	thrust::device_vector<double> residuals(numframes,0.0);
	thrust::device_vector<double> lgradf(numframes*paramlength,0.0);
	thrust::device_vector<double> forcegradf(3*coordsize*paramlength, 0.0);
	thrust::device_vector<double> forces(3*coordsize, 0.0);
	thrust::device_vector<double> localf(numframes,0.0);
	funcgradeval<<<(numframes+511)/512, 512, 0, stream>>>(thrust::raw_pointer_cast(residuals.data()), thrust::raw_pointer_cast(lgradf.data()),
		thrust::raw_pointer_cast(localf.data()), x, thrust::raw_pointer_cast(Gdata->framebondproduct_dev.data()), thrust::raw_pointer_cast(Gdata->frameangleproduct_dev.data()),
		thrust::raw_pointer_cast(Gdata->framedihedproduct_dev.data()), thrust::raw_pointer_cast(Gdata->frameoffsets.data()), thrust::raw_pointer_cast(Gdata->framecount_dev.data()),
		thrust::raw_pointer_cast(Gdata->sumnumbondtypes_dev.data()), thrust::raw_pointer_cast(Gdata->sumnumangletypes_dev.data()), thrust::raw_pointer_cast(Gdata->sumnumdihedtypes_dev.data()),
		thrust::raw_pointer_cast(Gdata->numbondtypes_dev.data()), thrust::raw_pointer_cast(Gdata->numangletypes_dev.data()), thrust::raw_pointer_cast(Gdata->numdihedtypes_dev.data()),
		thrust::raw_pointer_cast(Gdata->btypes_dev.data()), thrust::raw_pointer_cast(Gdata->atypes_dev.data()), thrust::raw_pointer_cast(Gdata->dtypes_dev.data()),
		thrust::raw_pointer_cast(Gdata->bgeom_dev.data()), thrust::raw_pointer_cast(Gdata->ageom_dev.data()), thrust::raw_pointer_cast(Gdata->ubgeom_dev.data()), thrust::raw_pointer_cast(Gdata->dgeom_dev.data()),
		nummols, nummols + 2 * numbondp, nummols + 2 * numbondp + 4 * numanglep, numframes, thrust::raw_pointer_cast(Gdata->targetE_dev.data()), thrust::raw_pointer_cast(Gdata->weights_dev.data()),
		thrust::raw_pointer_cast(Gdata->molmap.data()));
	//printf("Compute Bonded Launched\n");
	computeBondForces<<<(framemols+127)/128, 128, 0, stream>>>(x, thrust::raw_pointer_cast(Gdata->coord_dev.data()), thrust::raw_pointer_cast(forces.data()), thrust::raw_pointer_cast(forcegradf.data()),
		thrust::raw_pointer_cast(Gdata->minimolmap.data()), thrust::raw_pointer_cast(Gdata->moloffsets.data()), thrust::raw_pointer_cast(Gdata->numatoms_dev.data()), thrust::raw_pointer_cast(Gdata->sumnumminimumbondtypes_dev.data()),
		thrust::raw_pointer_cast(Gdata->sumnumminimumangletypes_dev.data()), thrust::raw_pointer_cast(Gdata->sumnumminimumdihedtypes_dev.data()), thrust::raw_pointer_cast(Gdata->numminimumbondtypes_dev.data()),
		thrust::raw_pointer_cast(Gdata->numminimumangletypes_dev.data()), thrust::raw_pointer_cast(Gdata->numminimumdihedtypes_dev.data()), thrust::raw_pointer_cast(Gdata->minibtypes_dev.data()),
		thrust::raw_pointer_cast(Gdata->miniatypes_dev.data()), thrust::raw_pointer_cast(Gdata->minidtypes_dev.data()), nummols, nummols + 2 * numbondp, nummols + 2 * numbondp + 4 * numanglep, framemols, coordsize,
		thrust::raw_pointer_cast(Gdata->minimumframecount_dev.data()));
	//Reweight residuals so that dihedral stuff doesn't take over.
	thrust::device_vector<double> residualweight(numframes);
	thrust::transform(thrust::cuda::par.on(stream), residuals.begin(), residuals.end(), Gdata->weights_dev.begin(), residualweight.begin(), thrust::multiplies<double>());
	//Add nonbonded forces to the current forces, which just come from the bonded terms. Together, the forces should be near zero, since the bonded terms and nonbonded terms should nearly cancel at the minimum.
	// printf("Forces:\n");
	// thrust::copy(forces.begin(), forces.begin() + 10, std::ostream_iterator<float>(std::cout, " "));
	// printf("\n");
	// thrust::copy(Gdata->nbforce_dev.begin(), Gdata->nbforce_dev.begin() + 10, std::ostream_iterator<float>(std::cout, " "));
	// printf("\n");
	thrust::transform(thrust::cuda::par.on(stream), forces.begin(), forces.end(), Gdata->nbforce_dev.begin(), forces.begin(), thrust::plus<double>());
	// thrust::copy(forces.begin(), forces.begin() + 10, std::ostream_iterator<float>(std::cout, " "));
	// printf("\n");
	//thrust::transform(thrust::cuda::par.on(stream), residuals.begin(), residuals.end(), residualweight.begin(), atanderiv<double>());
	cublasSetStream(cublasHd, stream);
	stat = cublasDgemv(cublasHd, CUBLAS_OP_T, numframes, paramlength, &alpha, thrust::raw_pointer_cast(lgradf.data()), numframes, thrust::raw_pointer_cast(residualweight.data()), 1, &beta, g, 1);
	alpha = forceweight;//Scale all forces and such down.
	stat = cublasDscal(cublasHd, 3*coordsize, &alpha, thrust::raw_pointer_cast(forces.data()), 1);
	stat = cublasDscal(cublasHd, 3 * coordsize * paramlength, &alpha, thrust::raw_pointer_cast(forcegradf.data()), 1);
	alpha = 2.0;
	beta = 1.0;

	stat = cublasDgemv(cublasHd, CUBLAS_OP_T, 3 * coordsize, paramlength, &alpha, thrust::raw_pointer_cast(forcegradf.data()), 3 * coordsize, thrust::raw_pointer_cast(forces.data()), 1, &beta, g, 1);
	if (stat != CUBLAS_STATUS_SUCCESS) {
		printf("CUBLAS failure\n");
	}
	f = thrust::reduce(localf.begin(), localf.end(), 0.0, thrust::plus<double>()) + thrust::transform_reduce(forces.begin(), forces.end(), square<double>(), 0.0, thrust::plus<double>());
	// thrust::copy(localf.begin(), localf.begin() + 10, std::ostream_iterator<float>(std::cout, " "));
	// printf("\n, localfsum=%f\n", f);
	//f += dihedralbias(x, g, nummols + 2 * numbondp + 4 * numanglep);
	cudaDeviceSynchronize();
}
//This function determines the residuals, which is how far off at each point we are
void residuals(real *x, real *residuals, const cudaStream_t& stream) {
	thrust::device_vector<double> lgradf(numframes*paramlength,0.0);
	thrust::device_vector<double> localf(numframes,0.0);
	funcgradeval<<<(numframes+511)/512, 512, 0, stream>>>(residuals, thrust::raw_pointer_cast(lgradf.data()),
		thrust::raw_pointer_cast(localf.data()), x, thrust::raw_pointer_cast(Gdata->framebondproduct_dev.data()), thrust::raw_pointer_cast(Gdata->frameangleproduct_dev.data()),
		thrust::raw_pointer_cast(Gdata->framedihedproduct_dev.data()), thrust::raw_pointer_cast(Gdata->frameoffsets.data()), thrust::raw_pointer_cast(Gdata->framecount_dev.data()),
		thrust::raw_pointer_cast(Gdata->sumnumbondtypes_dev.data()), thrust::raw_pointer_cast(Gdata->sumnumangletypes_dev.data()), thrust::raw_pointer_cast(Gdata->sumnumdihedtypes_dev.data()),
		thrust::raw_pointer_cast(Gdata->numbondtypes_dev.data()), thrust::raw_pointer_cast(Gdata->numangletypes_dev.data()), thrust::raw_pointer_cast(Gdata->numdihedtypes_dev.data()),
		thrust::raw_pointer_cast(Gdata->btypes_dev.data()), thrust::raw_pointer_cast(Gdata->atypes_dev.data()), thrust::raw_pointer_cast(Gdata->dtypes_dev.data()),
		thrust::raw_pointer_cast(Gdata->bgeom_dev.data()), thrust::raw_pointer_cast(Gdata->ageom_dev.data()), thrust::raw_pointer_cast(Gdata->ubgeom_dev.data()), thrust::raw_pointer_cast(Gdata->dgeom_dev.data()),
		nummols, nummols + 2 * numbondp, nummols + 2 * numbondp + 4 * numanglep, numframes, thrust::raw_pointer_cast(Gdata->targetE_dev.data()), thrust::raw_pointer_cast(Gdata->weights_dev.data()),
		thrust::raw_pointer_cast(Gdata->molmap.data()));
	//thrust::copy(thrust::device_pointer_cast(residuals), thrust::device_pointer_cast(residuals) + 10, std::ostream_iterator<float>(std::cout, " "));
}
__global__
void divideandadd(real *x, real *sumres, double *count, int n) {
	int i = blockIdx.x*blockDim.x + threadIdx.x;
	if (i < n) {
		x[i] += sumres[i] / count[i];
		//printf("%d: %.2f, %.3f, %.3f\n", i, x[i], sumres[i], count[i]);
	}
}
void resetoffsets(real *x , const cudaStream_t& stream) {
	thrust::device_vector<double> residuals(numframes,0.0);
	thrust::device_vector<double> lgradf(numframes*paramlength,0.0);
	thrust::device_vector<double> localf(numframes,0.0);
	thrust::device_vector<double> force_dev(3 * coordsize, 0.0);
	funcgradeval<<<(numframes+511)/512, 512, 0, stream>>>(thrust::raw_pointer_cast(residuals.data()), thrust::raw_pointer_cast(lgradf.data()),
		thrust::raw_pointer_cast(localf.data()), x, thrust::raw_pointer_cast(Gdata->framebondproduct_dev.data()), thrust::raw_pointer_cast(Gdata->frameangleproduct_dev.data()),
		thrust::raw_pointer_cast(Gdata->framedihedproduct_dev.data()), thrust::raw_pointer_cast(Gdata->frameoffsets.data()), thrust::raw_pointer_cast(Gdata->framecount_dev.data()),
		thrust::raw_pointer_cast(Gdata->sumnumbondtypes_dev.data()), thrust::raw_pointer_cast(Gdata->sumnumangletypes_dev.data()), thrust::raw_pointer_cast(Gdata->sumnumdihedtypes_dev.data()),
		thrust::raw_pointer_cast(Gdata->numbondtypes_dev.data()), thrust::raw_pointer_cast(Gdata->numangletypes_dev.data()), thrust::raw_pointer_cast(Gdata->numdihedtypes_dev.data()),
		thrust::raw_pointer_cast(Gdata->btypes_dev.data()), thrust::raw_pointer_cast(Gdata->atypes_dev.data()), thrust::raw_pointer_cast(Gdata->dtypes_dev.data()),
		thrust::raw_pointer_cast(Gdata->bgeom_dev.data()), thrust::raw_pointer_cast(Gdata->ageom_dev.data()), thrust::raw_pointer_cast(Gdata->ubgeom_dev.data()), thrust::raw_pointer_cast(Gdata->dgeom_dev.data()),
		nummols, nummols + 2 * numbondp, nummols + 2 * numbondp + 4 * numanglep, numframes, thrust::raw_pointer_cast(Gdata->targetE_dev.data()), thrust::raw_pointer_cast(Gdata->weights_dev.data()),
		thrust::raw_pointer_cast(Gdata->molmap.data()));
	thrust::device_vector<int> keys(nummols);
	thrust::device_vector<double> count(nummols);
	thrust::device_vector<double> sumres(nummols,0.0);
	thrust::constant_iterator<int> iter(1);
	thrust::equal_to<int> binary_pred;
	thrust::reduce_by_key(Gdata->molmap.begin(), Gdata->molmap.end(), Gdata->weights_dev.begin(), keys.begin(), count.begin(), binary_pred);
	thrust::device_vector<double> residualweight(numframes);
	thrust::transform(thrust::cuda::par.on(stream), residuals.begin(), residuals.end(), Gdata->weights_dev.begin(), residualweight.begin(), thrust::multiplies<double>());
	thrust::reduce_by_key(Gdata->molmap.begin(), Gdata->molmap.end(), residualweight.begin(), keys.begin(), sumres.begin(), binary_pred, thrust::plus<double>());
	divideandadd<<<(nummols+127)/128,128,0,stream>>>(x, thrust::raw_pointer_cast(sumres.data()), thrust::raw_pointer_cast(count.data()), nummols);
}
template<typename T>
struct absolute_value
{
	__host__ __device__ T operator()(const T &x) const
	{
	return x < T(0) ? -x : x;
	}
};
__global__
void toDouble(double *newarr, const float *oldarr, int n) {
	int i = blockIdx.x*blockDim.x + threadIdx.x;
	if ( i < n ) {
		newarr[i] = (double) oldarr[i];
	}
}
__global__
void toFloat(float *newarr, double *oldarr, int n) {
	int i = blockIdx.x*blockDim.x + threadIdx.x;
	if ( i < n ) {
		newarr[i] = (float) oldarr[i];
	}
}
void setforceweight(double fw) {
	forceweight = fw;
}
void loaddata(double* xg, int n) {
	long *temparray;
	forceweight = 0.1;
	cnpy::NpyArray arr = cnpy::npy_load("numbers.npy");
	long *numbers = reinterpret_cast<long*>(arr.data);
	for (int i = 0; i < (arr.shape)[0]; i++) {
		switch (i) {
			case 0:
				numframes = (int) numbers[i];
				break;
			case 1:
				nummols = (int) numbers[i];
				break;
			case 2:
				numbondp = (int) numbers[i];
				break;
			case 3:
				numanglep = (int) numbers[i];
				break;
			case 4:
				numdihedp = (int) numbers[i];
				break;
		}
		cout << numbers[i] << endl;
	}
	double *parameters = xg;
	//arr = cnpy::npy_load("parameters.npy");
	//arr = cnpy::npy_load("outputparametersconstrained.npy");
	//double *parameters = reinterpret_cast<double*>(arr.data);
	// for (int i = 0; i < (arr.shape)[0]; i++) {
		// cout << i << " " << parameters[i] << endl;
	// }
	//paramlength = (arr.shape)[0];
	paramlength = n;
	cout << paramlength << endl;
	arr = cnpy::npy_load("energies.npy");
	cout << (arr.shape)[0] << " " << (arr.shape)[1] << endl;
	double *qmE = reinterpret_cast<double*>(arr.data);
	double *mmE = &qmE[(arr.shape)[1]];
	
	arr = cnpy::npy_load("framecount.npy");
	temparray = reinterpret_cast<long*>(arr.data);
	framecount = new int[(arr.shape)[0]];
	for (int i = 0; i < (arr.shape)[0]; i++) {
		framecount[i] = (int) temparray[i];
	}
	arr = cnpy::npy_load("typecounts.npy");
	temparray = reinterpret_cast<long*>(arr.data);
	numbondtypes = new int[(arr.shape)[0]];
	numangletypes = new int[(arr.shape)[0]];
	numdihedtypes = new int[(arr.shape)[0]];
	for (int i = 0; i < (arr.shape)[0]; i++) {
		numbondtypes[i] = (int) temparray[3*i+0];
		numangletypes[i] = (int) temparray[3*i+1];
		numdihedtypes[i] = (int) temparray[3*i+2];
	}
	arr = cnpy::npy_load("btypes.npy");
	temparray = reinterpret_cast<long*>(arr.data);
	btypes = new int[(arr.shape)[0]];
	for (int i = 0; i < (arr.shape)[0]; i++) {
		btypes[i] = (int) temparray[i];
	}
	arr = cnpy::npy_load("atypes.npy");
	temparray = reinterpret_cast<long*>(arr.data);
	atypes = new int[(arr.shape)[0]];
	for (int i = 0; i < (arr.shape)[0]; i++) {
		atypes[i] = (int) temparray[i];
	}
	arr = cnpy::npy_load("dtypes.npy");
	temparray = reinterpret_cast<long*>(arr.data);
	dtypes = new int[(arr.shape)[0]];
	for (int i = 0; i < (arr.shape)[0]; i++) {
		dtypes[i] = (int) temparray[i];
	}
	arr = cnpy::npy_load("bgeom.npy");
	bgeom = reinterpret_cast<double*>(arr.data);
	arr = cnpy::npy_load("ageom.npy");
	ageom = reinterpret_cast<double*>(arr.data);
	arr = cnpy::npy_load("ubgeom.npy");
	ubgeom = reinterpret_cast<double*>(arr.data);
	arr = cnpy::npy_load("dgeom.npy");
	dgeom = reinterpret_cast<double*>(arr.data);

	arr = cnpy::npy_load("nbforces.npy");
	nbforces = reinterpret_cast<double*>(arr.data);
	arr = cnpy::npy_load("minimumcoordinates.npy");
	float *tmparr = reinterpret_cast<float*>(arr.data);
	geometries = new double[(arr.shape)[0]];
	for (int i = 0; i < (arr.shape)[0]; i++) {
		geometries[i] = (double) tmparr[i];
	}
	arr = cnpy::npy_load("minimumtypecounts.npy");
	temparray = reinterpret_cast<long*>(arr.data);
	numminimumbondtypes = new int[(arr.shape)[0]];
	numminimumangletypes = new int[(arr.shape)[0]];
	numminimumdihedtypes = new int[(arr.shape)[0]];
	for (int i = 0; i < (arr.shape)[0]; i++) {
		numminimumbondtypes[i] = (int) temparray[3*i+0];
		numminimumangletypes[i] = (int) temparray[3*i+1];
		numminimumdihedtypes[i] = (int) temparray[3*i+2];
	}
	arr = cnpy::npy_load("minimumbtypes.npy");
	temparray = reinterpret_cast<long*>(arr.data);
	minimumbtypes = new int[(arr.shape)[0]];
	for (int i = 0; i < (arr.shape)[0]; i++) {
		minimumbtypes[i] = (int) temparray[i];
	}
	arr = cnpy::npy_load("minimumatypes.npy");
	temparray = reinterpret_cast<long*>(arr.data);
	minimumatypes = new int[(arr.shape)[0]];
	for (int i = 0; i < (arr.shape)[0]; i++) {
		minimumatypes[i] = (int) temparray[i];
	}
	arr = cnpy::npy_load("minimumdtypes.npy");
	temparray = reinterpret_cast<long*>(arr.data);
	minimumdtypes = new int[(arr.shape)[0]];
	for (int i = 0; i < (arr.shape)[0]; i++) {
		minimumdtypes[i] = (int) temparray[i];
	}
	arr = cnpy::npy_load("minimumnumframes.npy");
	temparray = reinterpret_cast<long*>(arr.data);
	mininumframes = new int[(arr.shape)[0]];
	for (int i = 0; i < (arr.shape)[0]; i++) {
		mininumframes[i] = (int) temparray[i];
	}
	arr = cnpy::npy_load("minimumnumatoms.npy");
	temparray = reinterpret_cast<long*>(arr.data);
	mininumatoms = new int[(arr.shape)[0]];
	for (int i = 0; i < (arr.shape)[0]; i++) {
		mininumatoms[i] = (int) temparray[i];
	}
	mininummols = (arr.shape)[0];
	coordsize = 0;
	framemols = 0;
	for (int i = 0; i < mininummols; i++ ) {
		coordsize += mininumatoms[i] * mininumframes[i];
		framemols += mininumframes[i];
	}
	clock_t time;
	time = clock();
	Gdata = new GPUdata();
	
	cudaMalloc(&devg, paramlength * sizeof(double));
	cudaMalloc(&devx, paramlength * sizeof(double));
	cudaMemcpy(devx, parameters, paramlength * sizeof(double), cudaMemcpyHostToDevice);
	cublasCreate_v2(&cublasHd);
	time = clock() - time;
	cout << "GPU setup took " << ((float)time)/CLOCKS_PER_SEC << " seconds" << endl;
	time = clock(); 
	cout << "About to prepare GPU" << endl;
	prepGPU(qmE, mmE);
	//Some extra preparations involving the weighting factors.
	arr = cnpy::npy_load("weightingfactors.npy");
	double *weightingfactors = reinterpret_cast<double*>(arr.data);
	double *dev_weightfactor;
	cudaMalloc(&dev_weightfactor, nummols * sizeof(double));
	cudaMemcpy(dev_weightfactor, weightingfactors, nummols * sizeof(double), cudaMemcpyHostToDevice);
	reweightbondandangles<<<(numframes+511)/512, 512, 0>>>(thrust::raw_pointer_cast(Gdata->weights_dev.data()), dev_weightfactor,
		thrust::raw_pointer_cast(Gdata->molmap.data()), numframes);
	time = clock() - time;
	cout << "GPU prepared" << endl;
	cudaMalloc(&d_residuals, numframes * sizeof(double));
	resetoffsets(devx, NULL);
	cudaMemcpy(xg, devx, paramlength * sizeof(double), cudaMemcpyDeviceToHost);
	cout << "Copies and precomputation took " << ((float)time)/CLOCKS_PER_SEC << " seconds" << endl;
	time = clock();
	// double f;
	// funcgrad(devx, f, devg, NULL);
	// time = clock() - time;
	// cout << "Objective function and gradient took " << ((float)time)/CLOCKS_PER_SEC << " seconds" << endl;
	// printf("f: %f\n", f);
	//residuals(devx, d_residuals, NULL);
}
//The input to this will be the parameters (the x), and the output will be the gradient for each parameter. The return value will be the objective function value.
double fandg(double* xg, int n) {
	if (n != paramlength) {
		return -1.0;
	}
	cudaMemcpy(devx, xg, paramlength * sizeof(double), cudaMemcpyHostToDevice);
	double f;
	funcgrad(devx, f, devg, NULL);
	cudaMemcpy(xg, devg, paramlength * sizeof(double), cudaMemcpyDeviceToHost);
	return f;
}

void residual(double* resout, int n) {
	n = numframes;
	residuals(devx, d_residuals, NULL);
	cudaMemcpy(resout, d_residuals, numframes * sizeof(double), cudaMemcpyDeviceToHost);
}

void freemem () {
	cublasDestroy_v2(cublasHd);
	cudaFree(devg);
	cudaFree(devx);
	cudaFree(d_residuals);
	delete Gdata;
}

// int main() {
// 	cnpy::NpyArray arr = cnpy::npy_load("numbers.npy");
// 	double *x;
// 	int n = 0;
// 	arr = cnpy::npy_load("parameters.npy");
// 	//arr = cnpy::npy_load("outputparametersconstrained.npy");
// 	x = reinterpret_cast<double*>(arr.data);
// 	// for (int i = 0; i < (arr.shape)[0]; i++) {
// 		// cout << i << " " << parameters[i] << endl;
// 	// }
// 	n = (arr.shape)[0];
// 	loaddata(x, n);
	
// 	for (int i = 0; i < 1; i++) {
// 		printf("%d, %f\n", i, fandg(x, n));
// 	}
// 	return 0;
// }