#ifndef _opt_h
#define _opt_h
typedef double real;
#include <thrust/device_vector.h>
#include <thrust/scan.h>
#include <thrust/reduce.h>
#include <thrust/transform.h>
#include <thrust/functional.h>
#include <thrust/device_ptr.h>
#include <thrust/gather.h>
#include <thrust/fill.h>
#include <thrust/copy.h>
#include <thrust/extrema.h>
#include <thrust/transform_reduce.h>
#include <thrust/system/cuda/execution_policy.h>
#include <thrust/device_vector.h>
#include "cublas_v2.h"
extern int *framecount;
extern int numframes, nummols, numbondp, numanglep, numdihedp, paramlength;

extern int *numbondtypes, *numangletypes, *numdihedtypes;
extern int *btypes, *atypes, *dtypes;
extern double *bgeom, *ageom, *ubgeom, *dgeom;
extern double *targetE;
extern double *weights;
__host__ __device__
inline double radians(double degree) {
	return degree * M_PI / 180;
}
void prepGPU(double *qm, double *mm);
void funcgrad(real* x, real& f, real* g, const cudaStream_t& stream);
//The device vectors cannot be normal global variables because thrust can't deallocate them when main exits.
//Instead, we get this lovely kludge.
class GPUdata {
	public:
	thrust::device_vector<int> framecount_dev;
	thrust::device_vector<int> numbondtypes_dev;
	thrust::device_vector<int> numangletypes_dev;
	thrust::device_vector<int> numdihedtypes_dev;
	thrust::device_vector<int> numminimumbondtypes_dev;
	thrust::device_vector<int> numminimumangletypes_dev;
	thrust::device_vector<int> numminimumdihedtypes_dev;
	thrust::device_vector<int> sumnumbondtypes_dev;
	thrust::device_vector<int> sumnumangletypes_dev;
	thrust::device_vector<int> sumnumdihedtypes_dev;
	thrust::device_vector<int> sumnumminimumbondtypes_dev;
	thrust::device_vector<int> sumnumminimumangletypes_dev;
	thrust::device_vector<int> sumnumminimumdihedtypes_dev;
	thrust::device_vector<int> framebondproduct_dev;
	thrust::device_vector<int> frameangleproduct_dev;
	thrust::device_vector<int> framedihedproduct_dev;
	thrust::device_vector<int> btypes_dev;
	thrust::device_vector<int> atypes_dev;
	thrust::device_vector<int> dtypes_dev;
	thrust::device_vector<double> bgeom_dev;
	thrust::device_vector<double> ageom_dev;
	thrust::device_vector<double> ubgeom_dev;
	thrust::device_vector<double> dgeom_dev;
	thrust::device_vector<double> targetE_dev;
	thrust::device_vector<double> weights_dev;
	thrust::device_vector<double> coord_dev;
	thrust::device_vector<double> force_dev;
	thrust::device_vector<double> nbforce_dev;
	thrust::device_vector<int> minibtypes_dev;
	thrust::device_vector<int> miniatypes_dev;
	thrust::device_vector<int> minidtypes_dev;
	thrust::device_vector<int> molmap;
	thrust::device_vector<int> bondmolmap;
	thrust::device_vector<int> anglemolmap;
	thrust::device_vector<int> dihedmolmap;
	thrust::device_vector<int> frameoffsets;
	thrust::device_vector<int> numatoms_dev;
	thrust::device_vector<int> sumnumatoms_dev;
	thrust::device_vector<int> minimumframecount_dev;
	thrust::device_vector<int> molsize_dev; //Number of atoms * number of frames.
	thrust::device_vector<int> moloffsets;
	thrust::device_vector<int> molbondproduct_dev;
	thrust::device_vector<int> molangleproduct_dev;
	thrust::device_vector<int> moldihedproduct_dev;
	thrust::device_vector<int> minimolmap;
};
extern GPUdata *Gdata;
extern cublasHandle_t cublasHd;
#define gpuErrchk(ans) { gpuAssert((ans), __FILE__, __LINE__); }
inline void gpuAssert(cudaError_t code, const char *file, int line, bool abort=true)
{
   if (code != cudaSuccess) 
   {
      fprintf(stderr,"GPUassert: %s %s %d\n", cudaGetErrorString(code), file, line);
      if (abort) exit(code);
   }
}
#endif