import numpy as np
import cPickle as pickle
import time
import scipy.optimize
iscgenff = False
start = time.clock()
if not iscgenff:
	knownparameters, qmE, mmE, typesmap, geometries, btypes, atypes, dtypes, angletypes = pickle.load( open( "../everything.pkl", "rb" ) )
else:
	knownparameters, qmE, mmE, typesmap, geometries, btypes, atypes, dtypes = pickle.load( open( "../everything-cgenff.pkl", "rb" ) )
end = time.clock()
print "Spent %.2f seconds loading precalculated entries" % (end - start)


avgeomtypes = dict()
avgeomtypes['b'] = dict()
avgeomtypes['a'] = dict()
avgeomtypes['ub'] = dict()
for mol in qmE.keys():
	for k, tk in zip(['b', 'a', 'ub'], ['b','a', 'a']):
		geom = geometries[mol][k]
		types = typesmap[mol][tk]
		for i in range(len(types)):
			t = types[i]
			if t in avgeomtypes[k].keys():
				avgeomtypes[k][t].append(geom[i])
			else:
				avgeomtypes[k][t] = [geom[i]]
#print avgeomtypes
#exit()
bondp = np.zeros(2*len(btypes), dtype=np.float)
for i, k in enumerate(btypes):
	bondp[2*i+0] = knownparameters['bonds'][k][0]
	bondp[2*i+1] = np.mean(np.hstack(avgeomtypes['b'][i]))
anglep = np.zeros(4 * len(atypes), dtype=np.float)
for i, k in enumerate(atypes):
	anglep[4*i+0] = knownparameters['angles'][k][0]
	anglep[4*i+1] = np.mean(np.hstack(avgeomtypes['a'][i]))
	anglep[4*i+3] = np.mean(np.hstack(avgeomtypes['ub'][i]))
	if len(knownparameters['angles'][k]) > 2:
		anglep[4*i+2] = knownparameters['angles'][k][2]
dihedp = np.zeros(5*len(dtypes), dtype=np.float)
for i, k in enumerate(dtypes):
	setels = list(knownparameters['dihedrals'][k])
	for j in range(len(setels)):
		l = int(setels[j][1])
		if l == 6:
			l = 5
		sign = 1
		if setels[j][2] > 0:
			sign = -1
		dihedp[5*i+l-1] = sign * setels[j][0]
def objectivefunction(x, qmE, mmE, geometries, typesmap):
	objstart = time.clock()
	f = 0
	gradf = np.zeros(len(x), dtype=np.float)
	#print x.shape
	boffset = len(qmE.keys())
	aoffset = boffset + 2*len(btypes)
	doffset = aoffset + 4 * len(atypes)
	for mol in qmE.keys():
		weights = 1e-4 * np.exp(-qmE[mol]/4)
		targetE = qmE[mol] - mmE[mol]
		bondedE = np.zeros(len(qmE[mol]), dtype=np.float)
		lgradf = np.zeros((len(x), len(qmE[mol])), dtype=np.float)
		#print lgradf.shape
		for t, vals in zip(typesmap[mol]['b'],geometries[mol]['b']):
			#print vals, x[boffset + 2*t + 1]
			bondedE += x[boffset + 2*t] * np.square(vals - x[boffset + 2*t + 1])
			lgradf[boffset + 2*t+0] += np.square(vals - x[boffset + 2*t + 1])
			lgradf[boffset + 2*t+1] -= 2 * x[boffset + 2*t+0] * (vals - x[boffset + 2*t + 1])
		#print bondedE
		#print targetE
		# if mol == 1:
		# 	exit()
		#print lgradf.shape
		for t, valsA, valsUB in zip(typesmap[mol]['a'], geometries[mol]['a'], geometries[mol]['ub']):
			bondedE += x[aoffset + 4*t] * np.square(np.radians(valsA - x[aoffset + 4*t + 1]))
			lgradf[aoffset + 4*t] += np.square(np.radians(valsA - x[aoffset + 4*t + 1]))
			lgradf[aoffset + 4*t+1] -= 2 * x[aoffset + 4*t] * np.radians(valsA - x[aoffset + 4*t + 1]) * np.pi / 180 #The extra term comes from squaring "radians"
			bondedE += x[aoffset + 4*t+2] * np.square(valsUB - x[aoffset + 4*t + 3])
			lgradf[aoffset + 4*t+2] += np.square(valsUB - x[aoffset + 4*t + 3])
			lgradf[aoffset + 4*t+3] -= 2 * x[aoffset + 4*t+2] * (valsUB - x[aoffset + 4*t + 3])
		#print lgradf.shape
		for t, vals in zip(typesmap[mol]['d'], geometries[mol]['d']):
			rads = np.radians(vals)
			for i, n in enumerate([1,2,3,4,6]):
				tmp = (1+np.cos(n*rads))
				bondedE += x[doffset + 5*t + i] * tmp
				lgradf[doffset + 5*t + i] += tmp
		#print lgradf.shape
		lgradf[mol] = np.ones(len(qmE[mol]))
		residuals = targetE - bondedE - x[mol]
		f += np.sum(weights * np.square(residuals))
		gradf -= np.sum(2 * (weights * residuals) * lgradf, axis=1)
	#f += 1e-2 * np.sum(np.square(x[doffset:]))
	#gradf[doffset:] += 1e-2 * 2 * x[doffset:]
	objend = time.clock()
	print "Took %.2f seconds for objective function evaluation" % (objend - objstart)
	print f, gradf
	#np.save("gradfexample.npy", gradf)
	return (f, gradf)
def func(x):
	return fastobjectivefunction(x)[0]
def funcgrad(x):
	return fastobjectivefunction(x)[1]
import paramfandg
import os
for f in ['ageom', 'atypes', 'bgeom', 'btypes', 'dgeom', 'dtypes', 'energies', 'framecount', 'numbers', 'parameters', 'typecounts', 'ubgeom']:
	os.remove("%s.npy" % f)
	if iscgenff:
		os.symlink("../%s-cgenff.npy" %f, "%s.npy" % f)
	else:
		os.symlink("../%s.npy" %f, "%s.npy" % f)
#fout = open("fastprogress.log", "w")
def fastobjectivefunction(x):
	objstart = time.time()
	g = x.copy()
	f = paramfandg.fandg(g)
	#objend = time.time()
	#fout.write("%.6f %.4f\n" % (f, objend-objstart))
	print f
	return (f, g)
parameters = np.hstack([np.zeros(len(qmE.keys())), bondp, anglep, dihedp])
paramfandg.loaddata(parameters)
# start = time.clock()
# print objectivefunction(parameters, qmE, mmE, geometries, typesmap)
# end = time.clock()
print fastobjectivefunction(parameters)
sizesum = 0
for k in qmE.keys():
	sizesum += len(qmE[k])
res = np.zeros(sizesum)
# print np.mean(paramfandg.residual(len(res)))
# np.save("initialresiduals.npy", paramfandg.residual(len(res)))
# exit()
start = time.clock()
print fastobjectivefunction(parameters)
end = time.clock()
#exit()
# print "Spent %.2f seconds on a function evaluation" % (end - start)
approximatedg = scipy.optimize.approx_fprime(parameters, func, 1e-8)
g = funcgrad(parameters)
print approximatedg.shape
print g.shape
print approximatedg-g
offsetstuff = g[:-(len(dihedp)+len(anglep)+len(bondp))] - approximatedg[:-(len(dihedp)+len(anglep)+len(bondp))]
bondstuff = g[-(len(dihedp)+len(anglep)+len(bondp)):-(len(dihedp)+len(anglep))] - approximatedg[-(len(dihedp)+len(anglep)+len(bondp)):-(len(dihedp)+len(anglep))]
anglestuff =  g[-(len(dihedp)+len(anglep)):-(len(dihedp))] - approximatedg[-(len(dihedp)+len(anglep)):-(len(dihedp))]
dihedralstuff = g[-(len(dihedp)):] - approximatedg[-(len(dihedp)):]
print np.sum(np.square(offsetstuff))
print np.sum(np.square(bondstuff))
print np.sum(np.square(anglestuff))
print np.sum(np.square(dihedralstuff))
print offsetstuff
print bondstuff
print anglestuff
print dihedralstuff
exit()
# def checklocalgrad(i, x, qmE, mmE, geometries, typesmap):
# 	offset = np.zeros(len(x), dtype=np.float)
# 	offset[i] = 1e-4
# 	a, ga = objectivefunction(parameters-offset, qmE, mmE, geometries, typesmap)
# 	b, gb = objectivefunction(parameters+offset, qmE, mmE, geometries, typesmap)
# 	f, g = objectivefunction(parameters, qmE, mmE, geometries, typesmap)
# 	approxg = (b - a) / 2e-4
# 	print i, g[i], approxg, g[i] - approxg
# checklocalgrad(0, parameters, qmE, mmE, geometries, typesmap)
# checklocalgrad(len(qmE.keys()), parameters, qmE, mmE, geometries, typesmap)
# checklocalgrad(len(qmE.keys())+  2*len(knownparameters['bonds']), parameters, qmE, mmE, geometries, typesmap)
# checklocalgrad(len(qmE.keys())+  2*len(knownparameters['bonds'])+4 * len(knownparameters['angles']), parameters, qmE, mmE, geometries, typesmap)
# ivals = np.random.randint(0,len(parameters),size=20)
# for val in ivals:
# 	checklocalgrad(val, parameters, qmE, mmE, geometries, typesmap)
boundslist = []
for i in range(len(qmE.keys())):
	boundslist.append([None, None])
for i in range(len(btypes)):
	boundslist.append([0, 1000])
	boundslist.append([0, 10])
for i in range(len(atypes)):
	boundslist.append([0, 500])
	boundslist.append([0, 180])
	boundslist.append([0, 100])
	boundslist.append([0, 10])
for i in range(5*len(dtypes)):
	if i % 5 == 0: #n=1
		boundslist.append([-5, 3])
	elif i % 5 == 1: #n=2
		boundslist.append([-10,5])
	elif i % 5 == 2: #n=3
		boundslist.append([-5,4])
	elif i % 5 == 3: #n=4
		boundslist.append([-1.5,2.5])
	else:#n=6
		boundslist.append([-1,1])
	#boundslist.append([-10,10])
boffset = len(qmE.keys())
aoffset = boffset + 2*len(btypes)
doffset = aoffset + 4 * len(atypes)
tightboundslist = []
for i in range(len(qmE.keys())):
	tightboundslist.append([None, None])
for i in range(len(btypes)):
	if parameters[boffset+2*i] != 0:
		tightboundslist.append([0, 1000])
		tightboundslist.append([0, 10])
	else:
		tightboundslist.append([0,0])
		tightboundslist.append([0,0])
for i in range(len(atypes)):
	if parameters[aoffset+4*i] != 0:
		tightboundslist.append([0, 500])
		tightboundslist.append([0, 180])
	else:
		tightboundslist.append([0,0])
		tightboundslist.append([0,0])
	if parameters[aoffset+4*i+2] != 0:
		tightboundslist.append([0, 100])
		tightboundslist.append([0, 10])
	else:
		tightboundslist.append([0,0])
		tightboundslist.append([0,0])
for i in range(5*len(dtypes)):
	if parameters[doffset+i] != 0:
		if i % 5 == 0: #n=1
			tightboundslist.append([-5, 3])
		elif i % 5 == 1: #n=2
			tightboundslist.append([-10,5])
		elif i % 5 == 2: #n=3
			tightboundslist.append([-5,4])
		elif i % 5 == 3: #n=4
			tightboundslist.append([-1.5,2.5])
		else:#n=6
			tightboundslist.append([-1,1])
	else:
		tightboundslist.append([0,0])
counter = 0
itstart = time.clock()
start = time.clock()
print len(parameters)
sizesum = 0
for k in qmE.keys():
	sizesum += len(qmE[k])
res = np.zeros(sizesum)
print paramfandg.residual(len(res))
#exit()
print len(boundslist)
optionsdict = {'maxiter': 200000, 'maxfun': 400000}
result = scipy.optimize.minimize(fastobjectivefunction, parameters, jac=True, bounds=boundslist,options=optionsdict)

np.save("unrestrictedresiduals.npy", paramfandg.residual(len(res)))
end = time.clock()
print "Optimization took %.2f seconds" % (end-start)
print result
fout.close()
fout = open("fastprogress2.log", "w")
np.save("unrestrictedsolution.npy", result.x)
start = time.clock()
result2 = scipy.optimize.minimize(fastobjectivefunction, parameters, jac=True,bounds=tightboundslist,options=optionsdict)
end = time.clock()
print "Optimization took %.2f seconds" % (end-start)
print result2
fout.close()
np.save("restrictedsolution.npy", result2.x)
np.save("restrictedresiduals.npy", paramfandg.residual(len(res)))

bondp = np.zeros(2*len(btypes), dtype=np.float)
for i, k in enumerate(btypes):
	bondp[2*i+0] = knownparameters['bonds'][k][0]
	bondp[2*i+1] = knownparameters['bonds'][k][1]
anglep = np.zeros(4 * len(atypes), dtype=np.float)
for i, k in enumerate(atypes):
	anglep[4*i+0] = knownparameters['angles'][k][0]
	anglep[4*i+1] = knownparameters['angles'][k][1]
	if len(knownparameters['angles'][k]) > 2:
		anglep[4*i+2] = knownparameters['angles'][k][2]
		anglep[4*i+3] = knownparameters['angles'][k][3]
dihedp = np.zeros(5*len(dtypes), dtype=np.float)
for i, k in enumerate(dtypes):
	setels = list(knownparameters['dihedrals'][k])
	for j in range(len(setels)):
		l = int(setels[j][1])
		if l == 6:
			l = 5
		sign = 1
		if setels[j][2] > 0:
			sign = -1
		dihedp[5*i+l-1] = sign * setels[j][0]

parameters = np.hstack([np.zeros(len(qmE.keys())), bondp, anglep, dihedp])
fout = open("fastprogress3.log", "w")
result = scipy.optimize.minimize(fastobjectivefunction, parameters, jac=True, bounds=boundslist,options=optionsdict)

np.save("unrestrictedresiduals2.npy", paramfandg.residual(len(res)))
end = time.clock()
print "Optimization took %.2f seconds" % (end-start)
print result
fout.close()
fout = open("fastprogress4.log", "w")
np.save("unrestrictedsolution2.npy", result.x)
start = time.clock()
result2 = scipy.optimize.minimize(fastobjectivefunction, parameters, jac=True,bounds=tightboundslist,options=optionsdict)
end = time.clock()
print "Optimization took %.2f seconds" % (end-start)
print result2
fout.close()
np.save("restrictedsolution2.npy", result2.x)
np.save("restrictedresiduals2.npy", paramfandg.residual(len(res)))