import numpy as np
import cPickle as pickle
import time
import scipy.optimize
iscgenff = False
start = time.clock()
if not iscgenff:
	knownparameters, qmE, mmE, typesmap, geometries, btypes, atypes, dtypes, angletypes = pickle.load( open( "../everything.pkl", "rb" ) )
else:
	knownparameters, qmE, mmE, typesmap, geometries, btypes, atypes, dtypes = pickle.load( open( "../everything-cgenff.pkl", "rb" ) )
end = time.clock()
print "Spent %.2f seconds loading precalculated entries" % (end - start)
print len(qmE)
enterms = 0
for k in qmE:
	enterms += len(qmE[k])
print enterms


avgeomtypes = dict()
avgeomtypes['b'] = dict()
avgeomtypes['a'] = dict()
avgeomtypes['ub'] = dict()
for mol in qmE.keys():
	for k, tk in zip(['b', 'a', 'ub'], ['b','a', 'a']):
		geom = geometries[mol][k]
		types = typesmap[mol][tk]
		for i in range(len(types)):
			t = types[i]
			if t in avgeomtypes[k].keys():
				avgeomtypes[k][t].append(geom[i])
			else:
				avgeomtypes[k][t] = [geom[i]]
#print avgeomtypes
#exit()

bondp = np.zeros(2*len(btypes), dtype=np.float)
for i, k in enumerate(btypes):
	bondp[2*i+0] = knownparameters['bonds'][k][0]
	bondp[2*i+1] = np.mean(np.hstack(avgeomtypes['b'][i]))
anglep = np.zeros(4 * len(atypes), dtype=np.float)
for i, k in enumerate(atypes):
	anglep[4*i+0] = knownparameters['angles'][k][0]
	anglep[4*i+1] = np.mean(np.hstack(avgeomtypes['a'][i]))
	anglep[4*i+3] = np.mean(np.hstack(avgeomtypes['ub'][i]))
	if len(knownparameters['angles'][k]) > 2:
		anglep[4*i+2] = knownparameters['angles'][k][2]
dihedp = np.zeros(5*len(dtypes), dtype=np.float)
for i, k in enumerate(dtypes):
	setels = list(knownparameters['dihedrals'][k])
	for j in range(len(setels)):
		l = int(setels[j][1])
		if l == 6:
			l = 5
		sign = 1
		if setels[j][2] > 0:
			sign = -1
		dihedp[5*i+l-1] = sign * setels[j][0]
def objectivefunction(x, qmE, mmE, geometries, typesmap):
	objstart = time.clock()
	f = 0
	gradf = np.zeros(len(x), dtype=np.float)
	#print x.shape
	boffset = len(qmE.keys())
	aoffset = boffset + 2*len(btypes)
	doffset = aoffset + 4 * len(atypes)
	for k, mol in enumerate(qmE.keys()):
		weights = 1e-4 * np.exp(-qmE[mol]/4)
		targetE = qmE[mol] - mmE[mol]
		bondedE = np.zeros(len(qmE[mol]), dtype=np.float)
		lgradf = np.zeros((len(x), len(qmE[mol])), dtype=np.float)
		#print lgradf.shape
		for t, vals in zip(typesmap[mol]['b'],geometries[mol]['b']):
			#print vals, x[boffset + 2*t + 1]
			bondedE += x[boffset + 2*t] * np.square(vals - x[boffset + 2*t + 1])
			lgradf[boffset + 2*t+0] += np.square(vals - x[boffset + 2*t + 1])
			lgradf[boffset + 2*t+1] -= 2 * x[boffset + 2*t+0] * (vals - x[boffset + 2*t + 1])
		#print bondedE
		#print targetE
		# if mol == 1:
		# 	exit()
		#print lgradf.shape
		for t, valsA, valsUB in zip(typesmap[mol]['a'], geometries[mol]['a'], geometries[mol]['ub']):
			bondedE += x[aoffset + 4*t] * np.square(np.radians(valsA - x[aoffset + 4*t + 1]))
			lgradf[aoffset + 4*t] += np.square(np.radians(valsA - x[aoffset + 4*t + 1]))
			lgradf[aoffset + 4*t+1] -= 2 * x[aoffset + 4*t] * np.radians(valsA - x[aoffset + 4*t + 1]) * np.pi / 180 #The extra term comes from squaring "radians"
			bondedE += x[aoffset + 4*t+2] * np.square(valsUB - x[aoffset + 4*t + 3])
			lgradf[aoffset + 4*t+2] += np.square(valsUB - x[aoffset + 4*t + 3])
			lgradf[aoffset + 4*t+3] -= 2 * x[aoffset + 4*t+2] * (valsUB - x[aoffset + 4*t + 3])
		#print lgradf.shape
		for t, vals in zip(typesmap[mol]['d'], geometries[mol]['d']):
			rads = np.radians(vals)
			for i, n in enumerate([1,2,3,4,6]):
				tmp = (1+np.cos(n*rads))
				bondedE += x[doffset + 5*t + i] * tmp
				lgradf[doffset + 5*t + i] += tmp
		#print lgradf.shape
		lgradf[mol] = np.ones(len(qmE[mol]))
		residuals = targetE - bondedE - x[k]
		print weights * np.square(residuals)
		if np.any(np.isnan(residuals)):
			print mol
			print bondedE
			print targetE
			print x[k]
			exit()
		f += np.sum(weights * np.square(residuals))
		gradf -= np.sum(2 * (weights * residuals) * lgradf, axis=1)
	#f += 1e-2 * np.sum(np.square(x[doffset:]))
	#gradf[doffset:] += 1e-2 * 2 * x[doffset:]
	objend = time.clock()
	print "Took %.2f seconds for objective function evaluation" % (objend - objstart)
	print f, gradf
	#np.save("gradfexample.npy", gradf)
	return (f, gradf)
def func(x, qmE, mmE, geometries, typesmap):
	return objectivefunction(x,qmE,mmE,geometries,typesmap)[0]
def funcgrad(x, qmE, mmE, geometries, typesmap):
	return objectivefunction(x,qmE,mmE,geometries,typesmap)[1]
import paramfandg
import os
for f in ['ageom', 'atypes', 'bgeom', 'btypes', 'dgeom', 'dtypes', 'energies', 'framecount', 'numbers', 'parameters', 'typecounts', 'ubgeom']:
	os.remove("%s.npy" % f)
	if iscgenff:
		os.symlink("../%s-cgenff.npy" %f, "%s.npy" % f)
	else:
		os.symlink("../%s.npy" %f, "%s.npy" % f)
fout = open("altprogress.log", "w")

parameters = np.hstack([np.zeros(len(qmE.keys())), bondp, anglep, dihedp])
def anglesum(x, i):
	#print np.sum(x*matrixhelper[i]) - 360
	return np.sum(x*matrixhelper[i]) - 360
def anglejac(x,i):
	return matrixhelper[i]
counter = 0
matrixhelper = []
mcolcounter = []
conslist = []
for atype in angletypes:
	#print atype
	tmparr = np.zeros(len(parameters))
	for a in atype:
		tmparr[len(qmE.keys())+len(bondp)+1+4*a] += 1
	matrixhelper.append(tmparr)		
	matrix = np.vstack(matrixhelper)

	if np.linalg.matrix_rank(matrix) == len(conslist)+1:
		equalityconstraint = dict()
		equalityconstraint['type'] = 'eq'
		equalityconstraint['fun'] = anglesum
		equalityconstraint['jac'] = anglejac
		equalityconstraint['args'] = [(counter)]
		conslist.append(equalityconstraint)
		counter += 1
		#print counter
	else:
		matrixhelper.pop()
print conslist
print len(conslist)
matrixhelper = np.vstack(matrixhelper)
def keep360(x):
	diff = np.sum(matrixhelper * x, axis=1) - 360
	f = np.sum(np.square(diff))
	g = np.sum(matrixhelper.T * diff * 2,axis=1)
	f *= 1000
	g *= 1000
	return (f,g)
def fastobjectivefunction(x):
	objstart = time.time()
	g = x.copy()
	f = paramfandg.fandg(g)
	f2,g2 = keep360(x)
	f+=f2
	g+=g2
	objend = time.time()
	fout.write("%.6f %.4f\n" % (f, objend-objstart))
	print f, f2, objend-objstart
	return (f, g)
#exit()
#print objectivefunction(parameters, qmE, mmE, geometries, typesmap)
paramfandg.loaddata(parameters)
# start = time.clock()

# end = time.clock()
print fastobjectivefunction(parameters)
#exit()
#exit()
sizesum = 0
for k in qmE.keys():
	sizesum += len(qmE[k])
res = np.zeros(sizesum)
# print np.mean(paramfandg.residual(len(res)))
# np.save("initialresiduals.npy", paramfandg.residual(len(res)))
# exit()
start = time.clock()
print fastobjectivefunction(parameters)
end = time.clock()

#exit()
# print "Spent %.2f seconds on a function evaluation" % (end - start)
# #print scipy.optimize.check_grad(func, funcgrad, parameters, qmE, mmE, geometries, typesmap)
# def checklocalgrad(i, x, qmE, mmE, geometries, typesmap):
# 	offset = np.zeros(len(x), dtype=np.float)
# 	offset[i] = 1e-4
# 	a, ga = objectivefunction(parameters-offset, qmE, mmE, geometries, typesmap)
# 	b, gb = objectivefunction(parameters+offset, qmE, mmE, geometries, typesmap)
# 	f, g = objectivefunction(parameters, qmE, mmE, geometries, typesmap)
# 	approxg = (b - a) / 2e-4
# 	print i, g[i], approxg, g[i] - approxg
# checklocalgrad(0, parameters, qmE, mmE, geometries, typesmap)
# checklocalgrad(len(qmE.keys()), parameters, qmE, mmE, geometries, typesmap)
# checklocalgrad(len(qmE.keys())+  2*len(knownparameters['bonds']), parameters, qmE, mmE, geometries, typesmap)
# checklocalgrad(len(qmE.keys())+  2*len(knownparameters['bonds'])+4 * len(knownparameters['angles']), parameters, qmE, mmE, geometries, typesmap)
# ivals = np.random.randint(0,len(parameters),size=20)
# for val in ivals:
# 	checklocalgrad(val, parameters, qmE, mmE, geometries, typesmap)


boffset = len(qmE.keys())
aoffset = boffset + 2*len(btypes)
doffset = aoffset + 4 * len(atypes)
tightboundslist = []
for i in range(len(qmE.keys())):
	tightboundslist.append([None, None])
for i in range(len(btypes)):
	if parameters[boffset+2*i] != 0:
		tightboundslist.append([0, 1000])
		tightboundslist.append([bondp[2*i+1]*.9, bondp[2*i+1]*1.1])
	else:
		tightboundslist.append([0,0])
		tightboundslist.append([bondp[2*i+1]*.9, bondp[2*i+1]*1.1])
for i in range(len(atypes)):
	if parameters[aoffset+4*i] != 0:
		tightboundslist.append([0, 500])
		tightboundslist.append([anglep[4*i+1]-5, anglep[4*i+1]+5])
	else:
		tightboundslist.append([0,0])
		tightboundslist.append([anglep[4*i+1]-5, anglep[4*i+1]+5])
	if parameters[aoffset+4*i+2] != 0:
		tightboundslist.append([0, 500])
		tightboundslist.append([anglep[4*i+3]*.9, anglep[4*i+3]*1.1])
	else:
		tightboundslist.append([0,0])
		tightboundslist.append([anglep[4*i+3]*.9, anglep[4*i+3]*1.1])
for i in range(5*len(dtypes)):
	if parameters[doffset+i] != 0:
		if parameters[doffset+i] > 0:
			if i % 5 == 0: #n=1
				tightboundslist.append([0, min(3,dihedp[i] * 3)])
			elif i % 5 == 1: #n=2
				tightboundslist.append([0, min(5,dihedp[i] * 3)])
			elif i % 5 == 2: #n=3
				tightboundslist.append([0, min(4,dihedp[i] * 3)])
			elif i % 5 == 3: #n=4
				tightboundslist.append([0, min(2.5,dihedp[i] * 3)])
			else:#n=6
				tightboundslist.append([0, min(1,dihedp[i] * 3)])
		else:
			if i % 5 == 0: #n=1
				tightboundslist.append([max(-5, dihedp[i] * 3), 0])
			elif i % 5 == 1: #n=2
				tightboundslist.append([max(-10, dihedp[i] * 3),0])
			elif i % 5 == 2: #n=3
				tightboundslist.append([max(-5, dihedp[i] * 3),0])
			elif i % 5 == 3: #n=4
				tightboundslist.append([max(-1.5, dihedp[i] * 3),0])
			else:#n=6
				tightboundslist.append([max(-1, dihedp[i] * 3),0])
	else:
		tightboundslist.append([0,0])

bondsonly = []
for i in range(len(qmE.keys())):
	bondsonly.append([None, None])
for i in range(len(btypes)):
	if parameters[boffset+2*i] != 0:
		bondsonly.append([0, 1000])
		bondsonly.append([bondp[2*i+1]*.9, bondp[2*i+1]*1.1])
	else:
		bondsonly.append([0,0])
		bondsonly.append([bondp[2*i+1]*.9, bondp[2*i+1]*1.1])
for i in range(len(atypes)):
	if parameters[aoffset+4*i] != 0:
		bondsonly.append([0, 500])
		bondsonly.append([anglep[4*i+1]-5, anglep[4*i+1]+5])
	else:
		bondsonly.append([0,0])
		bondsonly.append([anglep[4*i+1]-5, anglep[4*i+1]+5])
	if parameters[aoffset+4*i+2] != 0:
		bondsonly.append([0, 500])
		bondsonly.append([anglep[4*i+3]*.9, anglep[4*i+3]*1.1])
	else:
		bondsonly.append([0,0])
		bondsonly.append([anglep[4*i+3]*.9, anglep[4*i+3]*1.1])
for i in range(5*len(dtypes)):
	bondsonly.append([parameters[doffset+i],parameters[doffset+i]])

counter = 0
itstart = time.clock()
start = time.clock()
print len(parameters)
sizesum = 0
for k in qmE.keys():
	sizesum += len(qmE[k])
res = np.zeros(sizesum)
print paramfandg.residual(len(res))
#exit()
optionsdict = {'maxiter': 20000, 'maxfun': 40000}
print "I'm now here!"
result = scipy.optimize.minimize(fastobjectivefunction, parameters, jac=True, bounds=bondsonly,options=optionsdict)
parameters = result.x
end = time.clock()
print "Optimization took %.2f seconds" % (end-start)
fout.write("Bond-only optimization finished. Now resetting for dihedral only-optimization.\n")
print result
dihedonly = []
for i in range(len(qmE.keys())):
	dihedonly.append([None, None])
for i in range(len(btypes)):
	dihedonly.append([parameters[boffset+2*i],parameters[boffset+2*i]])
	dihedonly.append([parameters[boffset+(2*i)+1],parameters[boffset+(2*i)+1]])
for i in range(len(atypes)):
	for j in range(4):
		dihedonly.append([parameters[aoffset+4*i+j], parameters[aoffset+4*i+j]])
for i in range(5*len(dtypes)):
	if parameters[doffset+i] != 0:
		if parameters[doffset+i] > 0:
			if i % 5 == 0: #n=1
				dihedonly.append([0, min(3,dihedp[i] * 3)])
			elif i % 5 == 1: #n=2
				dihedonly.append([0, min(5,dihedp[i] * 3)])
			elif i % 5 == 2: #n=3
				dihedonly.append([0, min(4,dihedp[i] * 3)])
			elif i % 5 == 3: #n=4
				dihedonly.append([0, min(2.5,dihedp[i] * 3)])
			else:#n=6
				dihedonly.append([0, min(1,dihedp[i] * 3)])
		else:
			if i % 5 == 0: #n=1
				dihedonly.append([max(-5, dihedp[i] * 3), 0])
			elif i % 5 == 1: #n=2
				dihedonly.append([max(-10, dihedp[i] * 3),0])
			elif i % 5 == 2: #n=3
				dihedonly.append([max(-5, dihedp[i] * 3),0])
			elif i % 5 == 3: #n=4
				dihedonly.append([max(-1.5, dihedp[i] * 3),0])
			else:#n=6
				dihedonly.append([max(-1, dihedp[i] * 3),0])
	else:
		dihedonly.append([0,0])
result = scipy.optimize.minimize(fastobjectivefunction, parameters, jac=True, bounds=dihedonly,options=optionsdict)
parameters = result.x
fout.write("Dihedral-only optimization finished. Setting up for additional optimization..\n")
for n in range(5):
	bondsonly = []
	for i in range(len(qmE.keys())):
		bondsonly.append([None, None])
	for i in range(len(btypes)):
		if parameters[boffset+2*i] != 0:
			bondsonly.append([0, 1000])
			bondsonly.append([bondp[2*i+1]*.9, bondp[2*i+1]*1.1])
		else:
			bondsonly.append([0,0])
			bondsonly.append([bondp[2*i+1]*.9, bondp[2*i+1]*1.1])
	for i in range(len(atypes)):
		if parameters[aoffset+4*i] != 0:
			bondsonly.append([0, 500])
			bondsonly.append([anglep[4*i+1]-5, anglep[4*i+1]+5])
		else:
			bondsonly.append([0,0])
			bondsonly.append([anglep[4*i+1]-5, anglep[4*i+1]+5])
		if parameters[aoffset+4*i+2] != 0:
			bondsonly.append([0, 500])
			bondsonly.append([anglep[4*i+3]*.9, anglep[4*i+3]*1.1])
		else:
			bondsonly.append([0,0])
			bondsonly.append([anglep[4*i+3]*.9, anglep[4*i+3]*1.1])
	for i in range(5*len(dtypes)):
		bondsonly.append([parameters[doffset+i],parameters[doffset+i]])
	result = scipy.optimize.minimize(fastobjectivefunction, parameters, jac=True, bounds=bondsonly,options=optionsdict)
	print result
	parameters = result.x
	end = time.clock()
	print "Optimization took %.2f seconds" % (end-start)
	fout.write("Bond-only optimization finished. Now resetting for dihedral only-optimization.\n")
	print result
	dihedonly = []
	for i in range(len(qmE.keys())):
		dihedonly.append([None, None])
	for i in range(len(btypes)):
		dihedonly.append([parameters[boffset+2*i],parameters[boffset+2*i]])
		dihedonly.append([parameters[boffset+(2*i)+1],parameters[boffset+(2*i)+1]])
	for i in range(len(atypes)):
		for j in range(4):
			dihedonly.append([parameters[aoffset+4*i+j], parameters[aoffset+4*i+j]])
	for i in range(5*len(dtypes)):
		if parameters[doffset+i] != 0:
			if parameters[doffset+i] > 0:
				if i % 5 == 0: #n=1
					dihedonly.append([0, min(3,dihedp[i] * 3)])
				elif i % 5 == 1: #n=2
					dihedonly.append([0, min(5,dihedp[i] * 3)])
				elif i % 5 == 2: #n=3
					dihedonly.append([0, min(4,dihedp[i] * 3)])
				elif i % 5 == 3: #n=4
					dihedonly.append([0, min(2.5,dihedp[i] * 3)])
				else:#n=6
					dihedonly.append([0, min(1,dihedp[i] * 3)])
			else:
				if i % 5 == 0: #n=1
					dihedonly.append([max(-5, dihedp[i] * 3), 0])
				elif i % 5 == 1: #n=2
					dihedonly.append([max(-10, dihedp[i] * 3),0])
				elif i % 5 == 2: #n=3
					dihedonly.append([max(-5, dihedp[i] * 3),0])
				elif i % 5 == 3: #n=4
					dihedonly.append([max(-1.5, dihedp[i] * 3),0])
				else:#n=6
					dihedonly.append([max(-1, dihedp[i] * 3),0])
		else:
			dihedonly.append([0,0])
	result = scipy.optimize.minimize(fastobjectivefunction, parameters, jac=True, bounds=dihedonly,options=optionsdict)
	print result
	parameters = result.x
	fout.write("Dihedral-only optimization finished. Setting up for additional optimization..\n")
fout.write("Last one started.\n")
result = scipy.optimize.minimize(fastobjectivefunction, parameters, jac=True, bounds=tightboundslist,options=optionsdict)
print result
parameters = result.x
np.save("tightboundsfixdeltaupperresiduals.npy", paramfandg.residual(len(res)))
np.save("tightboundsfixdeltauppersolution.npy", result.x)
fout.write("Tight optimization finished.\n")
fout.close()
