import matplotlib
import numpy as np
from scipy.interpolate import interp1d
from matplotlib.ticker import MaxNLocator
matplotlib.use('Cairo')
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}",r"\renewcommand{\seriesdefault}{\bfdefault}",
r"\usepackage{amsfonts}",r"\usepackage{textcomp}",r"\usepackage{gensymb}", r'\boldmath'] #Using \boldmath makes the axis label numbers bold as well! Looks good.
params = {'text.usetex' : True,
          'font.size' : 14,
          'font.family' : 'lmodern',
          'text.latex.unicode': True,
          'figure.figsize' : (5.6, 4.2), #8, 6 originally. Make it ~30% smaller so the text is ~30% bigger. (Multiply by .7)
          'figure.dpi' : 120
          }
matplotlib.rcParams.update(params)
import matplotlib.pyplot as plt
import glob
import subprocess
removedsteps = [np.array([0])]
for f in ['refineprogress.log']:
	output = np.array(subprocess.check_output("grep removed %s | awk '{print $1}'" % f, shell=True).split(), dtype=np.int)
	removedsteps.append(output)
removedsteps = np.hstack(removedsteps)
#exit()
dtypes = np.load("../dtypes.npy")
print len(dtypes)
print len(np.bincount(dtypes))
totaldparams = 5.0 * len(np.bincount(dtypes))
print totaldparams
#exit()
#exit()
weightfactors = np.load("../weightingfactors.npy")
numframes = np.load("../framecount.npy")
energies = np.load("../energies.npy")
qmenergies = energies[0]
weights = np.empty(len(qmenergies), dtype=np.float)
offset = 0
for i, fcount in enumerate(numframes):
	weights[offset:offset+fcount] = np.exp(-0.25*qmenergies[offset:offset+fcount]) * weightfactors[i]
	offset += fcount
import os
residualsinorder = ['looseboundsresiduals.npy']
parametersinorder = ["looseboundssolution.npy"]
oldmodtime = os.path.getmtime('looseboundsresiduals.npy')
for i in range(len(glob.glob("tightboundsremoval*residuals.npy"))):
	modtime = os.path.getmtime("tightboundsremoval%dresiduals.npy" % i)
	if modtime > oldmodtime:
		residualsinorder.append("tightboundsremoval%dresiduals.npy" % i)
		parametersinorder.append("tightboundsremoval%dsolution.npy" % i)
		oldmodtime = modtime

data = np.empty(len(residualsinorder), dtype=np.float)
data2 = np.empty(len(residualsinorder), dtype=np.float)
import paramfandg

for i, f in enumerate(residualsinorder):
	residuals = np.load(f)
	parameters = np.load(parametersinorder[i])
	paramfandg.loaddata(parameters)
	data2[i] = paramfandg.fandg(parameters)
	paramfandg.freemem()
	data[i] = np.sum(weights * residuals * residuals)
	print i, data[i]
minpoint = np.argmin(data2)
print minpoint
fig, ax =plt.subplots(1,1)
print data.shape
print removedsteps.shape
ax.plot(np.cumsum(removedsteps),data2, 'o-', markersize=2, linewidth=0.5, zorder=1)
ax.scatter(np.cumsum(removedsteps)[minpoint], data2[minpoint], color='r', zorder=2)
ax.annotate("Minimum", xy=(np.cumsum(removedsteps)[minpoint], data2[minpoint]), xytext=(np.cumsum(removedsteps)[minpoint], data2[minpoint]+300), arrowprops=dict(facecolor='black', shrink=0.05),
            horizontalalignment='center')
ax.set_ylabel("Objective Function Value")
ax.set_xlabel("Cumulative Dihedral Terms Removed")
ax.grid(True)
ax.set_axisbelow(True)
fig.tight_layout()
fig.savefig("optimizationprogress.png")
fig, ax =plt.subplots(1,1)
ax.plot(1-np.cumsum(removedsteps)/totaldparams,data2, 'o-', markersize=2, linewidth=0.5, zorder=1)
ax.scatter(1-np.cumsum(removedsteps)[minpoint]/totaldparams, data2[minpoint], color='r', zorder=2)
ax.annotate("Minimum", xy=(1-np.cumsum(removedsteps)[minpoint]/totaldparams, data2[minpoint]), xytext=(1-np.cumsum(removedsteps)[minpoint]/totaldparams, data2[minpoint]+300), arrowprops=dict(facecolor='black', shrink=0.05),
            horizontalalignment='center')
ax.set_ylabel("Objective Function Value")
ax.set_xlabel("Nonzero Dihedral Parameter Fraction")
ax.grid(True)
ax.set_axisbelow(True)
fig.tight_layout()
fig.savefig("optimizationprogress2.png")
fig, ax =plt.subplots(1,1)
ax.plot(totaldparams - np.cumsum(removedsteps),data2, 'o-', markersize=2, linewidth=0.5, zorder=1)
ax.scatter(totaldparams - np.cumsum(removedsteps)[minpoint], data2[minpoint], color='r', zorder=2)
ax.annotate("Minimum", xy=(totaldparams- np.cumsum(removedsteps)[minpoint], data2[minpoint]), xytext=(totaldparams-np.cumsum(removedsteps)[minpoint], data2[minpoint]+300), arrowprops=dict(facecolor='black', shrink=0.05),
            horizontalalignment='center')
ax.set_ylabel("Objective Function Value")
ax.set_xlabel("Cumulative Dihedral Terms")
ax.grid(True)
ax.set_axisbelow(True)
fig.tight_layout()
fig.savefig("optimizationprogress3.png")