import matplotlib
import numpy as np
from scipy.interpolate import InterpolatedUnivariateSpline
from matplotlib.ticker import MaxNLocator
matplotlib.use('Cairo')
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}",r"\renewcommand{\seriesdefault}{\bfdefault}",
r"\usepackage{amsfonts}",r"\usepackage{textcomp}",r"\usepackage{gensymb}", r'\boldmath'] #Using \boldmath makes the axis label numbers bold as well! Looks good.
params = {'text.usetex' : True,
          'font.size' : 14,
          'font.family' : 'lmodern',
          'text.latex.unicode': True,
          'figure.figsize' : (5.6, 4.2), #8, 6 originally. Make it ~30% smaller so the text is ~30% bigger. (Multiply by .7)
          'figure.dpi' : 120
          }
matplotlib.rcParams.update(params)
import matplotlib.pyplot as plt
import glob
import subprocess
from scipy import interpolate
import cPickle as pickle
#exit()
nummol = len(glob.glob("../scanresults/*.js"))
dtypes = np.load("../dtypes.npy")
print dtypes
atypes = np.load("../atypes.npy")
btypes = np.load("../btypes.npy")
doffset = nummol + 2 * len(np.bincount(btypes)) + 4 * len(np.bincount(atypes))
print len(dtypes)
print len(np.bincount(dtypes))
totaldparams = 5.0 * len(np.bincount(dtypes))
#exit()
weightfactors = np.load("../weightingfactors.npy")
numframes = np.load("../framecount.npy")
energies = np.load("../energies.npy")
qmenergies = energies[0]
weights = np.empty(len(qmenergies), dtype=np.float)
filelist = sorted(glob.glob("../scanresults/*.js"))
offset = 0
for i, fcount in enumerate(numframes):
	weights[offset:offset+fcount] = np.exp(-0.25*qmenergies[offset:offset+fcount]) * weightfactors[i]
	offset += fcount

residualsinorder = ['tightboundsresiduals.npy']
for i in range(len(glob.glob("tightboundsaddition*residuals.npy"))):
	residualsinorder.append("tightboundsaddition%dresiduals.npy" % i)
solutioninorder = [np.load('tightboundssolution.npy')]
for i in range(len(glob.glob("tightboundsaddition*solution.npy"))):
	solutioninorder.append(np.load("tightboundsaddition%dsolution.npy" % i))

knownparameters, qmE, mmE, typesmap, geometries, btypes, atypes, dtypes, anglecycles = pickle.load( open( "../everything.pkl", "rb" ) )

data = np.empty((len(residualsinorder), len(numframes)), dtype=np.float)
offsets = np.hstack([np.zeros(1, dtype=np.int),np.cumsum(numframes)])
print offsets
fout = open("additionlog.txt", "w")
for i, f in enumerate(residualsinorder):
	residuals = np.load(f)
	for j in range(len(numframes)):
		data[i][j] = np.sum(weights[offsets[j]:offsets[j+1]] * residuals[offsets[j]:offsets[j+1]] * residuals[offsets[j]:offsets[j+1]])
		#print i, data[i]
	if i > 0 and ((i < 30) or i % 10 == 1):
		print i, np.sum(data[i])-np.sum(data[i-1]), np.amin((data[i]-data[i-1])), np.amax((data[i]-data[i-1]))
	if i > 0 and i < 200:
		print np.nonzero(np.logical_and(solutioninorder[i][doffset:]!=0, solutioninorder[i-1][doffset:]==0 ))
		newidx = np.nonzero(np.logical_and(solutioninorder[i][doffset:]!=0, solutioninorder[i-1][doffset:]==0 ))[0][0]
		n = 1+(newidx % 5)
		if n == 5:
			n = 6
		sortidxs = np.argsort((data[i]-data[i-1]))
		print data[i][sortidxs] - data[i-1][sortidxs]
		fout.write("Step: %d, added n=%d dihedral for %s type. k=%.3f. Total value: %f Biggest changes in %s, %s, %s. \n" % (i, n, dtypes[newidx/5], solutioninorder[i][doffset+newidx], np.sum(data[i])-np.sum(data[i-1]), filelist[sortidxs[0]], filelist[sortidxs[1]], filelist[sortidxs[2]]))
		#exit()
fout.close()
from scipy import optimize
# def doubleexponent(x, a, b, c, d, e, f, g):
# 	return c + a*np.exp(-b*x)+d*np.exp(-e*x)+f*np.exp(-g*x)
# p , e = optimize.curve_fit(doubleexponent, range(len(data)),np.sum(data,axis=1), p0=(100, 1e-3,6000,100,1e-4, 100, 1e-5))

minpoint = np.argmin(data)
print totaldparams, totaldparams - len(data)
fig, axes =plt.subplots(1,1)
ax = axes
x = np.arange(len(data))[:50]
y = np.sum(data,axis=1)[:50]
boxarea = (np.amax(x)-x) * (np.amax(y)-y)
maxarea = np.argmax(boxarea)
print maxarea
ax.plot(1+x,y,'-', zorder=1)
ax.scatter(1+x[maxarea], y[maxarea], color='r', zorder=2,s=10)
#ax.plot(x, piecewise(x, p[0], p[1], p[2],p[3],p[4],p[5],p[6]))
# ax.plot(x, p[2] + p[0] * np.exp(-p[1]*x))
# ax.plot(x, p[2] + p[3] * np.exp(-p[4]*x))
# ax.plot(x, p[2] + p[5] * np.exp(-p[6]*x))
#ax.plot(range(len(data)), f(range(len(data))))
ax.set_ylabel("Objective Function Value")

ax.set_xlabel("Dihedral Terms Added")
ax.grid(True)
ax.set_axisbelow(True)
fig.tight_layout()
fig.savefig("optimizationprogressaddition.png")

exit()
fig, ax =plt.subplots(1,1)
ax.plot(1-np.arange(len(data))/totaldparams,data, 'o-', markersize=2, linewidth=0.5, zorder=1)
ax.scatter(1-np.arange(len(data))[minpoint]/totaldparams, data[minpoint], color='r', zorder=2)
ax.annotate("Minimum", xy=(1-range(len(data))[minpoint]/totaldparams, data[minpoint]), xytext=(1-np.arange(len(data))[minpoint]/totaldparams, data[minpoint]+300), arrowprops=dict(facecolor='black', shrink=0.05),
            horizontalalignment='center')
ax.set_ylabel("Objective Function Value")
ax.set_xlabel("Dihedral Terms Added")
ax.grid(True)
ax.set_axisbelow(True)
fig.tight_layout()
fig.savefig("optimizationprogress2addition.png")
fig, ax =plt.subplots(1,1)
ax.plot(totaldparams - np.arange(len(data)),data, 'o-', markersize=2, linewidth=0.5, zorder=1)
ax.scatter(totaldparams - np.arange(len(data))[minpoint], data[minpoint], color='r', zorder=2)
ax.annotate("Minimum", xy=(totaldparams- range(len(data))[minpoint], data[minpoint]), xytext=(totaldparams-np.arange(len(data))[minpoint], data[minpoint]+300), arrowprops=dict(facecolor='black', shrink=0.05),
            horizontalalignment='center')
ax.set_ylabel("Objective Function Value")
ax.set_xlabel("Dihedral Terms Added")
ax.grid(True)
ax.set_axisbelow(True)
fig.tight_layout()
fig.savefig("optimizationprogress3addition.png")