import glob
import os
from Molecule import Molecule
from VMD import evaltcl
import numpy as np
from atomsel import *
import subprocess
from pyparsing import nestedExpr, originalTextFor
evaltcl("package require forcefieldtoolkit; ::ForceFieldToolKit::DihOpt::init")
def handlescan(filename, reverse=False):
	output = evaltcl("::ForceFieldToolKit::DihOpt::parseGlog %s" % filename)
	nestedBraces1 = nestedExpr('{', '}')
	#nestedBraces2 = originalTextFor(nestedExpr('{', '}'))
	energies = []
	coords = []
	for nb in nestedBraces1.searchString(output):
		#print nb
		energies.append(float(nb[0][2]))
		coordlist = []
		for coord in nb[0][3]:
			arr = np.empty(3, dtype=np.float)
			for j in range(3):
				arr[j] = float(coord[j])
			coordlist.append(arr)
		coorlist = np.vstack(coordlist)
		coords.append(coorlist)
	if len(coords) == 0:
		return np.array([]), np.array([[],[]])
	if reverse:
		coords = np.stack(coords[::-1])
		energies = energies[::-1]
	else:
		coords = np.stack(coords)
	return np.array(energies), coords
def readcoordinatedata (filename, numatoms, numframes):
	fin = open(filename)
	coordinates = np.empty((numframes, numatoms,3), dtype=np.float)
	j = 0
	while j < numframes: #Because the gaussian logfile contains the final coordinates twice, we don't need to read it.
		line = fin.readline()
		if line == '':
			break
		elif line.strip() == "Standard orientation:":
			for i in range(4):
				fin.readline()
			for i in range(numatoms):
				coordinates[j][i] = np.array(fin.readline().split()[3:], dtype=np.float)
			j += 1
	return coordinates
dirs = next(os.walk("../bondedoptimization/missingterms"))[1]
counters = dict()
for d in dirs:
	if d.find('neg') == -1:
		continue 
	path = "../bondedoptimization/missingterms/" + d
	preflength = 1
	while True:
		try:
			int(d[:preflength])
			preflength += 1
		except:
			break
	prefix = int(d[:preflength-1])
	psf = path + "/system.psf"
	if not os.path.isfile(psf):
		print psf
		print "I'm continuing!"
		continue
	mol = Molecule()
	mol.load(psf)
	sel = atomsel("all")
	elist = []
	missinglogs = False
	for i in range(2):
		if i == 0:
			dname = path
		else:
			dname = "../bondedoptimization/missingterms/" + d.replace("neg", "pos")
		logfile = glob.glob(dname + "/*log")
		if len(logfile) == 0:
			missinglogs = True
			continue
		logfile = logfile[0]
		print logfile
		energies, coords = handlescan(logfile, i==0)
		if coords.shape[1] != mol.numAtoms():
			missinglogs = True
			continue
		#print energies, coords
		if i == 1:
			energies = energies[1:]
			coords = coords[1:]
		#exit()
		#s = subprocess.check_output("grep \"EUMP2\" %s | awk '{print $6}' | sed 's/D/E/'" % (logfile), shell=True).strip().split("\n")
		# if s[0] != '':
		# 	energies = np.array(s, dtype=np.float)
		# 	coords = readcoordinatedata(logfile, mol.numAtoms(), len(energies))
		# 	if i == 0:
		# 		energies = energies[::-1]
		# 		coords = coords[::-1]
		# 	elist.append(energies)
		# 	for j in range(len(energies)):
		# 		mol.dupFrame()
		# 		sel.frame = mol.numFrames() - 1
		# 		for k, coordinate in enumerate(['x', 'y', 'z']):
		# 			sel.set(coordinate, coords[j,:,k])
		# else:
		# 	print "Why is this empty?"
		# 	print path, psf, subdirs, subd, dname
		# 	exit()
		for j in range(len(energies)):
			mol.dupFrame()
			sel.frame = mol.numFrames() - 1
			for k, coordinate in enumerate(['x', 'y', 'z']):
				sel.set(coordinate, coords[j,:,k])
		elist.append(energies)
	print len(elist)
	if len(elist) == 0 and not missinglogs:
		exit()
	if len(elist) == 0:
		continue
	if prefix in counters.keys():
		counters[prefix] += 1
	else:
		counters[prefix] = 1
	energies = np.hstack(elist)
	energies -= np.amin(energies)
	#energies *= 627.509469
	np.save("scanresults/dimer-%d-%d.npy" % (prefix, counters[prefix]), energies)
	mol.save("scanresults/dimer-%d-%d.js" % (prefix, counters[prefix]))
	mol.delete()

dirs = next(os.walk("../../monomers/bondopt"))[1]
for d in dirs:
	path = "../../monomers/bondopt/" + d
	print path
	psftail = d
	while psftail[-2:] in ['rr', 'rs', 'ss', 'sr']:
		psftail = psftail[:-2]
	psf = "../chargegroups/psfs/%s.psf" % psftail
	print psf
	mol = Molecule()
	mol.load(psf)
	subdirs = glob.glob(path + "/*pos")
	print subdirs
	for subd in subdirs:
		sel = atomsel("all")
		elist = []
		# for i in range(2):
		# 	if i == 0:
		# 		dname = subd[:-3] + "neg"
		# 	else:
		# 		dname = subd
		# 	print dname
		# 	logfile = glob.glob(dname + "/*log")[0]
		# 	print logfile
		# 	s = subprocess.check_output("grep \"EUMP2\" %s | awk '{print $6}' | sed 's/D/E/'" % (logfile), shell=True).strip().split("\n")
		# 	if s[0] != '':
		# 		energies = np.array(s, dtype=np.float)
		# 		coords = readcoordinatedata(logfile, mol.numAtoms(), len(energies))
		# 		if i == 0:
		# 			energies = energies[::-1]
		# 			coords = coords[::-1]
		# 		elist.append(energies)
		# 		for j in range(len(energies)):
		# 			mol.dupFrame()
		# 			sel.frame = mol.numFrames() - 1
		# 			for k, coordinate in enumerate(['x', 'y', 'z']):
		# 				sel.set(coordinate, coords[j,:,k])
		# 	else:
		# 		print "Why is this empty?"
		# 		print path, psf, subdirs, subd, dname
		# 		exit()
		for i in range(2):
			if i == 0:
		 		dname = subd[:-3] + "neg"
			else:
		 		dname = subd
			logfile = glob.glob(dname + "/*log")[0]
			print logfile
			energies, coords = handlescan(logfile, i==0)
			if i == 1:
				energies = energies[1:]
				coords = coords[1:]
			#exit()
			#s = subprocess.check_output("grep \"EUMP2\" %s | awk '{print $6}' | sed 's/D/E/'" % (logfile), shell=True).strip().split("\n")
			# if s[0] != '':
			# 	energies = np.array(s, dtype=np.float)
			# 	coords = readcoordinatedata(logfile, mol.numAtoms(), len(energies))
			# 	if i == 0:
			# 		energies = energies[::-1]
			# 		coords = coords[::-1]
			# 	elist.append(energies)
			# 	for j in range(len(energies)):
			# 		mol.dupFrame()
			# 		sel.frame = mol.numFrames() - 1
			# 		for k, coordinate in enumerate(['x', 'y', 'z']):
			# 			sel.set(coordinate, coords[j,:,k])
			# else:
			# 	print "Why is this empty?"
			# 	print path, psf, subdirs, subd, dname
			# 	exit()
			for j in range(len(energies)):
				mol.dupFrame()
				sel.frame = mol.numFrames() - 1
				for k, coordinate in enumerate(['x', 'y', 'z']):
					sel.set(coordinate, coords[j,:,k])
			elist.append(energies)
		if psftail in counters.keys():
			counters[psftail] += 1
		else:
			counters[psftail] = 1
		energies = np.hstack(elist)
		energies -= np.amin(energies)
		#energies *= 627.509469
		np.save("scanresults/%s-%d.npy" % (psftail, counters[psftail]), energies)
		mol.save("scanresults/%s-%d.js" % (psftail, counters[psftail]))
		mol.delFrame()
	mol.delete()
dirs = next(os.walk("../../monomers/dihedscan"))[1]
for d in dirs:
	path = "../../monomers/dihedscan/" + d
	print path
	psftail = d
	while psftail[-2:] in ['rr', 'rs', 'ss', 'sr']:
		psftail = psftail[:-2]
	psf = "../chargegroups/psfs/%s.psf" % psftail
	print psf
	mol = Molecule()
	mol.load(psf)
	subdirs = glob.glob(path + "/*pos")
	print subdirs
	for subd in subdirs:
		sel = atomsel("all")
		elist = []
		# for i in range(2):
		# 	if i == 0:
		# 		dname = subd[:-4] + ".neg"
		# 	else:
		# 		dname = subd
		# 	print dname
		# 	logfile = glob.glob(dname + "/*log")[0]
		# 	print logfile
		# 	s = subprocess.check_output("grep \"EUMP2\" %s | awk '{print $6}' | sed 's/D/E/'" % (logfile), shell=True).strip().split("\n")
		# 	if s[0] != '':
		# 		energies = np.array(s, dtype=np.float)
		# 		coords = readcoordinatedata(logfile, mol.numAtoms(), len(energies))
		# 		if i == 0:
		# 			energies = energies[::-1]
		# 			coords = coords[::-1]
		# 		elist.append(energies)
		# 		for j in range(len(energies)):
		# 			mol.dupFrame()
		# 			sel.frame = mol.numFrames() - 1
		# 			for k, coordinate in enumerate(['x', 'y', 'z']):
		# 				sel.set(coordinate, coords[j,:,k])
		# 	else:
		# 		print "Why is this empty?"
		# 		print path, psf, subdirs, subd, dname
		# 		exit()
		for i in range(2):
			if i == 0:
		 		dname = subd[:-4] + ".neg"
			else:
		 		dname = subd
			logfile = glob.glob(dname + "/*log")[0]
			print logfile
			energies, coords = handlescan(logfile)
			if i == 1:
				energies = energies[1:]
				coords = coords[1:]
			for j in range(len(energies)):
				mol.dupFrame()
				sel.frame = mol.numFrames() - 1
				for k, coordinate in enumerate(['x', 'y', 'z']):
					sel.set(coordinate, coords[j,:,k])
			elist.append(energies)
		if psftail in counters.keys():
			counters[psftail] += 1
		else:
			counters[psftail] = 1
		energies = np.hstack(elist)
		energies -= np.amin(energies)
		#energies *= 627.509469
		np.save("scanresults/%s-%d.npy" % (psftail, counters[psftail]), energies)
		mol.save("scanresults/%s-%d.js" % (psftail, counters[psftail]))
		mol.delFrame()
	mol.delete()
#exit()
dirs = next(os.walk("../../dimers/dihedscan"))[1]
for d in dirs:
	path = "../../dimers/dihedscan/" + d
	print path
	psftail = d
	while psftail[-2:] in ['rr', 'rs', 'ss', 'sr']:
		psftail = psftail[:-2]
	psf = "../chargegroups/psfs/%s.psf" % psftail.lower()
	print psf
	mol = Molecule()
	mol.load(psf)
	subdirs = glob.glob(path + "/*pos")
	print subdirs
	for subd in subdirs:
		sel = atomsel("all")
		elist = []
		# for i in range(2):
		# 	if i == 0:
		# 		dname = subd[:-4] + ".neg"
		# 	else:
		# 		dname = subd
		# 	print dname
		# 	logfile = glob.glob(dname + "/*log")[0]
		# 	print logfile
		# 	s = subprocess.check_output("grep \"EUMP2\" %s | awk '{print $6}' | sed 's/D/E/'" % (logfile), shell=True).strip().split("\n")
		# 	if s[0] != '':
		# 		energies = np.array(s, dtype=np.float)
		# 		coords = readcoordinatedata(logfile, mol.numAtoms(), len(energies))
		# 		if i == 0:
		# 			energies = energies[::-1]
		# 			coords = coords[::-1]
		# 		elist.append(energies)
		# 		for j in range(len(energies)):
		# 			mol.dupFrame()
		# 			sel.frame = mol.numFrames() - 1
		# 			for k, coordinate in enumerate(['x', 'y', 'z']):
		# 				sel.set(coordinate, coords[j,:,k])
		# 	else:
		# 		print "Why is this empty?"
		# 		print path, psf, subdirs, subd, dname
		# 		exit()
		for i in range(2):
			if i == 0:
		 		dname = subd[:-4] + ".neg"
			else:
		 		dname = subd
		 	if len(glob.glob(dname + "/*log")) == 0:
		 		continue
			logfile = glob.glob(dname + "/*log")[0]
			print logfile
			energies, coords = handlescan(logfile)
			if i == 1:
				energies = energies[1:]
				coords = coords[1:]
			for j in range(len(energies)):
				mol.dupFrame()
				sel.frame = mol.numFrames() - 1
				for k, coordinate in enumerate(['x', 'y', 'z']):
					sel.set(coordinate, coords[j,:,k])
			elist.append(energies)
		if len(elist) == 0:
			mol.delFrame()
			continue
		if psftail in counters.keys():
			counters[psftail] += 1
		else:
			counters[psftail] = 1
		energies = np.hstack(elist)
		energies -= np.amin(energies)
		#energies *= 627.509469
		np.save("scanresults/%s-%d.npy" % (psftail, counters[psftail]), energies)
		mol.save("scanresults/%s-%d.js" % (psftail, counters[psftail]))
		mol.delFrame()
	mol.delete()
exit()