#This script has 3 missions:
#1.) load all the gaussian logfiles to make a nice set of adjusted trajectories
#2.) Deduce equivalent atoms, which should have the same charge. The criteria for this should be as follows:
#		a.) Same atomtype
#		b.) Same atomtype graph within 2 bonds
#3.) Calculate all the prefactors and targets.

from Molecule import Molecule
from VMD import evaltcl
import numpy as np
import subprocess
from atomsel import atomsel
import vmdnumpy as vnp
import os
import measure
import glob
import networkx as nx
import networkx.algorithms.isomorphism as iso
import sys
import matplotlib
matplotlib.use("Agg")
import matplotlib.pyplot as plt

numbonds = int(sys.argv[1])
print numbonds
#exit()

def isfloat(f):
	try:
		float(f)
		return True
	except ValueError:
		return False
def readLJparameters(ljdict, filename):
	fin = open(filename)
	lines = fin.readlines()[:-2]
	fin.close()
	dictfill = False
	for i, l in enumerate(lines):
		string = l.strip()
		if len(string):
			idx = string.find("!")
			if idx < 0:
				s = string
			else:
				s = string[:idx]
			if len(s):
				splt = s.split()
				if splt[0] == "NONBONDED":
					dictfill = True
				elif splt[0] == "HBOND":
					dictfill = False
				elif splt[0] == "NBFIX":
					dictfill = False
				elif dictfill and isfloat(splt[2]) and isfloat(splt[3]):
					ljdict[splt[0]] = (float(splt[2]),float(splt[3]))
				#print s, len(s)
def LJterm(R, types):
	total = 0
	for i in range(3):
		wpos = R[-(1+i)]
		wLJprm = LJparams[types[-(1+i)]]
		for j in range(len(R)-3):
			cpos = R[j]
			cLJprm = LJparams[types[j]]
			eps = np.sqrt(wLJprm[0] * cLJprm[0])
			rmin = wLJprm[1] + cLJprm[1]
			r = np.linalg.norm(cpos-wpos)
			total += eps * ((rmin/r)**12-2*(rmin/r)**6)
	return total
def Etermprefactors(R, charges):
	result = np.zeros(len(charges)-3)
	for i in range(3):
		wpos = R[-(1+i)]
		q = charges[-(1+i)]
		r = np.linalg.norm(R[:-3] - wpos, axis=1)
		result += 332.0716 * q / r #The 332.0716 is the coulombic constant in kcal/mol/Angs e^2 according to CHARMM.
	return result
LJparams = dict()
readLJparameters(LJparams, "/home/josh/toppar/toppar_water_ions.str")
readLJparameters(LJparams, "/home/josh/toppar/par_all36_cgenff.prm")
readLJparameters(LJparams, "/projects/LigninParameterization/typing/lignins-newtypes.prm")

print sorted(LJparams.keys())
#exit()

def makefullgraph(mol):
	sel = atomsel("all", molid=int(mol))
	types = sel.get("type")
	idxs = sel.get("index")
	charges = sel.get("charge")
	print types
	#exit()
	G = nx.Graph()
	for j, idx in enumerate(idxs):
		G.add_node(idx, t = types[j], q=charges[j])
	for j, b in enumerate(sel.bonds):
		idx = idxs[j]
		for bidx in b:
			if bidx in idxs and idx < bidx:
				G.add_edge(idx, bidx)
	return G

def generateatomgraphs(mol, counter):
	for i in range(mol.numAtoms()):
		sel = atomsel("withinbonds %d of index %d" % (numbonds,i), molid=int(mol))
		types = sel.get("type")
		charges = sel.get("charge")
		print types
		#exit()
		idxs = sel.get("index")
		G = nx.Graph()
		for j, idx in enumerate(idxs):
			if idx == i:
				G.add_node(idx, t = types[j]+"+", q=charges[j]) #This needs to be marked. There are some pathological problems that can result if this isn't done, with oxygens resulting in the same graph as hydrogens on the same carbon.
			else:
				G.add_node(idx, t = types[j], q=charges[j])
		for j, b in enumerate(sel.bonds):
			idx = idxs[j]
			for bidx in b:
				if bidx in idxs and idx < bidx:
					G.add_edge(idx, bidx)
		atomgraphs[(counter, i)] = G
	#exit()
def guesspsf(psflist, match):
	matchlist = []
	for psf in psflist:
		if psf.find(match) != -1:
			matchlist.append(psf)
	while not len(matchlist):
		match = match[:-2]
		for psf in psflist:
			if psf.find(match) != -1:
				matchlist.append(psf)
	matchlist.sort(key=len)
	return matchlist[0]
def readdipoledata (filename, numatoms):
	fin = open(filename)
	print numatoms
	coordinates = np.empty((numatoms,3), dtype=np.float)
	while True:
		line = fin.readline()
		if line == '':
			break
		elif line.strip() == "Standard orientation:":
			for i in range(4):
				fin.readline()
			for i in range(numatoms):
				coordinates[i] = np.array(fin.readline().split()[3:])
		elif line.strip() == "Dipole moment (field-independent basis, Debye):":
			dipoledata = np.array((fin.readline().strip().split())[1::2], dtype=np.float)
	return coordinates, dipoledata
watmol = Molecule()
watmol.load("../../monomers/onewater.psf")
evaltcl("package require forcefieldtoolkit; package require qmtool; ::QMtool::init_variables ::QMtool")
atomgraphs = dict()
dipolemoments = []
redo = 1
#singlepointenergysum = []
qoffsets = [0]
counter = 0
errlog = open("baddirectories.log", "w")
skiplist = ['trcn', 'feruarab', 'diferulateA', 'diferulateB']
for superd in ['monomers', 'dimers', 'trimers']:
	psflist = glob.glob("../../%s/*psf" % superd)
	for subd in sorted(next(os.walk("../../%s/watopt" % superd))[1]):
		path = "../../%s/watopt/%s" % (superd, subd)
		psfname = guesspsf(psflist, subd)
		print "PY", psfname
		basename = psfname.split("/")[-1][:-4]
		# if basename in skiplist:
		#	continue
		retypedpsf = "../../typing/%s-cgenff.psf" % basename
		if basename.find("dbbl") != -1:
			basename = basename[:-4]
		#print path, guesspsf(psflist, subd)
		mol = Molecule()
		mol.load(retypedpsf)
		qoffsets.append(qoffsets[-1] + mol.numAtoms())
		generateatomgraphs(mol, counter)
		if redo or not os.path.isfile("geom/%d-waterinteractiontrajectory.js" % counter):
			#Deal with the single points.
			singlepointfile = "%s/%s-sp-HF/%s-sp-HF.log" % (path, basename, basename)
			if not os.path.isfile(singlepointfile):
				singlepointfile = "%s/%s-sp-HF/%s-sp-HF.log" % (path, basename.upper(), basename.upper())
			coords, dipole = readdipoledata(singlepointfile, mol.numAtoms())
			dipolemoments.append(dipole)
			mol.dupFrame()
			allsel = atomsel("all")
			for j, coordinate in enumerate(['x', 'y', 'z']):
				allsel.set(coordinate, coords[:,j])
			energysum = 0
			energysum += float(subprocess.check_output("grep \"SCF Done\" %s | awk '{print $5}'" % (singlepointfile), shell=True))
			energysum += float(subprocess.check_output("grep \"SCF Done\" %s/wat-sp/wat-sp.log | awk '{print $5}'" % (path), shell=True))
			#singlepointenergysum.append(energysum)
			mol.save("geom/%d-singlepointtrajs.js" % counter)
			mol.delFrame()
			#Now deal with the other stuff.
			residueidxs = mol.numAtoms() -1
			#Since I was lazy, and didn't remake a psf with the compound + water, doing that here.
			evaltcl("package require topotools; set mol [::TopoTools::mergemols [list %d %d]]; animate write psf merged.psf $mol" % (int(mol),int(watmol)))
			mergemol = Molecule()
			mergemol.load("merged.psf")
			mergesel = atomsel("all")
			nowat = atomsel("not water")
			watersel = atomsel("water")
			waternames = np.array(watersel.get("name"), dtype="S1")
			eidx1 = np.nonzero(waternames == 'O')[0][0]
			wosel = atomsel("noh and water")
			qmintE = []
			dirlist = sorted(os.walk(path).next()[1])
			print dirlist
			for d in dirlist:
				#Single points need to be excluded.
				if d.find("-sp") != -1:
					continue
				#For whatever reason, I managed to construct situations where hydrogens were the acceptor. these are beyond useless.
				elif d.find("-ACC-H") != -1:
					continue
				print d
				atomname = d.split('-')[2]
				newmol = Molecule()
				#Find the energies
				logfile = "%s/%s/%s.log" % (path, d, d)
				print logfile
				if subprocess.call("grep Error %s" % logfile, shell=True) == 0:
					errlog.write("%s %s\n" % (path,d))
					continue #If there was an error, skip this directory.
				energies = np.array(subprocess.check_output("grep \"SCF Done\" %s | awk '{print $5}'" % (logfile), shell=True).strip().split("\n"), dtype=np.float)
				energies -= energysum
				energies *= 627.509469 #This converts hartrees to kcal/mol
				print energies
				
				#Read in the positions
				evaltcl("::QMtool::read_gaussian_log %s %d" % (logfile, int(newmol)))
				sel = atomsel("not name \"X.*\"")
				nowatsel = atomsel("index 0 to %d" % residueidxs)
				newwatsel = atomsel("not name \"X.*\" and index > %d" % residueidxs)
				newwaternames = np.array(newwatsel.get("name"), dtype='S1')
				eidx2 = np.nonzero(newwaternames == 'O')[0][0]
				#print len(nowatsel), len(nowat)
				nowatsel.set("name", nowat.get("name"))
				asel = atomsel("name %s" % atomname)
				#print len(mergesel), len(sel)
				#read_gaussian_log reads in the last position twice. Remove it.
				if len(energies) != newmol.numFrames():
					newmol.delFrame(first=newmol.numFrames()-1, last=newmol.numFrames()-1)
					if len(energies) != newmol.numFrames():
						print d
						print len(energies), newmol.numFrames()
						print evaltcl("puts [llength $::QMtool::scfenergies]")
						print evaltcl("puts $::QMtool::scfenergies")
						newmol.save("test.js")
						exit()
				for f in range(newmol.numFrames()):
					mergemol.dupFrame()
					mergesel.frame = mergemol.numFrames() - 1
					watersel.frame = mergemol.numFrames() - 1
					sel.frame = f
					for j, coordinate in enumerate(['x', 'y', 'z']):
						mergesel.set(coordinate, sel.get(coordinate))
					R = vnp.timestep(mergemol, mergemol.numFrames() -1)
					#Sometimes, the water geometry is screwy. I should fix that.
					if eidx1 != eidx2:
						tmp = np.copy(R[residueidxs+1+eidx1])
						R[residueidxs+1+eidx1] = np.copy(R[residueidxs+1+eidx2])
						R[residueidxs+1+eidx2] = tmp
					# v1 = wogeom[1] - wogeom[0]
					# v2 = wogeom[2] - wogeom[0]
					# v1 /= np.linalg.norm(v1)
					# v2 /= np.linalg.norm(v2)
					# angle = np.degrees(np.arccos(np.dot(v1,v2)))
					# if np.abs(angle-109.5) > 6 and np.abs(angle-37.75)>0.5:
					# 	print f
					# 	print angle
					# 	print logfile
					# 	exit()
					# print R.shape
					# print atomname
					# print len(asel), len(wosel)
					# print residue, combination, d
					#Move each water inward by 0.2 Angstroms
					vec = R[asel.get("index")[0]] - R[wosel.get("index")[0]] #This vector points *FROM* the water *TO* the interacting atom
					# if np.linalg.norm(vec) > 10:
					# 	#This only usually happens when the substrate is charged. We need to toss the data, since it will make things complicated.
					# 	mergemol.delFrame(first=mergemol.numFrames()-1, last=mergemol.numFrames()-1)
					# else:
					vec *= 0.2 / np.linalg.norm(vec)
					# print f
					# print vec
					watersel.moveby(vec)
					qmintE.append(energies[f])
				newmol.delete()
			qmintE = 1.16 * np.hstack(qmintE) #These energies need to be scaled.
			np.save("targetdata/%d-interactionenergytrajectory.npy" % counter, qmintE)
			mergemol.save("geom/%d-waterinteractiontrajectory.js" % counter)
			ljenergies = np.empty(mergemol.numFrames(), dtype=np.float)
			etermfactors = np.empty((mergemol.numFrames(), mergemol.numAtoms()-3), dtype=np.float)
			charges = np.array(atomsel("all", molid=mergemol).get("charge"), dtype=np.float)
			types = atomsel("all", molid=mergemol).get("type")
			#print mergemol.numFrames()
			#print mol.numFrames()
			for f in range(mergemol.numFrames()):
				R = vnp.timestep(mergemol, f)
				ljenergies[f] = LJterm(R, types)
				etermfactors[f] = Etermprefactors(R, charges)
			print ljenergies
			#print etermfactors
			np.save("targetdata/%d-ljenergies.npy" % counter, ljenergies)
			np.save("targetdata/%d-etermprefactors.npy" % counter, etermfactors)
			mergemol.delete()
			#exit()
		counter += 1
#exit()
if len(dipolemoments):
	np.save("targetdata/dipoles.npy", np.vstack(dipolemoments))
	np.save("targetdata/qoffsets.npy", np.array(qoffsets))
errlog.close()
graph = []
nm = iso.categorical_node_match('t', 'unk')

key1 = (0,18)
key2 = (0,20)
print nx.is_isomorphic(atomgraphs[key1], atomgraphs[key2], node_match=nm)
print atomgraphs[key1].nodes()
print atomgraphs[key2].nodes()
G = makefullgraph(1)
attributes = nx.get_node_attributes(G,'t')
attrset = set(attributes.values())
print attrset
colors = np.linspace(0,1,len(attrset))
colorlist = np.zeros(len(attributes))
for i, el in enumerate(attrset):
	print el
	colorlist[np.nonzero(np.array(attributes.values()) == el)[0]] = colors[i]
fig, ax = plt.subplots(1,1)
pos = nx.nx_agraph.graphviz_layout(G)
print colorlist
nx.draw_networkx(G, pos=pos, ax=ax, node_color=colorlist)
plt.axis('off')
fig.savefig("fullgraph.png", bbox_inches='tight')
#exit()
attrfix = [18, 20, 4, 7, 5, 8]
for i, G in enumerate([atomgraphs[key1], atomgraphs[key2], atomgraphs[(0,4)], atomgraphs[(0,7)], atomgraphs[(0,5)], atomgraphs[(0,8)]]):
	fig, ax = plt.subplots(1,1)
	pos = nx.nx_agraph.graphviz_layout(G)
	attributes = nx.get_node_attributes(G,'t')
	attributes[attrfix[i]] = attributes[attrfix[i]][:-1]
	colorlist = np.zeros(len(G.nodes()))
	for j, el in enumerate(attrset):
		nnz = np.nonzero(np.array(attributes.values()) == el)[0]
		if len(nnz):
			colorlist[nnz] = colors[j]
	nx.draw_networkx(G, pos=pos, ax=ax, node_color=colorlist)
	plt.axis('off')
	fig.savefig("isomorph-%d.png" % i, bbox_inches='tight')
#exit()
#gm = iso.GraphMatcher(atomgraphs[key1], atomgraphs[key2], node_match=nm)
#for pair in gm.candidate_pairs_iter():
#	print pair
##help(gm)
#exit()
fout = open("graphmatch.txt", "w")
nm = iso.categorical_node_match('t', 'unk')
for i, key1 in enumerate(sorted(atomgraphs.keys())):
	for j, key2 in enumerate(sorted(atomgraphs.keys())):
		if j > i and nx.is_isomorphic(atomgraphs[key1], atomgraphs[key2], node_match=nm):
			#Add an extra if here. We want to make sure the charges for the atoms being compared are close enough, based on some cutoff.
			types = nx.get_node_attributes(atomgraphs[key1], "t")
			charges = nx.get_node_attributes(atomgraphs[key1], "q")
			for k in atomgraphs[key1].nodes():
				if types[k][-1] == '+':
					q1 = charges[k]
			types = nx.get_node_attributes(atomgraphs[key2], "t")
			charges = nx.get_node_attributes(atomgraphs[key2], "q")
			for k in atomgraphs[key2].nodes():
				if types[k][-1] == '+':
					q2 = charges[k]
			print atomgraphs[key1].nodes()
			print nx.get_node_attributes(atomgraphs[key1], "t")
			print nx.get_node_attributes(atomgraphs[key1], "q")
			print atomgraphs[key2].nodes()
			print nx.get_node_attributes(atomgraphs[key2], "t")
			print nx.get_node_attributes(atomgraphs[key2], "q")
			print q1
			print q2
			#exit()
			if np.abs(q1-q2) < 0.05:
				print "Graphs for " + str(key1) + " and " + str(key2) + " match!"
				fout.write(str(key1) + " " + str(key2) + "\n")
				graph.append(np.array([key1[0], key1[1], key2[0], key2[1]], dtype=np.int))
np.save("graphmatch-%d.npy" % numbonds, np.vstack(graph))
fout.close()

exit()