#This script has 3 missions:
#1.) load all the gaussian logfiles to make a nice set of adjusted trajectories
#2.) Deduce equivalent atoms, which should have the same charge. The criteria for this should be as follows:
#		a.) Same atomtype
#		b.) Same atomtype graph within 2 bonds
#3.) Calculate all the prefactors and targets.

from Molecule import Molecule
from VMD import evaltcl
import numpy as np
import subprocess
from atomsel import atomsel
import vmdnumpy as vnp
import os
import measure
import glob
import networkx as nx
import networkx.algorithms.isomorphism as iso
import time
import scipy.optimize

import sys

numbonds = int(sys.argv[1])
print numbonds

def guesspsf(psflist, match):
	matchlist = []
	for psf in psflist:
		if psf.find(match) != -1:
			matchlist.append(psf)
	while not len(matchlist):
		match = match[:-2]
		for psf in psflist:
			if psf.find(match) != -1:
				matchlist.append(psf)
	matchlist.sort(key=len)
	return matchlist[0]
counter = 0
offsets = np.load("targetdata/qoffsets.npy")
frameoffsets = [0]
dipolemoments = np.load("targetdata/dipoles.npy")
ljenergies = []
qmenergies = []
etermfactors = []
initialcharges = np.zeros(offsets[-1], dtype=np.longdouble)
conslist = []
positions = []
elements = []

excludeidxs = []
skiplist = ['trcn', 'feruarab', 'diferulateA', 'diferulateB']
for superd in ['monomers', 'dimers', 'trimers']:
	psflist = glob.glob("../../%s/*psf" % superd)
	for subd in sorted(next(os.walk("../../%s/watopt" % superd))[1]):
		path = "../../%s/watopt/%s" % (superd, subd)
		psfname = guesspsf(psflist, subd)
		basename = psfname.split("/")[-1][:-4]
		# if basename in skiplist:
		# 	continue
		retypedpsf = "../../typing/%s-cgenff.psf" % basename
		if basename.find("dbbl") != -1:
			basename = basename[:-4]
		#print path, guesspsf(psflist, subd)
		mol = Molecule()
		mol.load(retypedpsf)
		mol.load("geom/%d-singlepointtrajs.js" % counter) #Needed for the dipole calculation.
		evaltcl("package require topotools; topo guessatom element mass")
		masses =np.array(atomsel("all").get("mass"))
		R = vnp.timestep(int(mol), 0)
		com = np.mean(R.T*masses, axis=1)
		positions.append(R-com)
		if basename == 'feruarab':
			sel = atomsel("resname AARB")
			sel.set("charge", [ -0.400000,  0.340000,  0.090000, -0.650000,  0.420000,  0.110000,  0.090000,  0.050000,  0.090000,  0.090000,  0.140000,  0.090000, -0.650000,  0.420000,  0.140000,  0.090000, -0.650000,  0.420000])
		initialcharges[offsets[counter]:offsets[counter+1]] = np.array(atomsel("all").get("charge"))
		elements.append(atomsel("all").get("element"))
		#There are some special cases to consider. Mostly the ferulate linker to xylose.
		excludesel = atomsel("resname AARB and not withinbonds %d of name C5" % (numbonds - 1))
		if len(excludesel) > 0:
			excludeidxs.append(offsets[counter] + np.array(excludesel.get("index")))
		ljenergies.append(np.load("targetdata/%d-ljenergies.npy" % counter)[-1:])
		frameoffsets.append(frameoffsets[-1] + len(ljenergies[-1:]))
		etermfactors.append(np.load("targetdata/%d-etermprefactors.npy" % counter)[-1:])
		qmenergies.append(np.load("targetdata/%d-interactionenergytrajectory.npy" % counter)[-1:])
		#equalityconstraint = dict()
		#equalityconstraint['type'] = 'eq'
		#equalityconstraint['fun'] = lambda x : np.sum(initialcharges[offsets[counter]:offsets[counter+1]]) - np.sum(x[offsets[counter]:offsets[counter+1]])
		#equalityconstraint['jac'] = lambda x : -(np.in1d(np.arange(len(x)), np.arange(offsets[counter],offsets[counter+1]), assume_unique=True))
		#print offsets[counter+1]
		# if np.abs(np.sum(initialcharges[offsets[counter]:offsets[counter+1]]) - np.round(np.sum(initialcharges[offsets[counter]:offsets[counter+1]]))) > 1e-4:
		# 	print "How did this happen???"
		# 	print np.abs(np.sum(initialcharges[offsets[counter]:offsets[counter+1]]) - np.round(np.sum(initialcharges[offsets[counter]:offsets[counter+1]])))
		# 	print np.round(np.sum(initialcharges[offsets[counter]:offsets[counter+1]]))
		# 	exit()
		#conslist.append(equalityconstraint)
		#for q in initialcharges[offsets[counter]:offsets[counter+1]]:
		#	#print q
		#	if np.any(np.abs(np.array([0.09, 0.115])-q) < 1e-4):
		#		boundslist.append((np.round(q, decimals=3), np.round(q, decimals=3)))
		#	else:
		#		boundslist.append((-1,1))
		counter += 1
print excludeidxs
elements = np.hstack(elements)
print elements
#exit()
#exit()
#if numbonds == 3:
#	initialcharges = np.load("targetdata/bigoutput-2.npy")
print initialcharges
print len(initialcharges)
np.save("targetdata/initialcharges.npy", initialcharges)
#exit()
#exit()
#print len(boundslist)
frameoffsets = np.array(frameoffsets)
# if not 'feruarab' in skiplist:
# 	excludeidxs = np.hstack(excludeidxs)
excludeidxs = np.hstack(excludeidxs)
print excludeidxs
graphmatch = np.load("graphmatch-%d.npy" % numbonds)
categories = 0
assignments = -np.ones(len(initialcharges), dtype=np.int)
for match in graphmatch:
	if offsets[match[0]] + match[1] in excludeidxs:
		continue
	if offsets[match[0]] + match[1] in excludeidxs:
		print "Continue doesn't mean what I think it means"
		exit()
	assn = assignments[offsets[match[0]] + match[1]]
	if assn == -1:
		assignments[offsets[match[0]] + match[1]] = categories
		categories += 1
	if not offsets[match[2]] + match[3] in excludeidxs:
		assignments[offsets[match[2]] + match[3]] = assignments[offsets[match[0]] + match[1]]
fixlist = []
for i in np.nonzero(assignments == -1)[0]:
	if i in excludeidxs:
		fixlist.append(categories)
	assignments[i] = categories
	categories += 1
minimalq = np.empty(np.amax(assignments)+1, dtype=np.longdouble)
print len(minimalq)
boundslist = []
reverseassignments = []
print fixlist

for i in range(len(minimalq)):
	reverseassignments.append(np.nonzero(assignments==i)[0])
	#The same assignment doesn't mean the same starting charge. This is what makes things interesting.
	if np.amax(initialcharges[reverseassignments[i]]) - np.amin(initialcharges[reverseassignments[i]]) > 0.01:
		print initialcharges[reverseassignments[i]]
	#print 
	minimalq[i] = initialcharges[reverseassignments[-1][0]]
	if np.any(np.abs(np.array([0.09, 0.115])-minimalq[i]) < 1e-4):
		boundslist.append((np.round(minimalq[i], decimals=3), np.round(minimalq[i], decimals=3)))
	elif i in fixlist:
		#print minimalq[i]
		boundslist.append((np.round(minimalq[i], decimals=3), np.round(minimalq[i], decimals=3)))
	elif elements[reverseassignments[-1][0]] == 'H' and minimalq[i] < 0.2:
		boundslist.append((0.09,.2))
	elif elements[reverseassignments[-1][0]] == 'H':
		boundslist.append((0.15,1))
	elif elements[reverseassignments[-1][0]] == 'O':
	 	boundslist.append((-0.85,-0.2))
	elif elements[reverseassignments[-1][0]] == 'C' and minimalq[i] > 0.5:
		boundslist.append((minimalq[i]-0.15,minimalq[i]+0.15))
	elif elements[reverseassignments[-1][0]] == 'C' and minimalq[i] > 0.10:
		boundslist.append((np.amax([minimalq[i]-0.25, 0]), minimalq[i]+0.25))
	elif elements[reverseassignments[-1][0]] == 'C' and minimalq[i] < -0.10:
		boundslist.append((minimalq[i]-0.25, np.amin([0,minimalq[i]+0.25])))
	else:
		boundslist.append((minimalq[i]-0.25,minimalq[i]+0.25))
		#boundslist.append((-.4,1))

#print boundslist
#exit()
def csum(x, i):
	return np.sum(np.take(x, assignments[offsets[i]:offsets[i+1]])) - np.round(np.sum(initialcharges[offsets[i]:offsets[i+1]]))
def cjac(x,i):
	return np.bincount(assignments[offsets[i]:offsets[i+1]], minlength=len(minimalq))

# for i in range(len(offsets)-1):
# 	if i and np.all(np.in1d(assignments[offsets[i]:offsets[i+1]], nset)):
# 		#print "I'm skipping this one."
# 		continue
# 	elif i == 0:
# 		nset = assignments[offsets[i]:offsets[i+1]]
# 	else:
# 		nset = np.union1d(nset, assignments[offsets[i]:offsets[i+1]])
# 	equalityconstraint = dict()
# 	equalityconstraint['type'] = 'eq'
# 	equalityconstraint['fun'] = csum
# 	equalityconstraint['jac'] = cjac
# 	equalityconstraint['args'] = [(i)]
# 	conslist.append(equalityconstraint)
# 	#print i, csum(minimalq,i)
counter = 0
matrixhelper = []
mcolcounter = []
psfset = set()
for superd in ['monomers', 'dimers']:
	psflist = glob.glob("../../%s/*psf" % superd)
	for subd in sorted(next(os.walk("../../%s/watopt" % superd))[1]):
		path = "../../%s/watopt/%s" % (superd, subd)
		psfname = guesspsf(psflist, subd)
		basename = psfname.split("/")[-1][:-4]
		if basename in skiplist:
			continue
		if not psfname in psfset:
			print psfname
			psfset.add(psfname)
			matrixhelper.append(np.bincount(assignments[offsets[counter]:offsets[counter+1]], minlength=len(minimalq)))		
			matrix = np.vstack(matrixhelper)
			if np.linalg.matrix_rank(matrix) == len(conslist)+1:
				equalityconstraint = dict()
				equalityconstraint['type'] = 'eq'
				equalityconstraint['fun'] = csum
				equalityconstraint['jac'] = cjac
				equalityconstraint['args'] = [(counter)]
				conslist.append(equalityconstraint)
			else:
				matrixhelper.pop()
		counter += 1
# print conslist
# print len(conslist)
# for superd in ['dimers']:
# 	psflist = glob.glob("../../%s/*psf" % superd)
# 	for subd in sorted(next(os.walk("../../%s/watopt" % superd))[1]):
# 		path = "../../%s/watopt/%s" % (superd, subd)
# 		psfname = guesspsf(psflist, subd)
# 		print psfname
# 		#linkname = psfname[1:-5]
# 		if not psfname in psfset:
# 			psfset.add(psfname)
# 			equalityconstraint = dict()
# 			equalityconstraint['type'] = 'eq'
# 			equalityconstraint['fun'] = csum
# 			equalityconstraint['jac'] = cjac
# 			equalityconstraint['args'] = [(counter)]
# 			conslist.append(equalityconstraint)
# 			matrixhelper.append(np.bincount(assignments[offsets[counter]:offsets[counter+1]], minlength=len(minimalq)))
# 			mcolcounter.append(counter)
# 		counter += 1
print len(conslist)
print len(minimalq)
matrix = np.vstack(matrixhelper)
print matrix
print np.linalg.matrix_rank(matrix)
S = np.linalg.svd(matrix, compute_uv=False)
print S
#conslist = conslist[:-20]
#exit()
def getdipole(bigq, i):
	#print positions[i]
	#print bigq
	dipole = np.sum(bigq * positions[i].T, axis=1)
	#print dipole
	#exit()
	return dipole

dipolescalefactor = 1
def dipoleterm(qarr, dm):
	bigq = np.take(qarr, assignments)
	retval = 0
	mmdipoles = np.empty((len(dm),3), dtype=np.longdouble)
	sizes = np.empty(len(dm), dtype=np.longdouble)
	for i in range(len(dm)):
		mmdipoles[i] = getdipole(bigq[offsets[i]:offsets[i+1]],i)
		sizes[i] = 1.0 / len(bigq[offsets[i]:offsets[i+1]])
	qmmags = np.linalg.norm(dm[:,:3], axis=1)
	mmmags = np.linalg.norm(mmdipoles, axis=1)
	qmnormv = (dm[:,:3].T / qmmags).T
	mmnormv = (mmdipoles.T / mmmags).T
	angleerror = np.degrees(np.arccos(np.inner(qmnormv, mmnormv).diagonal()))
	if np.all(np.isfinite(angleerror)):
		retval += np.sum(sizes * np.array(angleerror > 30, dtype=np.int) * np.square((angleerror-30)*.2))
	ratio = mmmags / (qmmags * 0.208216744) #This converts the QM value from Debye to e*Angstroms
	retval += np.sum(sizes * (1e2 * np.array(np.logical_or(ratio>1.5, ratio < 1.2), dtype=np.int) * np.minimum(np.square(ratio -1.2), np.square(ratio-1.5))))
	return retval * dipolescalefactor
print assignments[offsets[0]:offsets[1]]
print dipoleterm(minimalq, dipolemoments)
print len(assignments)
#exit()
ljenergies = np.hstack(ljenergies)
qmenergies = np.hstack(qmenergies)
weights = np.minimum(np.ones(len(ljenergies)),np.exp(-ljenergies))
biasscale = 0
def biasterm(qarr):
	return biasscale * np.sum(qarr**4) / len(qarr)
def biasgrad(qarr):
	return biasscale * 4 * qarr**3 / len(qarr)

rescalefactor = 1
def energycalc(qarr, efactor, numels):
	eterm = np.zeros(numels, dtype=np.longdouble)
	for i in range(len(offsets)-1):
		fstart = frameoffsets[i]
		fstop = frameoffsets[i+1]
		qstart = offsets[i]
		qstop = offsets[i+1]
		eterm[fstart:fstop] = np.sum(qarr[qstart:qstop] * efactor[i], axis=1)
	return eterm
def energyterm(qarr, efactor, ematch, weights):
	eterm = energycalc(qarr, efactor, len(ematch))
	x = (eterm - ematch) * 5
	f = np.sum(weights * rescalefactor * (x)**2)
	gradscale = weights * rescalefactor * 2 * x
	gradf = np.ones(len(qarr))
	for i in range(len(offsets)-1):
		fstart = frameoffsets[i]
		fstop = frameoffsets[i+1]
		qstart = offsets[i]
		qstop = offsets[i+1]
		gradf[qstart:qstop] *= np.sum(gradscale[fstart:fstop] * efactor[i].T, axis=1)
	return f, gradf
print energyterm(initialcharges, etermfactors, qmenergies-ljenergies, weights)
#exit()

def objectivefunc(qarr, efactor, ematch, weights, qmdipoles):
	realq = np.take(qarr, assignments)
	f, gradf = energyterm(realq, efactor, ematch, weights)
	dt = dipoleterm(qarr, qmdipoles)
	#print dt
	f += dt
	f += biasterm(qarr)
	#f += roundterm(qarr) #This didn't work how I hoped it would.
	smallgrad = np.zeros(len(qarr), dtype=np.longdouble)
	for i in range(len(qarr)):
		smallgrad[i] = np.sum(np.take(gradf, reverseassignments[i]))
	smallgrad += scipy.optimize.approx_fprime(qarr, dipoleterm, 1e-12, qmdipoles)
	smallgrad += biasgrad(qarr)
	#smallgrad += roundgrad(qarr)
	return f, smallgrad

#exit()
def cb(x):
	global itstart
	print time.clock()-itstart
	print energyterm(np.take(x, assignments), etermfactors, qmenergies-ljenergies, weights)[0], dipoleterm(x, dipolemoments), biasterm(x)#, roundterm(x)
	itstart = time.clock()
start = time.clock()
itstart = time.clock()
print minimalq
print len(minimalq)
print "numbonds =", numbonds
result = scipy.optimize.minimize(objectivefunc, minimalq, args=(etermfactors, qmenergies-ljenergies, weights, dipolemoments), 
		jac=True, bounds=boundslist, constraints=conslist, options={'maxiter': 1000}, callback=cb, tol=1e-4)
end = time.clock()
print result
print energyterm(np.take(result.x, assignments), etermfactors, qmenergies-ljenergies, weights)[0]
print dipoleterm(result.x, dipolemoments)
print objectivefunc(result.x, etermfactors, qmenergies-ljenergies, weights, dipolemoments)
np.save("targetdata/outputcharges.npy", result.x)
np.save("targetdata/assignments-%d.npy" % numbonds, assignments)
np.save("targetdata/bigoutput-%d.npy" % numbonds, np.take(result.x, assignments))
print len(assignments)
np.save("targetdata/waterenergycomparison-%d.npy" % numbonds, np.vstack([qmenergies, ljenergies, energycalc(initialcharges, etermfactors, len(qmenergies)), energycalc(np.take(result.x, assignments), etermfactors, len(qmenergies)), weights]).T)
exit()