#This script has 3 missions:
#1.) load all the gaussian logfiles to make a nice set of adjusted trajectories
#2.) Deduce equivalent atoms, which should have the same charge. The criteria for this should be as follows:
#		a.) Same atomtype
#		b.) Same atomtype graph within 2 bonds
#3.) Calculate all the prefactors and targets.

from Molecule import Molecule
from VMD import evaltcl
import numpy as np
import subprocess
from atomsel import atomsel
import vmdnumpy as vnp
import os
import measure
import glob
import networkx as nx
import networkx.algorithms.isomorphism as iso
import time
import scipy.optimize
from scipy.interpolate import InterpolatedUnivariateSpline
import sys

numbonds = int(sys.argv[1])
print numbonds

def guesspsf(psflist, match):
	matchlist = []
	for psf in psflist:
		if psf.find(match) != -1:
			matchlist.append(psf)
	while not len(matchlist):
		match = match[:-2]
		for psf in psflist:
			if psf.find(match) != -1:
				matchlist.append(psf)
	matchlist.sort(key=len)
	return matchlist[0]
counter = 0
offsets = np.load("targetdata/qoffsets.npy")
frameoffsets = [0]
dipolemoments = np.load("targetdata/dipoles.npy")
ljenergies = []
qmenergies = []
etermfactors = []
initialcharges = np.zeros(offsets[-1], dtype=np.longdouble)
conslist = []
positions = []
elements = []

excludeidxs = []
namelist = ['O4', 'C1', 'H1', 'O1', 'HO1','C4', 'H4', 'C5', 'H51','H52','O5', 'HO5','C2', 'H2', 'O2', 'HO2','C3', 'H3', 'O3', 'HO3']
chargelist = [-0.40, 0.34, 0.09,-0.65, 0.42, 0.11, 0.09,  0.05, 0.09, 0.09,-0.65, 0.42, 0.14, 0.09,-0.65, 0.42, 0.14, 0.09,-0.65, 0.42 ]
for superd in ['monomers', 'dimers', 'trimers']:
	psflist = glob.glob("../../%s/*psf" % superd)
	for subd in sorted(next(os.walk("../../%s/watopt" % superd))[1]):
		path = "../../%s/watopt/%s" % (superd, subd)
		psfname = guesspsf(psflist, subd)
		basename = psfname.split("/")[-1][:-4]
		retypedpsf = "../../typing/%s-cgo.psf" % basename
		if basename.find("dbbl") != -1:
			basename = basename[:-4]

		#print path, guesspsf(psflist, subd)
		mol = Molecule()
		mol.load(retypedpsf)
		mol.load("geom/%d-singlepointtrajs.js" % counter) #Needed for the dipole calculation.
		evaltcl("package require topotools; topo guessatom element mass")
		masses =np.array(atomsel("all").get("mass"))
		R = vnp.timestep(int(mol), 0)
		com = np.mean(R.T*masses, axis=1)
		positions.append(R-com)
		sel = atomsel("resname BARB AARB")
		if len(sel):
			for i in range(len(namelist)):
				asel = atomsel("resname BARB AARB and name %s" % namelist[i])
				if len(asel):
					asel.set("charge", chargelist[i])
		initialcharges[offsets[counter]:offsets[counter+1]] = np.array(atomsel("all").get("charge"))
		elements.append(atomsel("all").get("element"))
		#There are some special cases to consider. Mostly the ferulate linker to xylose.
		excludesel = atomsel("resname BARB AARB and not withinbonds %d of (not resname BARB AARB)" % (numbonds))
		if len(excludesel) > 0:
			excludeidxs.append(offsets[counter] + np.array(excludesel.get("index")))
		ljenergies.append(np.load("targetdata/%d-ljenergiesdist.npy" % counter))
		frameoffsets.append(frameoffsets[-1] + len(ljenergies[-1]))
		etermfactors.append(np.load("targetdata/%d-etermprefactorsdist.npy" % counter))
		qmenergies.append(np.load("targetdata/%d-interactionenergytrajectory.npy" % counter))
		counter += 1
print excludeidxs
elements = np.hstack(elements)
print elements
#exit()
#excludeidxs = np.hstack(excludeidxs)
#exit()
#if numbonds == 3:
#	initialcharges = np.load("targetdata/bigoutput-2.npy")
print initialcharges
print len(initialcharges)
np.save("targetdata/initialcharges.npy", initialcharges)
#exit()
#exit()
#print len(boundslist)
frameoffsets = np.array(frameoffsets)
np.save("targetdata/frameoffsets.npy", frameoffsets)
#exit()
excludeidxs = np.hstack(excludeidxs)
graphmatch = np.load("graphmatch-%d.npy" % numbonds)
categories = 0
assignments = -np.ones(len(initialcharges), dtype=np.int)
for match in graphmatch:
	if offsets[match[0]] + match[1] in excludeidxs:
		continue
	if offsets[match[0]] + match[1] in excludeidxs:
		print "Continue doesn't mean what I think it means"
		exit()
	assn = assignments[offsets[match[0]] + match[1]]
	if assn == -1:
		assignments[offsets[match[0]] + match[1]] = categories
		categories += 1
	if not offsets[match[2]] + match[3] in excludeidxs:
		assignments[offsets[match[2]] + match[3]] = assignments[offsets[match[0]] + match[1]]
fixlist = []
for i in np.nonzero(assignments == -1)[0]:
	if i in excludeidxs:
		fixlist.append(categories)
	assignments[i] = categories
	categories += 1
minimalq = np.zeros(np.amax(assignments)+1, dtype=np.double)
boundslist = []
reverseassignments = []
print fixlist
grids = []
freeq = 0
for i in range(len(minimalq)):
	reverseassignments.append(np.nonzero(assignments==i)[0])
	#The same assignment doesn't mean the same starting charge. This is what makes things interesting.
	if np.amax(initialcharges[reverseassignments[i]]) - np.amin(initialcharges[reverseassignments[i]]) > 0.01:
		print initialcharges[reverseassignments[i]]
	#print 
	for ii in reverseassignments[i]:
		if minimalq[i] == 0 or (elements[ii] != 'H' and abs(initialcharges[ii]) > abs(minimalq[i])):
			minimalq[i] = initialcharges[ii]
	if np.any(np.abs(np.array([0.09, 0.115, 0.15])-minimalq[i]) < 1e-4):
		boundslist.append((np.round(minimalq[i], decimals=3), np.round(minimalq[i], decimals=3)))
	elif elements[reverseassignments[-1][0]] == 'H' and np.any(np.abs(np.array([0.42])-minimalq[i]) < 3e-2):
		boundslist.append((0.42, 0.42))
	elif i in fixlist:
		#print minimalq[i]
		boundslist.append((np.round(minimalq[i], decimals=3), np.round(minimalq[i], decimals=3)))
	# elif elements[reverseassignments[-1][0]] == 'H' and minimalq[i] < 0.2:
	# 	boundslist.append((0.09,.2))
	# elif elements[reverseassignments[-1][0]] == 'H':
	# 	boundslist.append((0.15,1))
	elif elements[reverseassignments[-1][0]] == 'O':
		boundslist.append((np.amin(initialcharges[reverseassignments[i]])-0.07,np.amin([-0.2,np.amax(initialcharges[reverseassignments[i]])+0.07])))
	#  	#boundslist.append((minimalq[i]-0.1,minimalq[i]+0.1))
	# elif elements[reverseassignments[-1][0]] == 'C' and minimalq[i] > 0.5:
	# 	boundslist.append((minimalq[i]-0.25,minimalq[i]+0.15))
	elif elements[reverseassignments[-1][0]] == 'C' and minimalq[i] > 0.10:
		boundslist.append((np.amax([np.amin(initialcharges[reverseassignments[i]])-0.25, 0]), np.amax(initialcharges[reverseassignments[i]])+0.25))
	# elif elements[reverseassignments[-1][0]] == 'C' and minimalq[i] < -0.10 and minimalq[i] > -0.13:
	# 	boundslist.append((minimalq[i]-0.03, np.amin([0,minimalq[i]+0.03])))
	elif elements[reverseassignments[-1][0]] == 'C' and minimalq[i] < -0.1:
		boundslist.append((np.amin(initialcharges[reverseassignments[i]])-0.25, np.amin([0,np.amax(initialcharges[reverseassignments[i]])+0.25])))
	else:
		boundslist.append((np.amin(initialcharges[reverseassignments[i]])-0.25,np.amax(initialcharges[reverseassignments[i]])+0.25))
		freeq += 1
import itertools
def myarange(start, stop, step):
	retvalue = np.arange(start, stop, step, dtype=np.float)
	if len(retvalue) == 0:
		return np.array([start], dtype=np.float)
	return retvalue

#print mesh
#exit()
#print boundslist
#exit()
def csum(x, i):
	return np.sum(np.take(x, assignments[offsets[i]:offsets[i+1]])) - np.round(np.sum(initialcharges[offsets[i]:offsets[i+1]]))
def cjac(x,i):
	return np.bincount(assignments[offsets[i]:offsets[i+1]], minlength=len(minimalq))

# for i in range(len(offsets)-1):
# 	if i and np.all(np.in1d(assignments[offsets[i]:offsets[i+1]], nset)):
# 		#print "I'm skipping this one."
# 		continue
# 	elif i == 0:
# 		nset = assignments[offsets[i]:offsets[i+1]]
# 	else:
# 		nset = np.union1d(nset, assignments[offsets[i]:offsets[i+1]])
# 	equalityconstraint = dict()
# 	equalityconstraint['type'] = 'eq'
# 	equalityconstraint['fun'] = csum
# 	equalityconstraint['jac'] = cjac
# 	equalityconstraint['args'] = [(i)]
# 	conslist.append(equalityconstraint)
# 	#print i, csum(minimalq,i)
counter = 0
matrixhelper = []
mcolcounter = []
counterlist = []
psfset = set()
for superd in ['monomers', 'dimers', 'trimers']:
	psflist = glob.glob("../../%s/*psf" % superd)
	for subd in sorted(next(os.walk("../../%s/watopt" % superd))[1]):
		path = "../../%s/watopt/%s" % (superd, subd)
		psfname = guesspsf(psflist, subd)
		if not psfname in psfset:
			print psfname
			psfset.add(psfname)
			matrixhelper.append(np.bincount(assignments[offsets[counter]:offsets[counter+1]], minlength=len(minimalq)))		
			matrix = np.vstack(matrixhelper)
			if np.linalg.matrix_rank(matrix) == len(conslist)+1:
				equalityconstraint = dict()
				equalityconstraint['type'] = 'eq'
				equalityconstraint['fun'] = csum
				equalityconstraint['jac'] = cjac
				equalityconstraint['args'] = [(counter)]
				conslist.append(equalityconstraint)
				counterlist.append(counter)
			else:
				matrixhelper.pop()
		counter += 1
# print tuple([myarange(np.round(pair[0],decimals=2),np.round(pair[1]+0.0001,decimals=2),0.005) for pair in boundslist])
# biglist = itertools.product(*[myarange(np.round(pair[0],decimals=2),np.round(pair[1]+0.0001,decimals=2),0.005) for pair in boundslist])
# valid = []
# for i, el in enumerate(biglist):
# 	if i % 100 == 99:
# 		print "%d valid out of %d tried. %f success ratio" % (len(valid), i, float(len(valid))/i)
# 	arr = np.array(el)
# 	for c in counterlist:
# 		if csum(arr,c) != 0:
# 			break
# 	else:
# 		valid.append(arr)
# exit()
# print conslist
# print len(conslist)
# for superd in ['dimers']:
# 	psflist = glob.glob("../../%s/*psf" % superd)
# 	for subd in sorted(next(os.walk("../../%s/watopt" % superd))[1]):
# 		path = "../../%s/watopt/%s" % (superd, subd)
# 		psfname = guesspsf(psflist, subd)
# 		print psfname
# 		#linkname = psfname[1:-5]
# 		if not psfname in psfset:
# 			psfset.add(psfname)
# 			equalityconstraint = dict()
# 			equalityconstraint['type'] = 'eq'
# 			equalityconstraint['fun'] = csum
# 			equalityconstraint['jac'] = cjac
# 			equalityconstraint['args'] = [(counter)]
# 			conslist.append(equalityconstraint)
# 			matrixhelper.append(np.bincount(assignments[offsets[counter]:offsets[counter+1]], minlength=len(minimalq)))
# 			mcolcounter.append(counter)
# 		counter += 1
print len(conslist)
print len(minimalq)
matrix = np.vstack(matrixhelper)
print matrix
print np.linalg.matrix_rank(matrix)
S = np.linalg.svd(matrix, compute_uv=False)
print S
#conslist = conslist[:-20]
#exit()
def getdipole(bigq, i):
	#print positions[i]
	#print bigq
	dipole = np.sum(bigq * positions[i].T, axis=1)
	#print dipole
	#exit()
	return dipole
ljenergies = np.vstack(ljenergies)
print ljenergies.shape
print ljenergies

qmenergies = np.hstack(qmenergies)
print qmenergies.shape

weights = (ljenergies[:,3] < 5) * np.minimum(np.exp(-(qmenergies)*0.2), np.ones(len(ljenergies), dtype=np.float))
#weights = np.minimum(np.exp(-(ljenergies[:,3])*0.2), np.ones(len(ljenergies), dtype=np.float))
dipolescalefactor = 1 *  len(np.nonzero(weights)[0])/len(frameoffsets)
def dipoleterm(qarr, dm):
	bigq = np.take(qarr, assignments)
	retval = 0
	mmdipoles = np.empty((len(dm),3), dtype=np.double)
	sizes = np.empty(len(dm), dtype=np.double)
	for i in range(len(dm)):
		mmdipoles[i] = getdipole(bigq[offsets[i]:offsets[i+1]],i)
		sizes[i] = 1.0 / len(bigq[offsets[i]:offsets[i+1]])
	qmmags = np.linalg.norm(dm[:,:3], axis=1)
	mmmags = np.linalg.norm(mmdipoles, axis=1)
	qmnormv = (dm[:,:3].T / qmmags).T
	mmnormv = (mmdipoles.T / mmmags).T
	angleerror = np.degrees(np.arccos(np.inner(qmnormv, mmnormv).diagonal()))
	if np.all(np.isfinite(angleerror)):
		retval += np.sum(sizes * np.array(angleerror > 20, dtype=np.int) * np.square((angleerror-20)*.2))
	ratio = mmmags / (qmmags * 0.208216744) #This converts the QM value from Debye to e*Angstroms
	retval += np.sum(sizes * (1e2 * np.array(np.logical_or(ratio>1.5, ratio < 1.2), dtype=np.int) * np.minimum(np.square(ratio -1.2), np.square(ratio-1.5))))
	return retval * dipolescalefactor
print assignments[offsets[0]:offsets[1]]
print dipoleterm(minimalq, dipolemoments)
print len(assignments)
print ljenergies

biasscale = 0
def biasterm(qarr):
	return biasscale * np.sum(qarr**4) / len(qarr)
def biasgrad(qarr):
	return biasscale * 4 * qarr**3 / len(qarr)




rescalefactor = 1
def energycalc(qarr, efactor, numels):
	eterm = np.zeros((numels,7), dtype=np.double)
	#print factorcolumn
	for i in range(len(offsets)-1):
		fstart = frameoffsets[i]
		fstop = frameoffsets[i+1]
		qstart = offsets[i]
		qstop = offsets[i+1]
		#for j in range(7):
		#	eterm[fstart:fstop] += np.sum(qarr[qstart:qstop] * (efactor[i][:,:,j]), axis=1)
		eterm[fstart:fstop] = np.einsum('j,ijk->ik', qarr[qstart:qstop], efactor[i])
	return eterm

def energyterm(qarr, efactor, eterm, idxs, ematch, weights):
	#eterm = energycalc(qarr, efactor, len(ematch))
	x = (np.take(eterm, 7*np.arange(len(ematch))+idxs) - ematch) * 5 #Times 5 comes from the Escale
	f = np.sum(weights * rescalefactor * (x)**2)
	gradscale = weights * rescalefactor * 2 * x
	gradf = np.ones(len(qarr))
	
	for i in range(len(offsets)-1):
		fstart = frameoffsets[i]
		fstop = frameoffsets[i+1]
		qstart = offsets[i]
		qstop = offsets[i+1]
		tmp = np.zeros(qstop-qstart, dtype=np.float)
		for j in range(fstart, fstop):
			tmp += gradscale[j] * efactor[i][j-fstart,:,idxs[j]]
		gradf[qstart:qstop] *= tmp
		#Gradient only tries to optimize at the offset distance.
		#gradf[qstart:qstop] *= np.sum(gradscale[fstart:fstop] * efactor[i][:,:,3].T, axis=1)
	return f, gradf
#print energyterm(initialcharges, etermfactors, qmenergies-ljenergies[:,2], weights)
#exit()

#Ok, the way the objective function from the FFTK paper is setup, we care about MATCHING the distance. 
#This means we should compute the MM energy for several points, and fit those to a harmonic. From there, find the root, and compare the distance.
# def distanceterm(qarr, efactors, weights):
# 	bigq = np.take(qarr, assignments)
# 	numels = len(weights)
# 	distances = np.empty(numels, dtype=np.double)
# 	x = np.array([-0.2, -0.1, 0, 0.1, 0.2])
# 	#x = np.array([-0.2, 0, 0.2])
# 	energies = np.zeros((numels, 5), dtype=np.double)
# 	for i in range(5):
# 		energies[:,i] = energycalc(bigq, efactors, numels, i) + ljenergies[:,i]
# 	# for i in range(numels):
# 	# 	spline = InterpolatedUnivariateSpline(x, energies[i], k=4)
# 	# 	rootlist = spline.derivative().roots()
# 	# 	if len(rootlist) == 0:
# 	# 		if energies[i][0] < energies[i][-1]:
# 	# 			distances[i] = -1
# 	# 		else:
# 	# 			distances[i] = 1
# 	# 	else:
# 	# 		distances[i] = rootlist[np.argmin(spline(rootlist))]
# 	# print distances
# 	pfit = np.polyfit(x, energies.T, 2)
# 	distances = -pfit[1] / (2*pfit[0])
# 	# print distances2
# 	# exit()
# 	distances = np.where(np.abs(distances) > 2, 2 * np.sign(distances), distances)
# 	#print rootlist, distances[i], -pfit[1] / (2*pfit[0])
# 	#exit()
	
# 	# print weights
# 	# print np.amax(np.abs(distances))
# 	return 0.1 * np.sum(weights * np.square(distances * 10))

def distanceterm(qarr, efactor, eterm, idxs, weights):
	x = np.array([-0.4,-0.2,-0.1,0,0.1, 0.2, 0.4])
	distances = np.take(x,idxs)
	f = np.sum(weights * np.square(distances * 10))
	gradscale = 20 * weights * np.square(distances)
	gradf = np.ones(len(qarr))
	for i in range(len(offsets)-1):
		fstart = frameoffsets[i]
		fstop = frameoffsets[i+1]
		qstart = offsets[i]
		qstop = offsets[i+1]
		tmp = np.zeros(qstop-qstart, dtype=np.float)
		for j in range(fstart, fstop):
			if idxs[j] != 3:
				#print (-eterm[j,idxs[j]] + eterm[j,3]), (efactor[i][j-fstart,:,idxs[j]] - efactor[i][j-fstart,:,3]).shape, qstop-qstart
				tmp += gradscale[j] * (efactor[i][j-fstart,:,idxs[j]] - efactor[i][j-fstart,:,3]) / (-eterm[j,idxs[j]] + eterm[j,3])
		gradf[qstart:qstop] *= tmp
	return f, gradf
#print distanceterm(minimalq, etermfactors, weights)


def objectivefunc(qarr, efactor, ematch, weights, qmdipoles):
	realq = np.take(qarr, assignments)
	mmenergies = energycalc(realq, efactor, len(ematch)) + ljenergies
	#print mmenergies.shape
	minpositions = np.argmin(mmenergies, axis=1)
	#print np.bincount(minpositions)
	#print minpositions.shape
	#exit()
	f, gradf = energyterm(realq, efactor, mmenergies,minpositions,  ematch, weights)
	dt = dipoleterm(qarr, qmdipoles)
	#exit()
	distterm, distgrad = distanceterm(realq, efactor, mmenergies,minpositions,  weights)
	print f, dt, distterm
	# print gradf
	# print distgrad
	# print np.linalg.norm(gradf)
	# print np.linalg.norm(distgrad)
	# print f, distterm
	# exit()
	#print dt
	f += dt
	#f += biasterm(qarr)
	f += distterm
	#f += roundterm(qarr) #This didn't work how I hoped it would.
	gradf += distgrad
	smallgrad = np.zeros(len(qarr), dtype=np.double)
	for i in range(len(qarr)):
		smallgrad[i] = np.sum(np.take(gradf, reverseassignments[i]))
	smallgrad += scipy.optimize.approx_fprime(qarr, dipoleterm, 1e-4, qmdipoles)
	#smallgrad += scipy.optimize.approx_fprime(qarr, distanceterm, 1e-3, efactor, weights)
	f *= 0.01
	smallgrad *= 0.01
	return f, smallgrad
itcount = 0
#exit()
np.save("targetdata/assignments-%d.npy" % numbonds, assignments)
def cb(x):
	global itstart
	global itcount
	print time.clock()-itstart
	print "Iteration %d" % itcount
	#print energyterm(np.take(x, assignments), etermfactors, qmenergies, weights)[0], dipoleterm(x, dipolemoments)
	# consarr = np.empty(len(conslist), dtype=np.float)
	# for i, cons in enumerate(conslist):
	# 	consarr[i] = cons['fun'](x, cons['args'])
	# print np.amax(np.abs(consarr))
	itstart = time.clock()
	itcount += 1
	if itcount % 100 == 0:

		np.save("targetdata/outputcharges-%d.npy" % itcount, x)
start = time.clock()
itstart = time.clock()
print minimalq
print len(minimalq)
print "numbonds =", numbonds
#cb(minimalq)
#exit()
result = scipy.optimize.minimize(objectivefunc, minimalq, args=(etermfactors, qmenergies, weights, dipolemoments), 
		jac=True, bounds=boundslist, constraints=conslist, options={'maxiter': 2000}, callback=cb, tol=1e-4)
end = time.clock()
print result
#print energyterm(np.take(result.x, assignments), etermfactors, weights)[0]
#print dipoleterm(result.x, dipolemoments)
print objectivefunc(result.x, etermfactors, qmenergies, weights, dipolemoments)
np.save("targetdata/outputcharges-%d.npy" % numbonds, result.x)

np.save("targetdata/bigoutput-%d.npy" % numbonds, np.take(result.x, assignments))
cb(result.x)
#energyterm(qarr, efactor, eterm, idxs, ematch, weights):
#
#np.save("targetdata/waterenergycomparison-%d.npy" % numbonds, np.vstack([qmenergies, ljenergies[:,0], energycalc(initialcharges, etermfactors, len(qmenergies)), energycalc(np.take(result.x, assignments), etermfactors, len(qmenergies)), weights]).T)
np.save("targetdata/waterenergycomparison-%d.npy" % numbonds, np.vstack([qmenergies, ljenergies[:,3], energycalc(initialcharges, etermfactors, len(qmenergies))[:,3], energycalc(np.take(result.x, assignments), etermfactors, len(qmenergies))[:,3], weights]).T)
exit()
