import numpy as np
import matplotlib
matplotlib.use('Cairo')
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}",r"\renewcommand{\seriesdefault}{\bfdefault}",
r"\usepackage{amsfonts}",r"\usepackage{textcomp}",r"\usepackage{gensymb}", r'\boldmath'] #Using \boldmath makes the axis label numbers bold as well! Looks good.
params = {'text.usetex' : True,
          'font.size' : 14,
          'font.family' : 'lmodern',
          'text.latex.unicode': True,
          'figure.figsize' : (5.6, 4.2), #8, 6 originally. Make it ~30% smaller so the text is ~30% bigger. (Multiply by .7)
          'figure.dpi' : 120
          }
matplotlib.rcParams.update(params)
import matplotlib.pyplot as plt
from matplotlib.ticker import NullFormatter
import matplotlib.ticker as ticker
from scipy import stats
import statsmodels.api as sm
from scipy import stats
from scipy.optimize import curve_fit
nullfmt   = NullFormatter()


#fig, axes = plt.subplots(2,2, figsize=(8,8))
left, width = 0.125, 0.57
bottom, height = 0.135, 0.54
bottom_h = left_h = left+width+0.02
rect_scatter = [left, bottom, width, height]
rect_histx = [left, bottom_h, width, 0.25]
rect_histy = [left_h, bottom, 0.25, height]


fig = plt.figure(1)
ax = plt.axes(rect_scatter)
hxax = plt.axes(rect_histx, sharex=ax)
hyax = plt.axes(rect_histy, sharey=ax)

def func(x, m, b):
	return m * x + b
fout = open("results/fitstatistics.tex", "w")
fout.write('''
\\begin{tabular}{lrrrr}
\\toprule
 & \multicolumn{2}{c}{Small E (E$^{int}<0$\,kcal/mol)} & \multicolumn{2}{c}{Larger E (E$^{int}<20$\,kcal/mol)} \\\\
\cmidrule(rl){2-3}  \cmidrule(rl){4-5}
Charge Scheme & $\left< \left( E^{int}_{QM} - E^{int}_{MM} \\right)^2 \\right>^{\\frac{1}{2}}$  & $R^2$ & $\left< \left( E^{int}_{QM} - E^{int}_{MM} \\right)^2 \\right>^{\\frac{1}{2}}$  & $R^2$ \\\\
\midrule
''')
fig, ax = plt.subplots(1,1)
ax.plot([-10,5],[-10,5], color='k', zorder=1)
colors = ['red', 'skyblue', 'yellowgreen']
for i in range(1,4):
	# if i == 1:
	# 	data = np.load("targetdata/waterenergycomparisonsoft-%d.npy" % i).astype(np.float)
	data = np.load("targetdata/waterenergycomparison-%d.npy" % i).astype(np.float)
	print data.shape
	print np.amax(data, axis=0)
	x = data[:,0]
	lj = data[:,1]
	idxs = np.nonzero((x<0)*(lj<3))[0]
	idxs2 = np.nonzero((x<20)*(lj<3))[0]
	
	yo = data[:,2]
	yf = data[:,3]
	w = data[:,4]
	print x.shape
	print w
	if i == 1:
		mod_wls = sm.WLS(lj+yo, x, weights=w)
		res_wls = mod_wls.fit()
		print res_wls.summary()
		print res_wls.rsquared, res_wls.mse_resid, res_wls.params, res_wls.bse
		slope, intercept, r_value, p_value, std_err = stats.linregress(x[idxs],(lj+yo)[idxs])
		slope, intercept, r2, p_value, std_err = stats.linregress(x[idxs2],(lj+yo)[idxs2])
		fout.write("%s & %.4f kcal/mol & %.5f & %.4f kcal/mol & %.5f \\\\\n" % ("CGenFF", np.sqrt(np.mean(np.square(x[idxs]-(lj+yo)[idxs]))), r_value * r_value, np.sqrt(np.mean(np.square(x[idxs2]-(lj+yo)[idxs2]))), r2 * r2))
		ax.scatter(x[idxs2], (lj+yo)[idxs2], color='k', s=1, zorder=2, label='CGenFF')
		print func(np.array([-10,10]), res_wls.params[0], res_wls.bse[0])
		#ax.plot([-10,10], func(np.array([-10,10]), res_wls.params[0], res_wls.bse[0]), color='k', linestyle='--')
	mod_wls2 = sm.WLS(lj+yf, x, weights=w)
	res_wls2 = mod_wls2.fit()
	print res_wls2.rsquared, res_wls2.mse_resid, res_wls2.params, res_wls2.bse
	slope, intercept, r_value, p_value, std_err = stats.linregress(x[idxs],(lj+yf)[idxs])
	slope, intercept, r2, p_value, std_err = stats.linregress(x[idxs2],(lj+yf)[idxs2])
	fout.write("%s & %.4f kcal/mol & %.5f  & %.4f kcal/mol & %.5f \\\\\n" % ("Revised %d-bond" % i, np.sqrt(np.mean(np.square(x[idxs]-(lj+yf)[idxs]))), r_value * r_value, np.sqrt(np.mean(np.square(x[idxs2]-(lj+yf)[idxs2]))), r2 * r2))
	ax.scatter(x[idxs2], (lj+yf)[idxs2], color=colors[i-1], s=1, zorder=2, label="%d-bond fitting" % i)
ax.set_aspect('equal')
ax.set_xlim([-10,5])
ax.set_ylim([-10,5])
ax.set_xlabel("QM Interaction Energy (kcal/mol)")
ax.set_ylabel("MM Interaction Energy (kcal/mol)")
leg=ax.legend(loc=2,fontsize=12, frameon=False)
tcolors = ['k', 'r', 'skyblue', 'yellowgreen']
for i, text in enumerate(leg.get_texts()):
    text.set_color(tcolors[i])
fout.write('''
\\bottomrule
\end{tabular}
''')

fout.close()
fig.savefig("results/scatter.png", bbox_inches='tight', dpi=300)

fig, ax = plt.subplots(1,1)
ax.plot([-10,1e3],[-10,1e3], color='k', zorder=1)
ax.set_xscale("symlog", linthreshx=10)
ax.set_yscale("symlog", linthreshy=10)
ax.set_xlim(-10.00001,1e3)
ax.set_ylim(-10.00001,1e5)
for i in range(1,4):
	data = np.load("targetdata/waterenergycomparison-%d.npy" % i).astype(np.float)
	print data.shape
	print np.amax(data, axis=0)
	x = data[:,0]
	lj = data[:,1]
	yo = data[:,2]
	yf = data[:,3]
	w = data[:,4]
	print x.shape
	print w
	if i == 1:
		mod_wls = sm.WLS(lj+yo, x, weights=w)
		res_wls = mod_wls.fit()
		print res_wls.summary()
		print res_wls.rsquared, res_wls.mse_resid, res_wls.params, res_wls.bse
		ax.scatter(x, lj+yo, color='k', s=1, zorder=2, label='CGenFF')
		print func(np.array([-10,10]), res_wls.params[0], res_wls.bse[0])
		#ax.plot([-10,10], func(np.array([-10,10]), res_wls.params[0], res_wls.bse[0]), color='k', linestyle='--')
	mod_wls2 = sm.WLS(lj+yf, x, weights=w)
	res_wls2 = mod_wls2.fit()
	print res_wls2.rsquared, res_wls2.mse_resid, res_wls2.params, res_wls2.bse
	ax.scatter(x, lj+yf, color=colors[i-1], s=1, zorder=2, label="%d-bond fitting" % i)
#ax.set_aspect('equal')
ax.set_xlabel("QM Interaction Energy (kcal/mol)")
ax.set_ylabel("MM Interaction Energy (kcal/mol)")
leg=ax.legend(loc=2,fontsize=12, frameon=False)
for i, text in enumerate(leg.get_texts()):
    text.set_color(tcolors[i])

fout.close()
fig.savefig("results/scatterlog.png", bbox_inches='tight', dpi=300)