import matplotlib
import numpy as np
from scipy.interpolate import interp1d
from matplotlib.ticker import MaxNLocator
matplotlib.use('Cairo')
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}",r"\renewcommand{\seriesdefault}{\bfdefault}",
r"\usepackage{amsfonts}",r"\usepackage{textcomp}",r"\usepackage{gensymb}", r'\boldmath'] #Using \boldmath makes the axis label numbers bold as well! Looks good.
params = {'text.usetex' : True,
          'font.size' : 14,
          'font.family' : 'lmodern',
          'text.latex.unicode': True,
          'figure.figsize' : (5.6, 4.2), #8, 6 originally. Make it ~30% smaller so the text is ~30% bigger. (Multiply by .7)
          'figure.dpi' : 120
          }
matplotlib.rcParams.update(params)
import matplotlib.pyplot as plt
import glob
import os
import matplotlib.cm as cm
arrays = []
initialcharges = np.load("targetdata/initialcharges.npy")
outputcharges = np.load("targetdata/bigoutput-1.npy")
print initialcharges.shape
print np.load("targetdata/outputcharges.npy").shape
elements = np.load("targetdata/elements.npy")
noh = elements !='H'
arrays.append(initialcharges)
arrays.append(outputcharges)
for i in range(1,4):
	arrays.append(np.load("../chargecgo/targetdata/bigoutput-%d.npy" % i))
fout = open("results/differencematrix.tex", "w")
fout.write('''
\\begin{tabular}{lrrrr}
\\toprule
Charge Type & Group & 1-Neighbor & 2-Neighbor & 3-Neighbor \\\\
\midrule
''')
matrix = np.zeros((5,5), dtype=np.float)
for i in range(5):
	for j in range(5):
		matrix[i][j] = np.mean(np.abs(arrays[i][noh]-arrays[j][noh]))
labels = ['CGenFF', 'Group', '1-Neighbor', '2-Neighbor']
for i in range(4):
	s = labels[i]
	for j in range(i):
		s += " & --"
	for j in range(4-i):
		s += " & %.3f" % matrix[i][i+j+1]
	fout.write(s + "\\\\\n")
fout.write('''
\\bottomrule
\end{tabular}
''')
print matrix
exit()

print outputcharges.shape
fig, ax = plt.subplots(1,1, figsize=(8,6))
ax.grid(True)
ax.hist(outputcharges-initialcharges, bins=1000,range=[-0.3,0.3],cumulative=True, density=True,histtype='step',zorder=10, color='k', label="Overall")
ax.hist((outputcharges-initialcharges)[elements=='H'], bins=1000,range=[-0.3,0.3],cumulative=True, density=True,histtype='step',zorder=3, color='green', label="Hydrogens")
ax.hist((outputcharges-initialcharges)[elements=='O'], bins=1000,range=[-0.3,0.3],cumulative=True, density=True,histtype='step',zorder=2, color='red', label="Oxygens")
ax.hist((outputcharges-initialcharges)[elements=='C'], bins=1000,range=[-0.3,0.3],cumulative=True, density=True,histtype='step',zorder=1, color='gray', label="Carbons")
ax.set_xlabel("Charge Shift ($q_{opt}-q_{CGenFF}$)")
ax.set_ylabel("Cumulative Probability")
ax.set_axisbelow(True)
ax.set_ylim(0,1)
ax.set_xlim(-0.15,0.15)
ax.legend(fontsize=12,loc=2)
fig.tight_layout()
fig.savefig("chargeshift.png", dpi=300)
fig.savefig("chargeshift.pdf")