#This script has 3 missions:
#1.) load all the gaussian logfiles to make a nice set of adjusted trajectories
#2.) Deduce equivalent atoms, which should have the same charge. The criteria for this should be as follows:
#		a.) Same atomtype
#		b.) Same atomtype graph within 2 bonds
#3.) Calculate all the prefactors and targets.

from Molecule import Molecule
from VMD import evaltcl
import numpy as np
import subprocess
from atomsel import atomsel
import vmdnumpy as vnp
import os
import measure
import glob
import networkx as nx
import networkx.algorithms.isomorphism as iso
import sys
import matplotlib
matplotlib.use("Agg")
import matplotlib.pyplot as plt
import cPickle as pickle
numbonds = 1
#numbonds = int(sys.argv[1])
print numbonds
#exit()

def isfloat(f):
	try:
		float(f)
		return True
	except ValueError:
		return False
def readLJparameters(ljdict, filename):
	fin = open(filename)
	lines = fin.readlines()[:-2]
	fin.close()
	dictfill = False
	for i, l in enumerate(lines):
		string = l.strip()
		if len(string):
			idx = string.find("!")
			if idx < 0:
				s = string
			else:
				s = string[:idx]
			if len(s):
				splt = s.split()
				if splt[0] == "NONBONDED":
					dictfill = True
				elif splt[0] == "HBOND":
					dictfill = False
				elif splt[0] == "NBFIX":
					dictfill = False
				elif dictfill and isfloat(splt[2]) and isfloat(splt[3]):
					ljdict[splt[0]] = (float(splt[2]),float(splt[3]))
				#print s, len(s)
def LJterm(R, types, wmovevector):
	total = np.zeros(7)
	for k, multfactor in enumerate([-2,-1, -0.5, 0, 0.5, 1,2]):
		for i in range(3):
			wpos = R[-(1+i)] + multfactor * wmovevector
			wLJprm = LJparams[types[-(1+i)]]
			for j in range(len(R)-3):
				cpos = R[j]
				cLJprm = LJparams[types[j]]
				eps = np.sqrt(wLJprm[0] * cLJprm[0])
				rmin = wLJprm[1] + cLJprm[1]
				r = np.linalg.norm(cpos-wpos)
				total[k] += eps * ((rmin/r)**12-2*(rmin/r)**6)
	#exit()
	return total
def Etermprefactors(R, charges, wmovevector):
	result = np.zeros((len(charges)-3, 7))
	for j, multfactor in enumerate([-2,-1, -0.5, 0, 0.5, 1,2]):
		for i in range(3):
			wpos = R[-(1+i)] + multfactor * wmovevector
			q = charges[-(1+i)]
			r = np.linalg.norm(R[:-3] - wpos, axis=1)
			result[:,j] += 332.0716 * q / r #The 332.0716 is the coulombic constant in kcal/mol/Angs e^2 according to CHARMM.
	return result
LJparams = dict()
readLJparameters(LJparams, "/home/josh/toppar/toppar_water_ions.str")
readLJparameters(LJparams, "/home/josh/toppar/par_all36_cgenff.prm")

readLJparameters(LJparams, "../../typing/lignins-cgo2.prm")

print sorted(LJparams.keys())
def makefullgraph(mol, cutoff=0.051, allowcross=True, maxsize=8, expandcarboxy=True):
	sel = atomsel("all", molid=int(mol))
	sel.set("beta", -1)
	types = sel.get("type")
	idxs = sel.get("index")
	mass = np.array(sel.get("mass"))
	charges = np.array(sel.get("charge"))
	G = nx.Graph()
	for j, idx in enumerate(idxs):
		G.add_node(idx, t = types[j], q=charges[j])
	for j, b in enumerate(sel.bonds):
		idx = idxs[j]
		for bidx in b:
			if bidx in idxs and idx < bidx:
				G.add_edge(idx, bidx)
	orderdict = nx.algorithms.centrality.eigenvector_centrality_numpy(G)
	numbonds = np.array(sel.get("numbonds"))
	order = np.array([orderdict[key] + numbonds[key] * numbonds[key] for key in sorted(orderdict.keys())])
	print order
	#exit()
	
	groups = -np.ones(len(sel))
	groupid = 0
	print "nnz", np.nonzero(np.abs(charges)<cutoff)[0]
	for idx in np.nonzero(np.abs(charges)<cutoff)[0]:
		s = atomsel("(withinbonds 1 of index %d) and name \"H.*\" \"O.*\"" % idx, molid=int(mol))
		if len(s) == 0:
			groups[idx] = groupid
			groupid += 1
		s
	print mass
	print np.nonzero(mass < 2.0)
	print mass < 2.0
	for idx in np.nonzero(mass<2.0)[0]:
		s = atomsel("(not index %d) and (withinbonds 1 of index %d)" % (idx, idx), molid=int(mol))
		oidx = s.get("index")[0]
		print "small mass", idx, oidx, np.abs(charges[idx] + charges[oidx]), charges[idx], charges[oidx]
		if np.abs(charges[idx] + charges[oidx]) < cutoff and numbonds[oidx] == 3:
			groups[idx] = groupid
			groups[s.get("index")[0]] = groupid
			groupid += 1
			print idx, s.get("index")[0]
		s
	noreset = " ".join(np.nonzero(groups != -1)[0].astype('|S8'))
	sel.set("beta", groups)
	firstfail = True
	while np.any(groups == -1) and groupid < 100:
		if allowcross:
			restext = ""
		else:
			restext = "and same residue as beta == %d" % groupid

		#idx = missingidx[np.argmin(numbonds[missingidx])]
		missingidx = np.nonzero(groups==-1)[0]
		idx = missingidx[np.argmin(order[missingidx])]
		# #idx = missingidx[np.argmax(np.abs(charges[missingidx]))]
		# if numbonds[idx]>1:
		# 	missingidx = np.nonzero(np.logical_and(groups == -1, numbonds==numbonds[idx]))[0]
		# 	idx = missingidx[np.argmax(np.abs(charges[missingidx]))]
		# 	#idx = missingidx[np.argmin(numbonds[missingidx])]
		q = charges[idx]
		s = atomsel("index %d" % idx, molid=int(mol))
		s.set("beta", groupid)
		s = atomsel("beta == %d or (numbonds == 1 and withinbonds 1 of beta == %d)" % (groupid, groupid), molid=int(mol))
		s.set("beta", groupid)
		groups = np.array(sel.get("beta"))
		qsum = np.sum(s.get("charge"))
		while (np.abs(qsum-np.round(qsum))>cutoff and np.any(groups==-1)):
			heavyatoms = atomsel("beta == -1 and name \"O.*\" and (withinbonds 1 of beta == %d) %s" % ( groupid, restext), molid=int(mol))
			isoxy=True
			if len(heavyatoms) == 0:
				heavyatoms = atomsel("beta == -1 and (withinbonds 1 of beta == %d) %s" % (groupid, restext), molid=int(mol))
				isoxy=False
			hidxs = np.array(heavyatoms.get("index"))
			if len(hidxs) == 0:
				break
			#print hidxs, np.abs(charges[hidxs] - qsum)
			idx = hidxs[np.argmin(np.abs(charges[hidxs] + qsum))]

			s = atomsel("index %d" % idx, molid=int(mol))
			s.set("beta", groupid)
			oxysel = atomsel("name \"O.*\" and withinbonds 1 of index %d" % idx, molid=int(mol))
			if expandcarboxy and numbonds[idx] == 3 and len(oxysel) == 2:
				s = atomsel("beta == %d or (withinbonds 1 of beta == %d)" % (groupid, groupid), molid=int(mol))
				s.set("beta", groupid)
			s = atomsel("beta == %d or (numbonds == 1 and withinbonds 1 of beta == %d)" % (groupid, groupid), molid=int(mol))
			s.set("beta", groupid)
			qsum = np.sum(s.get("charge"))
			#print qsum, groupid, np.abs(qsum-np.round(qsum))
			#print s.get("index")
			groups = np.array(sel.get("beta"))
		print "Checking charge"
		if np.abs(qsum-np.round(qsum))>cutoff:
			print "Charge group isn't right!", np.abs(qsum-np.round(qsum))
			resetsel = atomsel("not (beta == %d or index %s) and same beta as withinbonds 1 of beta == %d" % (groupid, noreset, groupid), molid=int(mol))
			s = atomsel("beta == %d" % groupid, molid=int(mol))
			print s.get("index"), " reset group to -1"

			s.set("beta", -1)
			resetsel.set("beta", -1)
			resetidxs = resetsel.get("index")
			for i in resetidxs:
				order[i] += orderdict[i]
			groups = np.array(sel.get("beta"))
		print "Checking size"
		if (not np.any(groups == -1)) and (len(np.nonzero(groups == groupid)[0]) >= maxsize) and firstfail:
			print "Group too large!"
			s = atomsel("beta == %d" % groupid, molid=int(mol))
			s.set("beta", -1)
			resetidxs = s.get("index")
			H = G.subgraph(resetidxs)
			subsetorder = nx.algorithms.centrality.eigenvector_centrality_numpy(H)
			#between = nx.algorithms.centrality.betweenness_centrality(H)
			for i in resetidxs:
				order[i] = subsetorder[i]
				firstfail = False
			groups = np.array(sel.get("beta"))
		elif (not np.any(groups == -1)) and (len(np.nonzero(groups == groupid)[0]) >= maxsize) and len(atomsel("ringsize 6 from beta == %d" % groupid, molid=int(mol))) == 6:
			print "Group too big and I've failed already."
			s = atomsel("beta == %d" % groupid, molid=int(mol))
			s.set("beta", -1)
			resetidxs = np.array(s.get("index"))
			idx = resetidxs[np.argmin(order[resetidxs])]
			order[idx] += 1
			groups = np.array(sel.get("beta"))
		groupid += 1

	G = nx.Graph()
	for j, idx in enumerate(idxs):
		G.add_node(idx, t = types[j], q=charges[j], g=groups[j])
	for j, b in enumerate(sel.bonds):
		idx = idxs[j]
		for bidx in b:
			if bidx in idxs and idx < bidx:
				G.add_edge(idx, bidx)
	return G

def generateatomgraphs(mol, counter):
	asel = atomsel("all", molid=int(mol))
	betaset = set(asel.get("beta"))
	for i, beta in enumerate(betaset):
		sel = atomsel("abs(beta - %f)<0.1" % beta, molid=int(mol))
		types = sel.get("type")
		idxs = sel.get("index")
		charges = sel.get("charge")
		G = nx.Graph()
		for j, idx in enumerate(idxs):
			G.add_node(idx, t = types[j], q=charges[j])
		for j, b in enumerate(sel.bonds):
			idx = idxs[j]
			for bidx in b:
				if bidx in idxs and idx < bidx:
					G.add_edge(idx, bidx)
		bigatomgraphs[(counter, i)] = G
		nx.write_gpickle(G, "pickles/%d-%d.gpkl" % (counter, i))
	for i in range(mol.numAtoms()):
		sel = atomsel("same beta as index %d" % (i), molid=int(mol))
		types = sel.get("type")
		idxs = sel.get("index")
		charges = sel.get("charge")
		G = nx.Graph()
		for j, idx in enumerate(idxs):
			typ = types[j]
			if idx == i:
				G.add_node(idx, t = typ+"+", q=charges[j]) #This needs to be marked. There are some pathological problems that can result if this isn't done, with oxygens resulting in the same graph as hydrogens on the same carbon.
			else:
				G.add_node(idx, t = typ, q=charges[j])
		for j, b in enumerate(sel.bonds):
			idx = idxs[j]
			for bidx in b:
				if bidx in idxs and idx < bidx:
					G.add_edge(idx, bidx)
		atomgraphs[(counter, i)] = G

def guesspsf(psflist, match):
	matchlist = []
	for psf in psflist:
		if psf.find(match) != -1:
			matchlist.append(psf)
	while not len(matchlist):
		match = match[:-2]
		for psf in psflist:
			if psf.find(match) != -1:
				matchlist.append(psf)
	matchlist.sort(key=len)
	return matchlist[0]
def readdipoledata (filename, numatoms):
	fin = open(filename)
	print numatoms
	coordinates = np.empty((numatoms,3), dtype=np.float)
	while True:
		line = fin.readline()
		if line == '':
			break
		elif line.strip() == "Standard orientation:":
			for i in range(4):
				fin.readline()
			for i in range(numatoms):
				coordinates[i] = np.array(fin.readline().split()[3:])
		elif line.strip() == "Dipole moment (field-independent basis, Debye):":
			dipoledata = np.array((fin.readline().strip().split())[1::2], dtype=np.float)
	return coordinates, dipoledata
def drawgraph(G, graphname):
	nx.write_gpickle(G, "pickles/%s.gpkl" % (graphname))
	attributes = nx.get_node_attributes(G,'g')
	charge = nx.get_node_attributes(G,'q')
	attrset = set(attributes.values())
	colors = np.linspace(0,1,len(attrset))
	colorlist = np.zeros(len(attributes))
	maxsize = 0
	maxcharge = 0
	subgraphs = []
	for i, el in enumerate(attrset):
		idxs = np.nonzero(np.array(attributes.values()) == el)[0]
		subgraphs.append(G.subgraph(idxs))
		if len(idxs) > maxsize:
			maxsize = len(idxs)
		colorlist[idxs] = colors[i]
		charges = np.array([charge[key] for key in idxs])
		if np.abs(np.sum(charges)-np.round(np.sum(charges))) > maxcharge:
			maxcharge = np.abs(np.sum(charges))
	fig, ax = plt.subplots(1,1)
	pos = nx.nx_agraph.graphviz_layout(G)
	print colorlist
	if len(attrset) <= 10:
		cmapname ='Vega10'
	else:
		cmapname ='Vega20'
	nx.draw_networkx(G, pos=pos, ax=ax, node_color=colorlist, cmap=plt.get_cmap(cmapname))
	plt.axis('off')
	#fig.text(0.02, 0.85, "n=%d" % len(attrset), transform=fig.transFigure)
	#fig.text(0.02, 0.80, "maxsize=%d" % maxsize, transform=fig.transFigure)
	fig.savefig("identitygraph/fullgraph-%s.png" % graphname, bbox_inches='tight', dpi=300)
	fig.savefig("identitygraph/fullgraph-%s.svg" % graphname, bbox_inches='tight', dpi=300)
	if len(attrset) < 3:
		print "Attrset too short"
		print graphname
		exit()
	if maxsize >= 10 and graphname != 'trcn':
		print "Too many groups"
		print graphname
		exit()
	if maxsize >= 8 and graphname == 'pbdoc':
		print "Too many groups"
		print graphname
		exit()
	if maxcharge >= 0.051:
		print maxcharge
		print graphname
		exit()
	if -1 in attrset or 99 in attrset:
		print attrset
		print graphname
		exit()
	return subgraphs
watmol = Molecule()
watmol.load("../../monomers/onewater.psf")
evaltcl("package require forcefieldtoolkit; package require qmtool; ::QMtool::init_variables ::QMtool")
atomgraphs = dict()
bigatomgraphs = dict()
dipolemoments = []
redo = 1
#singlepointenergysum = []
qoffsets = [0]
counter = 0
allsubgraphs = dict()
errlog = open("baddirectories.log", "w")
for superd in ['monomers', 'dimers', 'trimers']:
	psflist = glob.glob("../../%s/*psf" % superd)
	for subd in sorted(next(os.walk("../../%s/watopt" % superd))[1]):
		path = "../../%s/watopt/%s" % (superd, subd)
		psfname = guesspsf(psflist, subd)
		print "PY", psfname
		basename = psfname.split("/")[-1][:-4]
		retypedpsf = "../../typing/%s-cgo.psf" % basename
		if basename.find("dbbl") != -1:
			basename = basename[:-4]
		if basename.find("dbac") != -1:
			basename = basename[:-4]
		if basename.find("dbal") != -1:
			basename = basename[:-4]
		if basename.find("dbap") != -1:
			basename = basename[:-4]
		if basename.find("lch") != -1:
			basename = basename[:-3]
		if basename.find("lcal") != -1:
			basename = basename[:-4]
		if basename.find("lcap") != -1:
			basename = basename[:-4]
		if basename.find("lcac") != -1:
			basename = basename[:-4]
		#print path, guesspsf(psflist, subd)
		mol = Molecule()
		mol.load(retypedpsf)
		
		maxsize = 8
		cutoff=0.051
		if 'trcn' in retypedpsf:
			maxsize=15
		elif 'bdo' in retypedpsf:
			maxsize=7
		excarbo = True
		if 'diferulateA' in retypedpsf:
			excarbo = False
			cutoff=0.0451
		G = makefullgraph(mol, cutoff=cutoff, allowcross=not ('pbbp' in retypedpsf), maxsize=maxsize, expandcarboxy=excarbo)
		allsubgraphs[counter] = drawgraph(G, os.path.basename(retypedpsf)[:-8])
		#exit()
		#continue
		qoffsets.append(qoffsets[-1] + mol.numAtoms())
		generateatomgraphs(mol, counter)
		# counter += 1
		# continue
		if redo or not os.path.isfile("geom/%d-waterinteractiontrajectory.js" % counter):
			#Deal with the single points.
			singlepointfile = "%s/%s-sp-HF/%s-sp-HF.log" % (path, basename, basename)
			if not os.path.isfile(singlepointfile):
				singlepointfile = "%s/%s-sp-HF/%s-sp-HF.log" % (path, basename.upper(), basename.upper())
			coords, dipole = readdipoledata(singlepointfile, mol.numAtoms())
			dipolemoments.append(dipole)
			mol.dupFrame()
			allsel = atomsel("all")
			for j, coordinate in enumerate(['x', 'y', 'z']):
				allsel.set(coordinate, coords[:,j])
			energysum = 0
			energysum += float(subprocess.check_output("grep \"SCF Done\" %s | awk '{print $5}'" % (singlepointfile), shell=True))
			energysum += float(subprocess.check_output("grep \"SCF Done\" %s/wat-sp/wat-sp.log | awk '{print $5}'" % (path), shell=True))
			#singlepointenergysum.append(energysum)
			mol.save("geom/%d-singlepointtrajs.js" % counter)
			mol.delFrame()
			#Now deal with the other stuff.
			residueidxs = mol.numAtoms() -1
			#Since I was lazy, and didn't remake a psf with the compound + water, doing that here.
			evaltcl("package require topotools; set mol [::TopoTools::mergemols [list %d %d]]; animate write psf merged.psf $mol" % (int(mol),int(watmol)))
			mergemol = Molecule()
			mergemol.load("merged.psf")
			mergesel = atomsel("all")
			nowat = atomsel("not water")
			watersel = atomsel("water")
			waternames = np.array(watersel.get("name"), dtype="S1")
			eidx1 = np.nonzero(waternames == 'O')[0][0]
			wosel = atomsel("noh and water")
			qmintE = []
			dirlist = sorted(os.walk(path).next()[1])
			watermovevectors = []
			print dirlist

			for d in dirlist:
				#Single points need to be excluded.
				if d.find("-sp") != -1:
					continue
				#For whatever reason, I managed to construct situations where hydrogens were the acceptor. these are beyond useless.
				elif d.find("-ACC-H") != -1:
					continue
				print d
				atomname = d.split('-')[2]
				newmol = Molecule()
				#Find the energies
				logfile = "%s/%s/%s.log" % (path, d, d)
				print logfile
				if subprocess.call("grep Error %s" % logfile, shell=True) == 0:
					errlog.write("%s %s\n" % (path,d))
					continue #If there was an error, skip this directory.
				energies = np.array(subprocess.check_output("grep \"SCF Done\" %s | awk '{print $5}'" % (logfile), shell=True).strip().split("\n"), dtype=np.float)
				energies -= energysum
				energies *= 627.509469 #This converts hartrees to kcal/mol
				print energies
				
				#Read in the positions
				evaltcl("::QMtool::read_gaussian_log %s %d" % (logfile, int(newmol)))
				sel = atomsel("not name \"X.*\"")
				nowatsel = atomsel("index 0 to %d" % residueidxs)
				newwatsel = atomsel("not name \"X.*\" and index > %d" % residueidxs)
				newwaternames = np.array(newwatsel.get("name"), dtype='S1')
				eidx2 = np.nonzero(newwaternames == 'O')[0][0]
				#print len(nowatsel), len(nowat)
				nowatsel.set("name", nowat.get("name"))
				asel = atomsel("name %s" % atomname)
				#print len(mergesel), len(sel)
				#read_gaussian_log reads in the last position twice. Remove it.
				if len(energies) != newmol.numFrames():
					newmol.delFrame(first=newmol.numFrames()-1, last=newmol.numFrames()-1)
					if len(energies) != newmol.numFrames():
						print d
						print len(energies), newmol.numFrames()
						print evaltcl("puts [llength $::QMtool::scfenergies]")
						print evaltcl("puts $::QMtool::scfenergies")
						newmol.save("test.js")
						exit()

				#We only want the last frame to be included here. This is the one for the optimized position.
				for f in range(newmol.numFrames() -1, newmol.numFrames()):
					# if energies[f] > 0:
					# 	continue
					mergemol.dupFrame()
					mergesel.frame = mergemol.numFrames() - 1
					watersel.frame = mergemol.numFrames() - 1
					sel.frame = f
					for j, coordinate in enumerate(['x', 'y', 'z']):
						mergesel.set(coordinate, sel.get(coordinate))
					R = vnp.timestep(mergemol, mergemol.numFrames() -1)
					#Sometimes, the water geometry is screwy. I should fix that.
					if eidx1 != eidx2:
						tmp = np.copy(R[residueidxs+1+eidx1])
						R[residueidxs+1+eidx1] = np.copy(R[residueidxs+1+eidx2])
						R[residueidxs+1+eidx2] = tmp
					# v1 = wogeom[1] - wogeom[0]
					# v2 = wogeom[2] - wogeom[0]
					# v1 /= np.linalg.norm(v1)
					# v2 /= np.linalg.norm(v2)
					# angle = np.degrees(np.arccos(np.dot(v1,v2)))
					# if np.abs(angle-109.5) > 6 and np.abs(angle-37.75)>0.5:
					# 	print f
					# 	print angle
					# 	print logfile
					# 	exit()
					# print R.shape
					# print atomname
					# print len(asel), len(wosel)
					# print residue, combination, d
					#Move each water inward by 0.2 Angstroms
					vec = R[asel.get("index")[0]] - R[wosel.get("index")[0]] #This vector points *FROM* the water *TO* the interacting atom
					# if np.linalg.norm(vec) > 10:
					# 	#This only usually happens when the substrate is charged. We need to toss the data, since it will make things complicated.
					# 	mergemol.delFrame(first=mergemol.numFrames()-1, last=mergemol.numFrames()-1)
					# else:
					if np.linalg.norm(vec) > 10:
						mergemol.delFrame(first=mergemol.numFrames() - 1, last=mergemol.numFrames() - 1)
						print "I continued!"
						continue
					vec *= 0.2 / np.linalg.norm(vec)
					watermovevectors.append(vec)
					# print f
					# print vec
					watersel.moveby(vec)
					
					qmintE.append(energies[f])
				newmol.delete()
			if len(qmintE) == 0:
				print "Hey, this thing has nothing! I wonder how that happened?"
				print psfname
			if np.round(np.sum(allsel.get("charge"))) == 0:
				sf = 1.16
			else:
				sf = 1
			qmintE = sf * np.hstack(qmintE) #These energies need to be scaled.
			print mergemol.numFrames()
			print len(qmintE)
			#exit()
			np.save("targetdata/%d-interactionenergytrajectory.npy" % counter, qmintE)
			mergemol.save("geom/%d-waterinteractiontrajectory.js" % counter)
			ljenergies = np.empty((mergemol.numFrames(), 7), dtype=np.float)
			etermfactors = np.empty((mergemol.numFrames(), mergemol.numAtoms()-3, 7), dtype=np.float)
			charges = np.array(atomsel("all", molid=mergemol).get("charge"), dtype=np.float)
			types = atomsel("all", molid=mergemol).get("type")
			#print mergemol.numFrames()
			#print mol.numFrames()
			for f in range(mergemol.numFrames()):
				R = vnp.timestep(mergemol, f)
				ljenergies[f] = LJterm(R, types, watermovevectors[f])
				etermfactors[f] = Etermprefactors(R, charges, watermovevectors[f])
			#print ljenergies
			#print etermfactors
			np.save("targetdata/%d-ljenergiesdist.npy" % counter, ljenergies)
			np.save("targetdata/%d-etermprefactorsdist.npy" % counter, etermfactors)
			mergemol.delete()
			# print ljenergies
			# comparisondata = np.load("targetdata/%d-ljenergies.npy" % counter)
			# print ljenergies[:,3] - comparisondata
			# comparisondata = np.load("targetdata/%d-etermprefactors.npy" % counter)
			# print etermfactors[:,:,3] - comparisondata
			# exit()
		mol.delete()
		counter += 1
print "Exited the loop"
print allsubgraphs
#exit()
uniquegraphs = dict()
nm = iso.categorical_node_match('t', 'unk')
for k in sorted(bigatomgraphs.keys()):
	g = bigatomgraphs[k]
	foundmatch = False
	for h in uniquegraphs.values():
		foundmatch |= nx.is_isomorphic(g, h, node_match=nm)
	if not foundmatch:
		uniquegraphs[k] = g
		print k, " is unique!"
pickle.dump(uniquegraphs, open("uniquegroups.pkl", "wb"))
graph = []
nm = iso.categorical_node_match('t', 'unk')
for i, key1 in enumerate(sorted(atomgraphs.keys())):
	for j, key2 in enumerate(sorted(atomgraphs.keys())):
		if j > i and nx.is_isomorphic(atomgraphs[key1], atomgraphs[key2], node_match=nm):
			graph.append(np.array([key1[0], key1[1], key2[0], key2[1]], dtype=np.int))
np.save("graphmatch.npy", np.vstack(graph))
if len(dipolemoments):
	np.save("targetdata/dipoles.npy", np.vstack(dipolemoments))
	np.save("targetdata/qoffsets.npy", np.array(qoffsets))
errlog.close()
exit()