import numpy as np
import matplotlib
matplotlib.use('Cairo')
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}",r"\renewcommand{\seriesdefault}{\bfdefault}",
r"\usepackage{amsfonts}",r"\usepackage{textcomp}",r"\usepackage{gensymb}", r'\boldmath'] #Using \boldmath makes the axis label numbers bold as well! Looks good.
params = {'text.usetex' : True,
          'font.size' : 14,
          'font.family' : 'lmodern',
          'text.latex.unicode': True,
          'figure.figsize' : (5.6, 4.2), #8, 6 originally. Make it ~30% smaller so the text is ~30% bigger. (Multiply by .7)
          'figure.dpi' : 120
          }
matplotlib.rcParams.update(params)
import matplotlib.pyplot as plt
from matplotlib.ticker import NullFormatter
import matplotlib.ticker as ticker
from scipy import stats
import statsmodels.api as sm
from scipy import stats
from scipy.optimize import curve_fit
nullfmt   = NullFormatter()


#fig, axes = plt.subplots(2,2, figsize=(8,8))
# left, width = 0.125, 0.57
# bottom, height = 0.135, 0.54
# bottom_h = left_h = left+width+0.02
# rect_scatter = [left, bottom, width, height]
# rect_histx = [left, bottom_h, width, 0.25]
# rect_histy = [left_h, bottom, 0.25, height]


# fig = plt.figure(1)
# ax = plt.axes(rect_scatter)
# hxax = plt.axes(rect_histx, sharex=ax)
# hyax = plt.axes(rect_histy, sharey=ax)

def func(x, m, b):
	return m * x + b
fout = open("results/fitstatistics.tex", "w")
fout.write('''
\\begin{tabular}{lrrrrr}
\\toprule
 & \multicolumn{3}{c}{Small E (E$^{int}<0$\,kcal/mol)} & \multicolumn{2}{c}{Larger E (E$^{int}<20$\,kcal/mol)} \\\\
\cmidrule(rl){2-4}  \cmidrule(rl){5-6}
Charge Scheme & $\left< \left( E^{int}_{QM} - E^{int}_{MM} \\right)^2 \\right>^{\\frac{1}{2}}$  & $\left< E^{int}_{QM} - E^{int}_{MM} \\right>$  & $R^2$ & $\left< \left( E^{int}_{QM} - E^{int}_{MM} \\right)^2 \\right>^{\\frac{1}{2}}$  & $R^2$ \\\\
\midrule
''')
fig, axes = plt.subplots(1,2, figsize=(12,7))
ax = axes[0]
ax.plot([-12,5],[-12,5], color='k', zorder=1)
ax.plot([-12,5],[-13,4], '--', color='k', zorder=1, alpha=0.5)
ax.plot([-12,5],[-11,6], '--', color='k', zorder=1, alpha=0.5)
ax.fill_between([-12,5], [-13,4], [-11,6], color='k', alpha=0.5)
ax.fill_between([-12,5], [-14,3], [-13,4], color='k', alpha=0.25)
ax.fill_between([-12,5], [-11,6], [-10,7], color='k', alpha=0.25)
ax.plot([-12,5],[-14,3], '--', color='k', zorder=1, alpha=0.25)
ax.plot([-12,5],[-10,7], '--', color='k', zorder=1, alpha=0.25)
datasets = []
labels = []
for i in range(1,4):
	data = np.load("../chargecgo/targetdata/waterenergycomparison-%d.npy" % i).astype(np.float)
	labels.append("%d-Neighbor" % i)
	datasets.append(data)
datasets.append(np.load("targetdata/waterenergycomparison-1.npy").astype(np.float))
print datasets[0][:,1]-datasets[3][:,1]
print datasets[0][:,1]-datasets[1][:,1]
#exit()
labels.append("Group")
fig2, ax2 = plt.subplots(1,1)
ax2.plot([-12,5],[-12,5], color='k', zorder=1)
ax2.plot([-12,5],[-13,4], '--', color='k', zorder=1, alpha=0.5)
ax2.plot([-12,5],[-11,6], '--', color='k', zorder=1, alpha=0.5)
ax2.fill_between([-12,5], [-13,4], [-11,6], color='k', alpha=0.5)
ax2.fill_between([-12,5], [-14,3], [-13,4], color='k', alpha=0.25)
ax2.fill_between([-12,5], [-11,6], [-10,7], color='k', alpha=0.25)
ax2.plot([-12,5],[-14,3], '--', color='k', zorder=1, alpha=0.25)
ax2.plot([-12,5],[-10,7], '--', color='k', zorder=1, alpha=0.25)
colors = ['red', 'skyblue', 'yellowgreen', 'violet']
histax = axes[1]
for i in range(4):
	
	data = datasets[i]
	#print data.shape
	#print np.amax(data, axis=0)
	x = data[:,0]
	lj = data[:,1]
	#idxs = np.nonzero((x<0)*(lj<0))[0]
	#idxs2 = np.nonzero((x<5)*(lj<0))[0]
	
	yo = data[:,2]
	yf = data[:,3]
	w = data[:,4]
	idxs = np.nonzero((lj<1)*(x<0))[0]
	idxs2 = np.nonzero((lj<1)*(x<20))[0]
	altidxs = np.nonzero((lj<5)*(x<20))[0]
	#print x.shape
	#print w
	#exit()
	if i == 0:
		mod_wls = sm.WLS(lj+yo, x, weights=w)
		res_wls = mod_wls.fit()
		#print res_wls.summary()
		#print res_wls.rsquared, res_wls.mse_resid, res_wls.params, res_wls.bse
		slope, intercept, r_value, p_value, std_err = stats.linregress(x[idxs],(lj+yo)[idxs])
		slope, intercept, r2, p_value, std_err = stats.linregress(x[idxs2],(lj+yo)[idxs2])
		fout.write("%s & %.2f kcal/mol & %.2f kcal/mol & %.2f & %.2f kcal/mol & %.2f \\\\\n" % ("CGenFF", np.sqrt(np.mean(np.square(x[idxs]-(lj+yo)[idxs]))), np.mean(x[idxs]-(lj+yo)[idxs]), r_value * r_value, np.sqrt(np.mean(np.square(x[idxs2]-(lj+yo)[idxs2]))), r2 * r2))
		ax.scatter(x[idxs2], (lj+yo)[idxs2], color='k', s=1, zorder=2, label='CGenFF')
		ax2.scatter(x[altidxs], (lj+yo)[altidxs], color='k', s=1, zorder=2, label='CGenFF')
		print np.mean(-x[idxs2]+(lj+yo)[idxs2])
		#print func(np.array([-10,10]), res_wls.params[0], res_wls.bse[0])
		#ax.plot([-10,10], func(np.array([-10,10]), res_wls.params[0], res_wls.bse[0]), color='k', linestyle='--')
		histax.hist(x[idxs2]-(lj+yo)[idxs2], bins=1000, range=(-3,3), color='k', label='CGenFF', cumulative=True, density=True, histtype='step')
	mod_wls2 = sm.WLS(lj+yf, x, weights=w)
	res_wls2 = mod_wls2.fit()
	#print res_wls2.rsquared, res_wls2.mse_resid, res_wls2.params, res_wls2.bse
	slope, intercept, r_value, p_value, std_err = stats.linregress(x[idxs],(lj+yf)[idxs])
	slope, intercept, r2, p_value, std_err = stats.linregress(x[idxs2],(lj+yf)[idxs2])
	fout.write("%s & %.2f kcal/mol & %.2f kcal/mol & %.2f  & %.2f kcal/mol & %.2f \\\\\n" % (labels[i], np.sqrt(np.mean(np.square(x[idxs]-(lj+yf)[idxs]))), np.mean(x[idxs]-(lj+yf)[idxs]), r_value * r_value, np.sqrt(np.mean(np.square(x[idxs2]-(lj+yf)[idxs2]))), r2 * r2))
	ax.scatter(x[idxs2], (lj+yf)[idxs2], color=colors[i], s=1, zorder=2, label=labels[i])
	ax2.scatter(x[altidxs], (lj+yf)[altidxs], color=colors[i], s=1, zorder=2, label=labels[i])
	histax.hist(x[idxs2]-(lj+yf)[idxs2], bins=1000, range=(-3,3), color=colors[i], label=labels[i], cumulative=True, density=True, histtype='step')
	print np.mean(-x[idxs2]+(lj+yf)[idxs2])
histax.set_xlim([-3,3])
histax.set_ylim([0,1])
histax.fill_between([-0.5,0.5],[0,0],[1,1], color='k', alpha=0.2)
ax.set_aspect('equal')
ax.set_xlim([-10.5,5])
ax.set_ylim([-12,7])
ax.set_yticks(np.arange(-12,6.5,2))
ax.set_xticks(np.arange(-10,4.5,2))
ax2.set_aspect('equal')
ax2.set_xlim([-10.5,5])
ax2.set_ylim([-12,7])
ax2.set_yticks(np.arange(-12,6.5,2))
ax2.set_xticks(np.arange(-10,4.5,2))
histax.set_yticks(np.arange(0,1.01, 0.1))
#ax.grid(True)
ax.set_xlabel("QM Interaction Energy (kcal/mol)")
ax.set_ylabel("MM Interaction Energy (kcal/mol)")
ax2.set_xlabel("QM Interaction Energy (kcal/mol)")
ax2.set_ylabel("MM Interaction Energy (kcal/mol)")
histax.set_xlabel("Interaction Energy Residual (kcal/mol)")
histax.set_ylabel("Cumulative Probability")
histax.grid(True)
leg=histax.legend(loc=2,fontsize=12, frameon=False)
tcolors = ['k', 'r', 'skyblue', 'yellowgreen', 'violet']
for i, text in enumerate(leg.get_texts()):
    text.set_color(tcolors[i])
fout.write('''
\\bottomrule
\end{tabular}
''')
ax.text(0.03, 0.93, "A", transform=ax.transAxes, size=18)
histax.text(-0.13, 0.93, "B", transform=histax.transAxes, size=18)
fout.close()
fig2.savefig("results/allscatter.png", bbox_inches='tight', dpi=300)
fig2.savefig("results/allscatter.pdf", bbox_inches='tight')
fig.savefig("results/scatter.png", bbox_inches='tight', dpi=300)
fig.savefig("results/scatter.pdf", bbox_inches='tight')
fig.savefig("results/scatter.svg", bbox_inches='tight')

fig, ax = plt.subplots(1,1)
ljs = np.sort(lj)
x = ljs
y = np.arange(len(ljs), dtype=np.float)/len(ljs)
ax.plot(x,y)
ax.set_xlim(-2.5,5)
ax.set_ylim(0,1)
nearest5 = np.argmin(np.abs(x-5))
ax.annotate("%.2f" % y[nearest5], xy=(5,y[nearest5]), horizontalalignment='right', verticalalignment='bottom')

ax.set_xlabel("LJ Interaction Energy (kcal/mol)")
ax.set_ylabel("Cumulative Probability")
ax.grid(True)
fig.tight_layout()
fig.savefig("results/ljdistribution.png", dpi=300)
fig.savefig("results/ljdistribution.pdf")
exit()
fig, ax = plt.subplots(1,1)
ax.plot([-10,1e3],[-10,1e3], color='k', zorder=1)
ax.set_xscale("symlog", linthreshx=10)
ax.set_yscale("symlog", linthreshy=10)
ax.set_xlim(-10.00001,1e3)
ax.set_ylim(-10.00001,1e5)
for i in range(1,4):
	data = np.load("../chargecgo/targetdata/waterenergycomparison-%d.npy" % i).astype(np.float)
	print data.shape
	print np.amax(data, axis=0)
	x = data[:,0]
	lj = data[:,1]
	yo = data[:,2]
	yf = data[:,3]
	w = data[:,4]
	print x.shape
	print w
	if i == 1:
		mod_wls = sm.WLS(lj+yo, x, weights=w)
		res_wls = mod_wls.fit()
		print res_wls.summary()
		print res_wls.rsquared, res_wls.mse_resid, res_wls.params, res_wls.bse
		ax.scatter(x, lj+yo, color='k', s=1, zorder=2, label='CGenFF')
		print func(np.array([-10,10]), res_wls.params[0], res_wls.bse[0])
		#ax.plot([-10,10], func(np.array([-10,10]), res_wls.params[0], res_wls.bse[0]), color='k', linestyle='--')
	mod_wls2 = sm.WLS(lj+yf, x, weights=w)
	res_wls2 = mod_wls2.fit()
	print res_wls2.rsquared, res_wls2.mse_resid, res_wls2.params, res_wls2.bse
	ax.scatter(x, lj+yf, color=colors[i-1], s=1, zorder=2, label="%d-bond fitting" % i)
data = np.load("targetdata/waterenergycomparison-1.npy").astype(np.float)
print data.shape
print np.amax(data, axis=0)
x = data[:,0]
lj = data[:,1]
yo = data[:,2]
yf = data[:,3]
w = data[:,4]
mod_wls2 = sm.WLS(lj+yf, x, weights=w)
res_wls2 = mod_wls2.fit()
print res_wls2.rsquared, res_wls2.mse_resid, res_wls2.params, res_wls2.bse
ax.scatter(x, lj+yf, color='violet', s=1, zorder=2, label="Group Bond")
#ax.set_aspect('equal')
ax.set_xlabel("QM Interaction Energy (kcal/mol)")
ax.set_ylabel("MM Interaction Energy (kcal/mol)")
leg=ax.legend(loc=2,fontsize=12, frameon=False)
for i, text in enumerate(leg.get_texts()):
    text.set_color(tcolors[i])

fout.close()
fig.savefig("results/scatterlog.png", bbox_inches='tight', dpi=300)