import numpy as np
import matplotlib
matplotlib.use('Cairo')
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}",r"\renewcommand{\seriesdefault}{\bfdefault}",
r"\usepackage{amsfonts}",r"\usepackage{textcomp}",r"\usepackage{gensymb}", r'\boldmath'] #Using \boldmath makes the axis label numbers bold as well! Looks good.
params = {'text.usetex' : True,
          'font.size' : 14,
          'font.family' : 'lmodern',
          'text.latex.unicode': True,
          'figure.figsize' : (5.6, 4.2), #8, 6 originally. Make it ~30% smaller so the text is ~30% bigger. (Multiply by .7)
          'figure.dpi' : 120
          }
matplotlib.rcParams.update(params)
import matplotlib.pyplot as plt

labels = ['CGenFF']
midfixes = ['noforce', '', 'strongforce', 'reallystrongforce']
prefixes = ['tightbounds', 'tightbounds', 'tightbounds', 'loosebounds']
postfix = ['','fixdelta', 'fixdeltaupper2', '']
for i in range(4):
     for j in range(4):
          labels.append(prefixes[j] + midfixes[i] + postfix[j])
fig, ax = plt.subplots(1,1)

for i in range(17):
	data = np.load("%d.npy" % i)
	print np.amax(data), np.mean(data), np.median(data)

	H, edges = np.histogram(data, bins=100, range=[0,100])
	H = H.astype(float) / np.sum(H)
	ax.plot(0.5*(edges[1:]+edges[:-1]), np.cumsum(H), label=labels[i])
ax.set_xlabel("Force Magnitude (kcal\,mol$^{-1}$\,\AA$^{-1}$)")
ax.set_ylabel("Cumulative Probability")
ax.legend(fontsize=10)
fig.tight_layout()
fig.savefig("forcedistribution.png", dpi=300)

fig, axes = plt.subplots(1,4, sharey=True, figsize=(12,4))
fig.subplots_adjust(wspace=0,hspace=0, left=0.08, top=0.92, right=0.98, bottom=0.15)
f = np.array([0, 0.02, 0.1, 0.5])
colors = ['red', 'goldenrod', 'green', 'skyblue']
midfixes = ['noforce', '', 'strongforce', 'reallystrongforce']
prefixes = ['tightbounds', 'tightbounds', 'tightbounds', 'loosebounds']
postfix = ['', 'fixdelta', 'fixdeltaupper2', '']
labels = ['Dihedral Set A', 'Dihedral Set B', 'Dihedral Set C', 'Free Dihedral']
for i in range(4):
	ax = axes[i]
	data = np.load("0.npy")
	ax.text(0.5, 1.01, labels[i], transform=ax.transAxes, ha='center')
	H, edges = np.histogram(data, bins=100, range=[0,100])
	H = H.astype(float) / np.sum(H)
	ax.plot(0.5*(edges[1:]+edges[:-1]), np.cumsum(H), label='CGenFF', color='k')
	for j in range(4):
		data = np.load("%d.npy" % (1+i+4*j))
		H, edges = np.histogram(data, bins=100, range=[0,100])
		H = H.astype(float) / np.sum(H)
		vlabel = "v=%.1f" % f[j]
		if j == 1:
			vlabel = "v=%.2f" % f[j]
		ax.plot(0.5*(edges[1:]+edges[:-1]), np.cumsum(H), label=vlabel, color=colors[j])
	ax.set_ylim(0, 1)
	ax.set_xlim(0,100)
	if i != 0:
		ax.get_xticklabels()[0].set_visible(False)
axes[3].legend(fontsize=10)
#axes[0].get_yaxis().set_ticklabels([])
fig.text(0.5, 0.03, "Force Magnitude (kcal\,mol$^{-1}$\,\AA$^{-1}$)", transform=fig.transFigure, ha='center')
fig.text(0.02, 0.5, "Cumulative Probability", transform=fig.transFigure, rotation='vertical', va='center')
fig.savefig("forcedistributioncompare.png", dpi=300)
fig.savefig("forcedistributioncompare.pdf")
